@@ -66,6 +66,12 @@ class WithSliceSelection:
     @property
     def slice_index(self):
         # index in state.slices corresponding to the slice axis
+        for layer in self.layers:
+            data_obj = layer.layer.data
+            if 'spectral_axis_index' in data_obj.meta:
+                return data_obj.meta['spectral_axis_index']
+
+        # Default to 2 if spectral axis index not found
         return 2
 
     @property
@@ -110,7 +116,7 @@ class WithSliceSelection:
                     # rampviz uses coordinate components:
                     world_comp_ids = layer.layer.data.coordinate_components
 
-                if self.slice_index >= len(world_comp_ids):
+                if len(world_comp_ids) < 3:
                     # Case where 2D image is loaded in image viewer
                     continue
 
@@ -153,7 +159,10 @@ class WithSliceSelection:
 
     @property
     def slice_value(self):
-        return self.slice_values[self.slice]
+        if len(self.slice_values):
+            return self.slice_values[self.slice]
+        else:
+            return None
 
     @slice_value.setter
     def slice_value(self, slice_value):
