@@ -1,5 +1,4 @@
 import warnings
-from pathlib import Path
 
 import numpy as np
 import pytest
@@ -11,6 +10,7 @@ from numpy.testing import assert_allclose
 from specutils import SpectralRegion
 
 from jdaviz.core.custom_units_and_equivs import PIX2, SPEC_PHOTON_FLUX_DENSITY_UNITS
+from jdaviz.utils import cached_uri
 
 
 @pytest.mark.parametrize("cube_type", ["Surface Brightness", "Flux"])
@@ -30,18 +30,19 @@ def test_user_api(cubeviz_helper, spectrum1d_cube, spectrum1d_cube_sb_unit, cube
         cubeviz_helper.load_data(cube, data_label='test')
 
     mm = cubeviz_helper.plugins['Moment Maps']
-    assert not mm._obj.continuum_marks['center'].visible
+    cmc = mm._obj.continuum_marks['center']
+    assert len(cmc.marks) == 0
     with mm.as_active():
-        assert mm._obj.continuum_marks['center'].visible
         mm.n_moment = 0
         # no continuum so marks should be empty
-        assert len(mm._obj.continuum_marks['center'].x) == 0
+        assert len(cmc.marks) == 0
 
         mom = mm.calculate_moment()
 
         mm.continuum = 'Surrounding'
         mm.continuum_width = 10
-        assert len(mm._obj.continuum_marks['center'].x) > 0
+        cm = cmc.marks_list[0]
+        assert len(cm.x) > 0
 
         mom_sub = mm.calculate_moment()
         assert isinstance(mom_sub.wcs, WCS)
@@ -59,10 +60,13 @@ def test_user_api(cubeviz_helper, spectrum1d_cube, spectrum1d_cube_sb_unit, cube
         with pytest.raises(ValueError, match="units must be in"):
             mm.calculate_moment()
 
+        mm.continuum = 'None'
+        assert len(cm.x) == 0
+
 
 @pytest.mark.parametrize("cube_type", ["Surface Brightness", "Flux"])
 def test_moment_calculation(cubeviz_helper, spectrum1d_cube,
-                            spectrum1d_cube_sb_unit, cube_type, tmp_path):
+                            spectrum1d_cube_sb_unit, cube_type):
 
     moment_unit = "Jy m"
     moment_value_str = "+6.40166e-10"
@@ -109,17 +113,17 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube,
     assert len(mv_data) == 1
     assert mv_data[0].label == 'moment 0'
 
-    assert len(dc.links) == 19
+    assert len(dc.links) == 17
 
     # label should remain unchanged but raise overwrite warnings
     assert mm._obj.results_label == 'moment 0'
     assert mm._obj.results_label_overwrite is True
 
     # Make sure coordinate display works in flux viewer (loaded data, not the moment map)
-    label_mouseover = cubeviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = cubeviz_helper._coords_info
     label_mouseover._viewer_mouse_event(flux_viewer, {'event': 'mousemove',
                                                       'domain': {'x': 0, 'y': 0}})
-    assert flux_viewer.state.slices == (0, 0, 1)
+    assert flux_viewer.state.slices == (1, 0, 0)
     # Slice 0 has 8 pixels, this is Slice 1
     assert label_mouseover.as_text() == (f"Pixel x=00.0 y=00.0 Value +8.00000e+00 {cube_unit}",
                                          "World 13h39m59.9731s +27d00m00.3600s (ICRS)",
@@ -151,14 +155,13 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube,
     mm._obj.vue_calculate_moment()
 
     assert dc[-1].label == 'moment 1'
-
-    assert len(dc.links) == 27
-    assert len(dc.external_links) == 6  # pixel linked
+    assert len(dc.links) == 25
+    assert len(dc.external_links) == 5  # pixel linked
     # Link 3D z to 2D x and 3D y to 2D y
-    assert (dc.external_links[2].cids1[0].label == "Pixel Axis 0 [z]" and
-            dc.external_links[2].cids2[0].label == "Pixel Axis 1 [x]" and
-            dc.external_links[3].cids1[0].label == "Pixel Axis 1 [y]" and
-            dc.external_links[3].cids2[0].label == "Pixel Axis 0 [y]")
+    assert (dc.external_links[3].cids1[0].label == "Pixel Axis 1 [y]" and
+            dc.external_links[3].cids2[0].label == "Pixel Axis 0 [y]" and
+            dc.external_links[4].cids1[0].label == "Pixel Axis 2 [x]" and
+            dc.external_links[4].cids2[0].label == "Pixel Axis 1 [x]")
 
     # Coordinate display should be unaffected.
     label_mouseover._viewer_mouse_event(flux_viewer, {'event': 'mousemove',
@@ -196,7 +199,7 @@ def test_moment_velocity_calculation(cubeviz_helper, spectrum1d_cube):
     mm.calculate_moment()
 
     # Make sure coordinate display works
-    label_mouseover = cubeviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = cubeviz_helper._coords_info
     label_mouseover._viewer_mouse_event(uncert_viewer, {'event': 'mousemove',
                                                         'domain': {'x': 0, 'y': 0}})
     assert label_mouseover.as_text() == ("Pixel x=00.0 y=00.0 Value -4.14668e+02 km / s",
@@ -272,13 +275,12 @@ def test_write_momentmap(cubeviz_helper, spectrum1d_cube, tmp_path):
 
 
 @pytest.mark.remote_data
-def test_momentmap_nirspec_prism(cubeviz_helper, tmp_path):
+def test_momentmap_nirspec_prism(cubeviz_helper):
     uri = "mast:jwst/product/jw02732-o003_t002_nirspec_prism-clear_s3d.fits"
-    local_path = str(tmp_path / Path(uri).name)
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
-        cubeviz_helper.load_data(uri, cache=True, local_path=local_path)
+        cubeviz_helper.load_data(cached_uri(uri), cache=True)
     uc = cubeviz_helper.plugins["Unit Conversion"]
     uc.open_in_tray()  # plugin has to be open for unit change to take hold
     uc._obj.show_translator = True
@@ -289,10 +291,8 @@ def test_momentmap_nirspec_prism(cubeviz_helper, tmp_path):
     mm.calculate_moment()
     assert isinstance(mm.moment.wcs, WCS)
 
-    # Because cube axes order is re-arranged by specutils on load, this gets confusing.
-    # There is a swapaxes within Moment Map WCS calculation.
     sky_moment = mm.moment.wcs.pixel_to_world(50, 30)
-    sky_cube = cubeviz_helper.app.data_collection[0].meta["_orig_spec"].wcs.celestial.pixel_to_world(30, 50)  # noqa: E501
+    sky_cube = cubeviz_helper.app.data_collection[0].coords.pixel_to_world(50, 30, 0)[0]  # noqa: E501
     assert_allclose((sky_moment.ra.deg, sky_moment.dec.deg),
                     (sky_cube.ra.deg, sky_cube.dec.deg))
 
@@ -390,7 +390,7 @@ def test_moment_zero_unit_flux_conversions(cubeviz_helper,
 
     # and flux viewer for mouseover info
     flux_viewer = cubeviz_helper.app.get_viewer(cubeviz_helper._default_flux_viewer_reference_name)
-    label_mouseover = cubeviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = cubeviz_helper._coords_info
 
     # convert to new flux unit
     uc.flux_unit.selected = new_flux_unit_str
