@@ -12,19 +12,20 @@ from glue.core.roi import CircularROI, RectangularROI
 from numpy.testing import assert_allclose, assert_array_equal
 from regions import (CirclePixelRegion, CircleAnnulusPixelRegion, EllipsePixelRegion,
                      RectanglePixelRegion, PixCoord)
-from specutils import Spectrum1D
+from specutils import Spectrum
 from specutils.manipulation import FluxConservingResampler
 
 from jdaviz.core.custom_units_and_equivs import PIX2, SPEC_PHOTON_FLUX_DENSITY_UNITS
 from jdaviz.core.unit_conversion_utils import (all_flux_unit_conversion_equivs,
                                                flux_conversion_general)
+from jdaviz.utils import cached_uri
 
 calspec_url = "https://archive.stsci.edu/hlsps/reference-atlases/cdbs/current_calspec/"
 
 
 def test_version_after_nddata_update(cubeviz_helper, spectrum1d_cube_with_uncerts):
     # Also test that plugin is disabled before data is loaded.
-    plg = cubeviz_helper.plugins['Spectral Extraction']
+    plg = cubeviz_helper.plugins['3D Spectral Extraction']
     assert plg._obj.disabled_msg != ''
 
     cubeviz_helper.load_data(spectrum1d_cube_with_uncerts)
@@ -34,8 +35,8 @@ def test_version_after_nddata_update(cubeviz_helper, spectrum1d_cube_with_uncert
     spectral_cube.uncertainty = uncert_cube
 
     # Collapse the spectral cube using the astropy.nddata machinery.
-    # Axes 0, 1 are the spatial ones.
-    collapsed_cube_nddata = spectral_cube.sum(axis=(0, 1))  # return NDDataArray
+    # Axes 1, 2 are the spatial ones.
+    collapsed_cube_nddata = spectral_cube.sum(axis=(1, 2))  # return NDDataArray
 
     # when loaded into app, cubes loaded in flux are converted to per-pixel-squared
     # surface brightness, so multiply by pix**2 to compare to NDData, if input
@@ -43,11 +44,11 @@ def test_version_after_nddata_update(cubeviz_helper, spectrum1d_cube_with_uncert
     collapsed_cube_nddata = collapsed_cube_nddata * (u.pix ** 2)
 
     # Collapse the spectral cube using the methods in jdaviz:
-    collapsed_cube_s1d = plg.extract(add_data=False)  # returns Spectrum1D
+    collapsed_cube_s1d = plg.extract(add_data=False)  # returns Spectrum
 
     assert plg._obj.disabled_msg == ''
     assert isinstance(spectral_cube, NDDataArray)
-    assert isinstance(collapsed_cube_s1d, Spectrum1D)
+    assert isinstance(collapsed_cube_s1d, Spectrum)
 
     assert_allclose(
         collapsed_cube_nddata.data,
@@ -56,16 +57,15 @@ def test_version_after_nddata_update(cubeviz_helper, spectrum1d_cube_with_uncert
 
 
 def test_gauss_smooth_before_spec_extract(cubeviz_helper, spectrum1d_cube_with_uncerts):
-    # Also test if gaussian smooth plugin is run before spec extract
+    # Test if gaussian smooth plugin is run before spec extract
     # that spec extract yields results of correct cube data
-    gs_plugin = cubeviz_helper.plugins['Gaussian Smooth']._obj
-
-    # give uniform unit uncertainties for spec extract test:
+    # give uniform unit uncertainties for spec extract test
     spectrum1d_cube_with_uncerts.uncertainty = StdDevUncertainty(
         np.ones_like(spectrum1d_cube_with_uncerts.data)
     )
 
     cubeviz_helper.load_data(spectrum1d_cube_with_uncerts)
+    gs_plugin = cubeviz_helper.plugins['Gaussian Smooth']._obj
 
     gs_plugin.dataset_selected = f'{cubeviz_helper.app.data_collection[0].label}'
     gs_plugin.mode_selected = 'Spatial'
@@ -76,7 +76,7 @@ def test_gauss_smooth_before_spec_extract(cubeviz_helper, spectrum1d_cube_with_u
             match='The following attributes were set on the data object, but will be ignored'):
         gs_plugin.vue_apply()
 
-    gs_data_label = cubeviz_helper.app.data_collection[2].label
+    gs_data_label = cubeviz_helper.app.data_collection[3].label
     cubeviz_helper.app.add_data_to_viewer('flux-viewer', gs_data_label)
 
     # create a subset with a single pixel:
@@ -88,7 +88,7 @@ def test_gauss_smooth_before_spec_extract(cubeviz_helper, spectrum1d_cube_with_u
     ]
     cubeviz_helper.plugins['Subset Tools'].import_region(regions, combination_mode='new')
 
-    extract_plugin = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plugin = cubeviz_helper.plugins['3D Spectral Extraction']
     extract_plugin.function = "Sum"
     expected_uncert = 2
 
@@ -128,7 +128,7 @@ def test_subset(
     cubeviz_helper.load_data(spectrum1d_cube_with_uncerts)
     cubeviz_helper.plugins['Subset Tools'].import_region(regions, combination_mode='new')
 
-    plg = cubeviz_helper.plugins['Spectral Extraction']
+    plg = cubeviz_helper.plugins['3D Spectral Extraction']
     plg.function = function
 
     # single pixel region:
@@ -151,10 +151,10 @@ def test_extracted_file_in_export_plugin(cubeviz_helper, spectrum1d_cube_with_un
 
     cubeviz_helper.load_data(spectrum1d_cube_with_uncerts)
 
-    extract_plugin = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plugin = cubeviz_helper.plugins['3D Spectral Extraction']
 
     # make sure export enabled is true, and that before the collapse function
-    # is run `collapsed_spec_available` is correctly set to False
+    # is run `extraction_available` is correctly set to False
     assert extract_plugin._obj.export_enabled
     assert extract_plugin._obj.extraction_available is False
 
@@ -180,7 +180,7 @@ def test_aperture_markers(cubeviz_helper, spectrum1d_cube):
     cubeviz_helper.plugins['Subset Tools'].import_region(
         [CirclePixelRegion(PixCoord(0.5, 0), radius=1.2)])
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
     slice_plg = cubeviz_helper.plugins['Slice']
 
     mark = extract_plg.aperture.marks[0]
@@ -235,7 +235,7 @@ def test_cone_aperture_with_different_methods(cubeviz_helper, spectrum1d_cube_la
     cubeviz_helper.plugins['Subset Tools'].import_region(
         EllipsePixelRegion(center, width=5, height=5), combination_mode='new')
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = subset
     extract_plg.aperture_method.selected = aperture_method
@@ -267,7 +267,7 @@ def test_cylindrical_aperture_with_different_methods(cubeviz_helper, spectrum1d_
         CirclePixelRegion(center, radius=2.5),
         EllipsePixelRegion(center, width=5, height=5)], combination_mode='new')
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = subset
     extract_plg.aperture_method.selected = aperture_method
@@ -290,7 +290,7 @@ def test_rectangle_aperture_with_exact(cubeviz_helper, spectrum1d_cube_largest):
     cubeviz_helper.plugins['Subset Tools'].import_region(
         RectanglePixelRegion(PixCoord(5, 10), width=4, height=4))
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = "Subset 1"
     extract_plg.aperture_method.selected = "Exact"
@@ -319,7 +319,7 @@ def test_background_subtraction(cubeviz_helper, spectrum1d_cube_largest):
         CirclePixelRegion(PixCoord(5, 10), radius=2.5),
         EllipsePixelRegion(PixCoord(13, 10), width=3, height=5)], combination_mode='new')
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
     with extract_plg.as_active():
         extract_plg.aperture = 'Subset 1'
         spec_no_bg = extract_plg.extract()
@@ -372,7 +372,7 @@ def test_cone_and_cylinder_errors(cubeviz_helper, spectrum1d_cube_largest):
         CirclePixelRegion(center, radius=2.5),
         CircleAnnulusPixelRegion(center, inner_radius=2.5, outer_radius=4)], combination_mode='new')
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = 'Subset 1'
     extract_plg.aperture_method.selected = 'Exact'
@@ -393,12 +393,12 @@ def test_cone_and_cylinder_errors(cubeviz_helper, spectrum1d_cube_largest):
 
 
 def test_cone_aperture_with_frequency_units(cubeviz_helper, spectral_cube_wcs):
-    data = Spectrum1D(flux=np.ones((128, 129, 256)) * u.nJy, wcs=spectral_cube_wcs)
+    data = Spectrum(flux=np.ones((128, 129, 256)) * u.nJy, wcs=spectral_cube_wcs)
     cubeviz_helper.load_data(data, data_label="Test Flux")
     cubeviz_helper.plugins['Subset Tools'].import_region(
         [CirclePixelRegion(PixCoord(14, 15), radius=2.5)])
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = 'Subset 1'
     extract_plg.aperture_method.selected = 'Exact'
@@ -412,7 +412,7 @@ def test_cone_aperture_with_frequency_units(cubeviz_helper, spectral_cube_wcs):
 def test_cube_extraction_with_nan(cubeviz_helper, image_cube_hdu_obj):
     image_cube_hdu_obj[1].data[:, :2, :2] = np.nan
     cubeviz_helper.load_data(image_cube_hdu_obj, data_label="with_nan")
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
     sp = extract_plg.extract()  # Default settings (sum)
     assert_allclose(sp.flux.value, 9.6E-16)  # (10 x 10) - 4
 
@@ -428,7 +428,7 @@ def test_autoupdate_results(cubeviz_helper, spectrum1d_cube_largest):
     cubeviz_helper.plugins['Subset Tools'].import_region(
         CircularROI(xc=5, yc=5, radius=2))
 
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
     extract_plg.aperture = 'Subset 1'
     extract_plg.add_results.label = 'extracted'
     extract_plg.add_results._obj.auto_update_result = True
@@ -453,7 +453,7 @@ def test_autoupdate_results(cubeviz_helper, spectrum1d_cube_largest):
 def test_aperture_composite_detection(cubeviz_helper, spectrum1d_cube):
     cubeviz_helper.load_data(spectrum1d_cube)
     subset_plugin = cubeviz_helper.plugins['Subset Tools']
-    spec_extr_plugin = cubeviz_helper.plugins['Spectral Extraction']._obj
+    spec_extr_plugin = cubeviz_helper.plugins['3D Spectral Extraction']._obj
 
     # create a rectangular subset with all spaxels:
     rectangle = RectangularROI(-0.5, 1.5, -0.5, 3.5)
@@ -466,8 +466,7 @@ def test_aperture_composite_detection(cubeviz_helper, spectrum1d_cube):
     # now remove from this subset a circular region in the center:
     circle = CircularROI(0.5, 1.5, 1)
 
-    subset_plugin.combination_mode = 'andnot'
-    subset_plugin.import_region(circle)
+    subset_plugin.import_region(circle, edit_subset='Subset 1', combination_mode='andnot')
 
     # now the subset is composite:
     assert spec_extr_plugin.aperture.is_composite
@@ -477,13 +476,12 @@ def test_extraction_composite_subset(cubeviz_helper, spectrum1d_cube):
     cubeviz_helper.load_data(spectrum1d_cube)
 
     subset_plugin = cubeviz_helper.plugins['Subset Tools']
-    spec_extr_plugin = cubeviz_helper.plugins['Spectral Extraction']._obj
+    spec_extr_plugin = cubeviz_helper.plugins['3D Spectral Extraction']._obj
 
     lower_aperture = RectangularROI(-0.5, 0.5, -0.5, 1.5)
     upper_aperture = RectangularROI(2.5, 3.5, -0.5, 1.5)
 
     subset_plugin.import_region(lower_aperture)
-    subset_plugin.combination_mode = 'new'
     subset_plugin.import_region(upper_aperture)
 
     spec_extr_plugin.aperture_selected = 'Subset 1'
@@ -494,14 +492,13 @@ def test_extraction_composite_subset(cubeviz_helper, spectrum1d_cube):
 
     rectangle = RectangularROI(-0.5, 3.5, -0.5, 1.5)
 
-    subset_plugin.combination_mode = 'new'
     subset_plugin.import_region(rectangle)
 
     subset_plugin._obj.subset_selected = 'Subset 3'
     circle = CircularROI(1.5, 0.5, 1.1)
 
-    subset_plugin.combination_mode = 'andnot'
-    subset_plugin.import_region(circle)
+    subset_plugin.import_region(circle, edit_subset='Subset 3',
+                                combination_mode='andnot')
 
     spec_extr_plugin.aperture_selected = 'Subset 3'
 
@@ -518,11 +515,20 @@ def test_extraction_composite_subset(cubeviz_helper, spectrum1d_cube):
 def test_spectral_extraction_with_correct_sum_units(cubeviz_helper,
                                                     spectrum1d_cube_fluxunit_jy_per_steradian):
     cubeviz_helper.load_data(spectrum1d_cube_fluxunit_jy_per_steradian)
-    spec_extr_plugin = cubeviz_helper.plugins['Spectral Extraction']._obj
+    spec_extr_plugin = cubeviz_helper.plugins['3D Spectral Extraction']._obj
     collapsed = spec_extr_plugin.extract()
+
+    assert '_pixel_scale_factor' in collapsed.meta
+
+    # Original units in Jy / sr
+    # After collapsing, sr is removed via the scale factor and the extracted spectrum is in Jy
+    expected_flux_values = (np.array([190., 590., 990., 1390., 1790.,
+                                      2190., 2590., 2990., 3390., 3790.]) *
+                            collapsed.meta.get('_pixel_scale_factor'))
+
     np.testing.assert_allclose(
         collapsed.flux.value,
-        [190., 590., 990., 1390., 1790., 2190., 2590., 2990., 3390., 3790.]
+        expected_flux_values
     )
     assert collapsed.flux.unit == u.Jy
     assert collapsed.uncertainty.unit == u.Jy
@@ -561,7 +567,7 @@ def test_spectral_extraction_unit_conv_one_spec(
     assert uc.flux_unit == "Jy"
     uc.flux_unit.selected = "MJy"
     assert spectrum_viewer.state.y_display_unit == "MJy"
-    spec_extr_plugin = cubeviz_helper.plugins['Spectral Extraction']
+    spec_extr_plugin = cubeviz_helper.plugins['3D Spectral Extraction']
     # Overwrite the one and only default extraction.
     collapsed = spec_extr_plugin.extract()
     # Actual values not in display unit but should not affect display unit.
@@ -570,17 +576,16 @@ def test_spectral_extraction_unit_conv_one_spec(
     assert spectrum_viewer.state.y_display_unit == "MJy"
 
 
-@pytest.mark.usefixtures('_jail')
 @pytest.mark.remote_data
 @pytest.mark.parametrize(
     "start_slice, aperture, expected_rtol, uri, calspec_url",
     (
         (5.2, (20.5, 17, 10.9), 0.03,
-         "mast:jwst/product/jw01524-o003_t002_miri_ch1-shortmediumlong_s3d.fits",
+         "mast:jwst/product/jw01524-o003_t002_miri_ch1-medium_s3d.fits",
          calspec_url + "delumi_mod_005.fits"),  # delta UMi
 
         (4.85, (28, 21, 12), 0.03,
-         "mast:jwst/product/jw01050-o003_t005_miri_ch1-shortmediumlong_s3d.fits",
+         "mast:jwst/product/jw01050-o003_t005_miri_ch1-medium_s3d.fits",
          calspec_url + "hd159222_mod_007.fits"),  # HD 159222
     )
 )
@@ -607,11 +612,11 @@ def test_spectral_extraction_scientific_validation(
     evolve with time. For the latest updates on MIRI flux calibration, see:
     https://jwst-docs.stsci.edu/jwst-calibration-status/miri-calibration-status/
     """
-    # Download CALSPEC model spectrum, initialize Spectrum1D.
+    # Download CALSPEC model spectrum, initialize Spectrum.
     calspec_fitsrec = fits.getdata(calspec_url)
     column_units = [u.AA] + 2 * [u.Unit('erg s-1 cm-2 AA-1')]
     spectra_table = QTable(calspec_fitsrec, units=column_units)
-    model_spectrum = Spectrum1D(
+    model_spectrum = Spectrum(
         flux=spectra_table['FLUX'],
         spectral_axis=spectra_table['WAVELENGTH']
     )
@@ -619,7 +624,7 @@ def test_spectral_extraction_scientific_validation(
     # load observations into Cubeviz
     with warnings.catch_warnings():
         warnings.simplefilter('ignore')
-        cubeviz_helper.load_data(uri, cache=True)
+        cubeviz_helper.load_data(cached_uri(uri), cache=True)
 
     # add a subset with an aperture centered on each source
     subset_plugin = cubeviz_helper.plugins['Subset Tools']
@@ -630,7 +635,7 @@ def test_spectral_extraction_scientific_validation(
     slice_plugin.value = start_slice
 
     # run a conical spectral extraction
-    spectral_extraction = cubeviz_helper.plugins['Spectral Extraction']
+    spectral_extraction = cubeviz_helper.plugins['3D Spectral Extraction']
     spectral_extraction.aperture = 'Subset 1'
     spectral_extraction.wavelength_dependent = True
     spectral_extraction._obj.results_label = 'conical-extraction'
@@ -679,7 +684,7 @@ def test_spectral_extraction_flux_unit_conversions(cubeviz_helper,
         cubeviz_helper._default_spectrum_viewer_reference_name)
 
     uc = cubeviz_helper.plugins["Unit Conversion"]
-    se = cubeviz_helper.plugins['Spectral Extraction']
+    se = cubeviz_helper.plugins['3D Spectral Extraction']
     se.keep_active = True  # keep active for access to preview markers
 
     # equivalencies for unit conversion, for comparison of outputs
