@@ -8,8 +8,11 @@ from jdaviz.core.template_mixin import (TemplateMixin, LayerSelect,
 from jdaviz.core.user_api import UserApiWrapper
 from jdaviz.core.events import (IconsUpdatedMessage, AddDataMessage,
                                 ChangeRefDataMessage, ViewerRenamedMessage)
+from glue.core.message import SubsetDeleteMessage
+from jdaviz.core.sonified_layers import SonifiedLayerState, SonifiedDataLayerArtist
 from jdaviz.utils import cmap_samples, is_not_wcs_only
 
+
 from glue.core.edit_subset_mode import (AndMode, AndNotMode, OrMode,
                                         ReplaceMode, XorMode, NewMode)
 from glue.icons import icon_path
@@ -44,6 +47,9 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
     :ref:`public API <plugin-apis>`:
 
     * :meth:`open_menu`
+    * :meth:`data_labels_loaded`
+    * :meth:`data_labels_visible`
+    * :meth:`data_labels_unloaded`
     * ``layer`` (:class:`~jdaviz.core.template_mixin.LayerSelect`):
         actively selected layer(s)
     * ``orientation`` (:class:`~jdaviz.core.template_mixin.LayerSelect`):
@@ -99,8 +105,6 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
 
     subset_edit_modes = List().tag(sync=True)
 
-    dev_data_menu = Bool(False).tag(sync=True)
-
     def __init__(self, viewer, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._viewer = viewer
@@ -118,19 +122,24 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
         # changing the selection has no consequence.
         def data_not_in_viewer(data):
             return data.label not in self.layer.choices
-        self.dataset.filters = ['is_not_wcs_only', 'not_child_layer', data_not_in_viewer]
+
+        self.dataset.filters = ['is_not_wcs_only', 'not_child_layer',
+                                data_not_in_viewer]
 
         self.orientation = LayerSelect(self,
                                        'orientation_layer_items',
                                        'orientation_layer_selected',
                                        'viewer_id',
                                        only_wcs_layers=True)
-        self.orientation_enabled = self.config == 'imviz'
+
+        self.orientation_enabled = (self.config in ('imviz', 'deconfigged')
+                                    and viewer.__class__.__name__ == 'ImvizImageView')
 
         # first attach callback to catch any updates to viewer/layer icons and then
         # set their initial state
         self.hub.subscribe(self, IconsUpdatedMessage, self._on_app_icons_updated)
         self.hub.subscribe(self, AddDataMessage, handler=lambda _: self._set_viewer_id())
+        self.hub.subscribe(self, SubsetDeleteMessage, handler=lambda msg: self._remove_subset_from_layers(msg.subset))  # noqa
         self.hub.subscribe(self, ChangeRefDataMessage, handler=self._on_refdata_change)
         self.hub.subscribe(self, ViewerRenamedMessage, handler=self._on_viewer_renamed_message)
         self.viewer_icons = dict(self.app.state.viewer_icons)
@@ -155,9 +164,10 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
         expose = ['open_menu', 'layer', 'set_layer_visibility', 'toggle_layer_visibility',
                   'create_subset', 'modify_subset', 'add_data', 'view_info',
                   'remove_from_viewer', 'remove_from_app']
-        if self.app.config == 'imviz':
+        readonly = ['data_labels_loaded', 'data_labels_visible', 'data_labels_unloaded']
+        if self.app.config in ('imviz', 'deconfigged'):
             expose += ['orientation']
-        return UserApiWrapper(self, expose=expose)
+        return UserApiWrapper(self, expose=expose, readonly=readonly)
 
     def open_menu(self):
         """
@@ -169,6 +179,36 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
     def existing_subset_labels(self):
         return [sg.label for sg in self.app.data_collection.subset_groups]
 
+    @property
+    def existing_sonified_labels(self):
+        return [sd.layer.label for sd in self._viewer.state.layers
+                if isinstance(sd, SonifiedLayerState)]
+
+    @property
+    def data_labels_loaded(self):
+        return [layer['label'] for layer in self.layer_items
+                if layer['label'] not in self.existing_subset_labels]
+
+    @property
+    def data_labels_visible(self):
+        return [layer['label'] for layer in self.layer_items
+                if layer['label'] not in self.existing_subset_labels and layer['visible']]
+
+    @property
+    def data_labels_unloaded(self):
+        return self.dataset.choices
+
+    def _remove_subset_from_layers(self, msg={}):
+        if not hasattr(self, 'layer'):
+            return
+
+        # on SubsetDeleteMessage, message is sent before self.layer is updated, so update
+        # layer visible to ensure icon is updated in data menu
+        if msg.label not in self.existing_subset_labels:
+            for layer in self._viewer.layers:
+                if layer.layer.label == msg.label:
+                    self.layer._update_items(remove_subset=msg.label)
+
     def _set_viewer_id(self):
         # viewer_ids are not populated on the viewer at init, so we'll keep checking and set
         # these the first time that they are available
@@ -180,6 +220,7 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
             self.layer.viewer = self._viewer.reference
         except AttributeError:
             return
+        self._on_refdata_change()
 
     def _on_app_icons_updated(self, msg):
         if msg.icon_type == 'viewer':
@@ -188,13 +229,18 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
             self.layer_icons = msg.icons
         self._set_viewer_id()
 
-    def _on_refdata_change(self, msg):
-        if msg.viewer_id != self.viewer_id:
+    def _on_refdata_change(self, msg=None):
+        if msg is not None and msg.viewer_id != self.viewer_id:
+            return
+        if getattr(self._viewer.state, 'reference_data', None) is None:
             return
         self.orientation_align_by_wcs = self._viewer.state.reference_data.meta.get('_WCS_ONLY', False)  # noqa
         if self.orientation_align_by_wcs:
             with self.during_select_sync():
-                self.orientation.selected = str(self._viewer.state.reference_data.label)
+                ref_label = self._viewer.state.reference_data.label
+                if ref_label not in self.orientation.choices:
+                    self.orientation._update_items()
+                self.orientation.selected = str(ref_label)
 
     def _on_viewer_renamed_message(self, msg):
         if self.viewer_reference == msg.old_viewer_ref:
@@ -220,13 +266,21 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
             raise
         self._during_select_sync = False
 
-    @observe('dm_layer_selected')
+    @observe('dm_layer_selected', 'layer_multiselect')
     def _dm_layer_selected_changed(self, event={}):
-        if not hasattr(self, 'layer') or not self.layer.multiselect:  # pragma: no cover
+        if not hasattr(self, 'layer'):  # pragma: no cover
             return
         if self._during_select_sync:
             return
-        if len(event.get('new')) == len(event.get('old')):
+        if event.get('name') == 'layer_multiselect' and event.get('new'):
+            return
+        if not self.layer.multiselect and len(self.dm_layer_selected) > 1:
+            # vue will still treat the element as a list, so we will include the
+            # logic here to enforce single-select toggling
+            self.dm_layer_selected = [self.dm_layer_selected[-1]]
+            return
+        if (event.get('name') == 'dm_layer_selected'
+                and len(event.get('new')) == len(event.get('old'))):
             # not possible from UI interaction, but instead caused by a selected
             # layer being removed (deleting a selected subset, etc).  We want
             # to update dm_layer_selected in order to preserve layer.selected
@@ -235,8 +289,15 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
         with self.during_select_sync():
             # map index in dm_layer_selected (inverse order of layer_items)
             # to set self.layer.selected
-            self.layer.selected = [self.layer_items[i]['label']
-                                   for i in self.dm_layer_selected]
+            if len(self.layer_items):
+                selected = [self.layer_items[i]['label']
+                            for i in self.dm_layer_selected]
+                if self.layer.multiselect:
+                    self.layer.selected = selected
+                else:
+                    self.layer.selected = selected[0] if len(selected) else ''
+            else:
+                self.layer.selected = ''
 
     @observe('layer_selected', 'layer_items')
     def _layers_changed(self, event={}):
@@ -246,7 +307,8 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
             with self.during_select_sync():
                 # map list of strings in self.layer.selected to indices in dm_layer_selected
                 layer_labels = [layer['label'] for layer in self.layer_items]
-                self.dm_layer_selected = [layer_labels.index(label) for label in self.layer.selected
+                layer_selected = self.layer_selected if self.layer.multiselect else [self.layer_selected]  # noqa
+                self.dm_layer_selected = [layer_labels.index(label) for label in layer_selected
                                           if label in layer_labels]
 
         subset_labels = self.existing_subset_labels
@@ -278,9 +340,9 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
 
         # layer info rules
         if self.selected_n_layers == 1:
-            if max(self.dm_layer_selected) >= len(self.layer_items):  # pragma: no cover
-                # can happen during state transition but should immediately be followed up
-                # with an update
+            if self.dm_layer_selected == [] or max(self.dm_layer_selected) >= len(self.layer_items):  # noqa pragma: no cover
+                # Can happen during state transition but should immediately be followed up
+                # with an update. Also get here when unloading all data.
                 self.info_enabled = False
                 self.info_tooltip = ''
             elif self.layer_items[self.dm_layer_selected[0]].get('from_plugin', False):
@@ -321,7 +383,14 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
             # forbid deleting non-plugin generated data
             selected_items = self.layer.selected_item
             for i, layer in enumerate(self.layer.selected):
-                if (layer not in self.existing_subset_labels
+                if layer == '':
+                    # In this case we removed everything from the viewer
+                    continue
+                if isinstance(self.layer.selected_obj[0][0], SonifiedDataLayerArtist):
+                    self.delete_app_enabled = False
+                    self.delete_app_tooltip = "Cannot delete sonified data from app"
+                    break
+                elif (layer not in self.existing_subset_labels
                         and selected_items['from_plugin'][i] is None):
                     self.delete_app_enabled = False
                     self.delete_app_tooltip = f"Cannot delete imported data from {self.app.config}"
@@ -343,6 +412,7 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
                 self.subset_edit_tooltip = "Select a subset to edit"
             else:
                 self.subset_edit_tooltip = "Select a single subset to edit"
+        self.layer._update_items()
 
     def set_layer_visibility(self, layer_label, visible=True):
         """
@@ -362,7 +432,10 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
         """
         for layer in self._viewer.layers:
             if layer.layer.label == layer_label:
-                layer.visible = visible
+                if isinstance(layer, SonifiedDataLayerArtist):
+                    layer.audible = visible
+                else:
+                    layer.visible = visible
             elif hasattr(layer.layer, 'data') and layer.layer.data.label == layer_label:
                 layer.visible = layer.layer.label in self.visible_layers
             if not visible and self.app._get_assoc_data_parent(layer.layer.label) == layer_label:
@@ -370,6 +443,11 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
                 # so also hide the child-layer
                 layer.visible = False
 
+            if layer.state.visible != layer.visible:
+                layer.state.visible = layer.visible
+
+            self.layer._update_items()
+
         if visible and (parent_label := self.app._get_assoc_data_parent(layer_label)):
             # ensure the parent layer is also visible
             self.set_layer_visibility(parent_label, visible=True)
@@ -515,14 +593,16 @@ class DataMenu(TemplateMixin, LayerSelectMixin, DatasetSelectMixin):
         Remove the selected layers from the entire app and all viewers.
         """
         # future improvement: allow overriding layer.selected via *args, with pre-validation
-        for layer in self.layer.selected:
+        for layer in (self.layer.selected if isinstance(self.layer.selected,
+                                                        (list, tuple)) else [self.layer.selected]):
             if layer in self.existing_subset_labels:
                 for sg in self.app.data_collection.subset_groups:
                     if sg.label == layer:
                         self.app.data_collection.remove_subset_group(sg)
+                        self.layer._update_items()
                         break
             else:
-                self.app.vue_data_item_remove({'item_name': layer})
+                self.app.data_item_remove(layer)
 
     def vue_remove_from_app(self, *args):
         self.remove_from_app()  # pragma: no cover
