@@ -31,7 +31,7 @@ telescope_names = {
 }
 
 
-@tray_registry('g-data-quality', label="Data Quality", viewer_requirements="image")
+@tray_registry('g-data-quality', label="Data Quality")
 class DataQuality(PluginTemplateMixin, ViewerSelectMixin):
     """
     See the :ref:`Data Quality Plugin Documentation <imviz-data-quality>` for more details.
@@ -53,8 +53,6 @@ class DataQuality(PluginTemplateMixin, ViewerSelectMixin):
     """
     template_file = __file__, "data_quality.vue"
 
-    irrelevant_msg = Unicode("").tag(sync=True)
-
     # `layer` is the science data layer
     science_layer_multiselect = Bool(False).tag(sync=True)
     science_layer_items = List().tag(sync=True)
@@ -97,6 +95,7 @@ class DataQuality(PluginTemplateMixin, ViewerSelectMixin):
             'viewer_selected', 'dq_layer_multiselect', is_root=False,
             is_child_of=self.science_layer.selected
         )
+        self.dq_layer.add_filter('is_dq_layer')
 
         self.load_default_flag_maps()
         self.init_decoding()
