@@ -1,8 +1,15 @@
 import os
 import time
 from pathlib import Path
-from traitlets import Bool, List, Unicode, observe
+import threading
+
+from astropy import units as u
+from astropy.nddata import CCDData
+from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage, SubsetUpdateMessage
 from glue_jupyter.bqplot.image import BqplotImageView
+from regions import CircleSkyRegion, EllipseSkyRegion
+from specutils import Spectrum
+from traitlets import Bool, List, Unicode, observe
 
 from jdaviz.core.custom_traitlets import FloatHandleEmpty, IntHandleEmpty
 from jdaviz.core.marks import ShadowMixin
@@ -12,27 +19,22 @@ from jdaviz.core.template_mixin import (PluginTemplateMixin, SelectPluginCompone
                                         SubsetSelectMixin, PluginTableSelectMixin,
                                         PluginPlotSelectMixin, AutoTextField,
                                         MultiselectMixin, with_spinner)
-from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage, SubsetUpdateMessage
-
 from jdaviz.core.events import AddDataMessage, SnackbarMessage
 from jdaviz.core.user_api import PluginUserApi
-
-from specutils import Spectrum1D
-from astropy import units as u
-from astropy.nddata import CCDData
+from jdaviz.core.region_translators import region2stcs_string
 
 try:
     import cv2
 except ImportError:
     HAS_OPENCV = False
 else:
-    import threading
     HAS_OPENCV = True
 
 __all__ = ['Export']
 
 
-@tray_registry('export', label="Export")
+@tray_registry('export', label="Export",
+               category='core', sidebar='save')
 class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
              DatasetMultiSelectMixin, PluginTableSelectMixin, PluginPlotSelectMixin,
              MultiselectMixin):
@@ -47,6 +49,11 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     * ``viewer`` (:class:`~jdaviz.core.template_mixin.ViewerSelect`)
     * ``viewer_format`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
+    * ``image_custom_size`` (Bool)
+    * ``image_width`` (int)
+        The width of the image to export, in pixels, if ``image_custom_size`` is `True`.
+    * ``image_height`` (int)
+        The height of the image to export, in pixels, if ``image_custom_size`` is `True`.
     * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`)
     * ``dataset_format`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
     * ``subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`)
@@ -64,6 +71,9 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
 
     viewer_format_items = List().tag(sync=True)
     viewer_format_selected = Unicode().tag(sync=True)
+    image_custom_size = Bool(False).tag(sync=True)
+    image_width = IntHandleEmpty(800).tag(sync=True)
+    image_height = IntHandleEmpty(600).tag(sync=True)
 
     plugin_table_format_items = List().tag(sync=True)
     plugin_table_format_selected = Unicode().tag(sync=True)
@@ -90,6 +100,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
     # if selected subset is spectral or composite, display message and disable export
     subset_invalid_msg = Unicode().tag(sync=True)
     data_invalid_msg = Unicode().tag(sync=True)
+    subset_format_invalid_msg = Unicode().tag(sync=True)
 
     # We currently disable exporting spectrum-viewer in Cubeviz
     viewer_invalid_msg = Unicode().tag(sync=True)
@@ -124,6 +135,8 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
         self.dataset.filters = ['is_not_wcs_only', 'not_child_layer',
                                 'from_plugin']
 
+        self.viewer.add_filter('is_not_empty')
+
         viewer_format_options = ['png', 'svg']
         if self.config == 'cubeviz':
             if not self.app.state.settings.get('server_is_remote'):
@@ -145,13 +158,22 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                                                          selected='plugin_table_format_selected',
                                                          manual_options=plugin_table_format_options)
 
-        subset_format_options = [{'label': 'fits', 'value': 'fits', 'disabled': False},
-                                 {'label': 'reg', 'value': 'reg', 'disabled': False},
-                                 {'label': 'ecsv', 'value': 'ecsv', 'disabled': True}]
+        subset_format_options = [{'label': 'fits', 'value': 'fits'},
+                                 {'label': 'reg', 'value': 'reg'},
+                                 {'label': 'ecsv', 'value': 'ecsv'}]
+
+        if self.config == 'imviz':
+            subset_format_options.append({'label': 'stcs', 'value': 'stcs'})
+
         self.subset_format = SelectPluginComponent(self,
                                                    items='subset_format_items',
                                                    selected='subset_format_selected',
-                                                   manual_options=subset_format_options)
+                                                   manual_options=subset_format_options,
+                                                   filters=[self._is_valid_item],
+                                                   apply_filters_to_manual_options=True)
+
+        if 'specviz' in self.config:
+            self.subset_format.selected = 'ecsv'
 
         dataset_format_options = ['fits']
         self.dataset_format = SelectPluginComponent(self,
@@ -190,11 +212,73 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             # on the user's machine, so export support in cubeviz should be disabled
             self.serverside_enabled = False
 
+        if self.config == 'deconfigged':
+            self.observe_traitlets_for_relevancy(
+                traitlets_to_observe=['viewer_items',
+                                      'dataset_items',
+                                      'subset_items',
+                                      'plugin_table_items',
+                                      'plugin_plot_items'],
+                irrelevant_msg_callback=self.relevant_if_any_truthy)
+
+    def _is_valid_item(self, item):
+        return self._is_not_stcs(item) or self._is_stcs_region_supported(item)
+
+    def _is_not_stcs(self, item):
+        return item.get('label', '') != 'stcs'
+
+    def _is_stcs_region_supported(self, item):
+        region = getattr(self.subset, 'selected_spatial_region', None)
+        return isinstance(region, (CircleSkyRegion, EllipseSkyRegion))
+
+    def _is_subset_format_supported(self):
+        """
+        Check if the format of the subset to be exported is supported.
+        """
+        if self.subset.selected == '' or self.subset.selected is None:
+            return
+
+        subset = self.app.get_subsets(self.subset.selected)
+        selected = self.subset_format.selected
+
+        self.subset_format_invalid_msg = ''
+        is_supported = True
+
+        try:
+            is_spectral = self.app._is_subset_spectral(subset[0])
+        except KeyError:
+            is_spectral = False
+
+        # Disable selecting a bad subset+format combination from the API
+        if is_spectral and selected != 'ecsv':
+            is_supported = False
+        elif not is_spectral and selected == 'ecsv':
+            is_supported = False
+
+        # raise vue message
+        if not is_supported:
+            self.subset_format_invalid_msg = (f"Export of '{self.subset.selected}' "
+                                              f"in '{selected}' format is not supported.")
+
+    @observe('subset_selected')
+    def _on_subset_selected(self, event):
+        if hasattr(self, 'subset_format'):
+            self.subset_format._update_items()
+            self._is_subset_format_supported()
+
+    # Use an observer function rather than slap the decorator on
+    # _is_subset_format_supported for clarity and to follow the
+    # convention of the method above.
+    @observe('subset_format_selected')
+    def _on_subset_format_selected(self, event):
+        self._is_subset_format_supported()
+
     @property
     def user_api(self):
         # TODO: backwards compat for save_figure, save_movie,
         # i_start, i_end, movie_fps, movie_filename
         expose = ['viewer', 'viewer_format',
+                  'image_custom_size', 'image_width', 'image_height',
                   'dataset', 'dataset_format',
                   'subset', 'subset_format',
                   'plugin_table', 'plugin_table_format',
@@ -210,7 +294,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
         # NOTE: This needs revising if we allow loading more than one cube.
         if isinstance(msg.viewer, BqplotImageView):
             if len(msg.data.shape) == 3:
-                self.i_end = msg.data.shape[-1] - 1
+                self.i_end = msg.data.shape[msg.data.meta['spectral_axis_index']] - 1
 
     @observe('multiselect', 'viewer_multiselect')
     def _sync_multiselect_traitlets(self, event):
@@ -226,9 +310,8 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
     @observe('viewer_selected', 'dataset_selected', 'subset_selected',
              'plugin_table_selected', 'plugin_plot_selected')
     def _sync_singleselect(self, event):
-
         if not hasattr(self, 'dataset') or not hasattr(self, 'viewer'):
-            # plugin not fully intialized
+            # plugin not fully initialized
             return
         # if multiselect is not enabled, only allow a single selection across all select components
         if self.multiselect:
@@ -241,8 +324,6 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             if name != attr:
                 setattr(self, attr, '')
             if attr == 'subset_selected':
-                if self.subset.selected != '':
-                    self._update_subset_format_disabled()
                 self._set_subset_not_supported_msg()
             if attr == 'dataset_selected':
                 self._set_dataset_not_supported_msg()
@@ -287,51 +368,6 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
         # Clear overwrite warning when user changes filename
         self.overwrite_warn = False
 
-    def _update_subset_format_disabled(self):
-        new_items = []
-        if self.subset.selected is not None:
-            try:
-                subset = self.app.get_subsets(self.subset.selected)
-            except Exception:
-                # subset invalid message will already be set,
-                # no need to set valid/invalid formats.
-                return
-            if self.app._is_subset_spectral(subset[0]):
-                good_formats = ["ecsv"]
-            else:
-                good_formats = ["fits", "reg"]
-            for item in self.subset_format_items:
-                if item["label"] in good_formats:
-                    item["disabled"] = False
-                else:
-                    item["disabled"] = True
-                    if item["label"] == self.subset_format.selected:
-                        self.subset_format.selected = good_formats[0]
-                new_items.append(item)
-        self.subset_format_items = []
-        self.subset_format_items = new_items
-
-    @observe('subset_format_selected')
-    def _disable_subset_format_combo(self, event):
-        # Disable selecting a bad subset+format combination from the API
-        if self.subset.selected == '' or self.subset.selected is None:
-            return
-        subset = self.app.get_subsets(self.subset.selected)
-        bad_combo = False
-        if self.app._is_subset_spectral(subset[0]):
-            if event['new'] != "ecsv":
-                bad_combo = True
-        elif event['new'] == "ecsv":
-            bad_combo = True
-
-        if bad_combo:
-            # Set back to a good value and raise error
-            good_format = [format["label"] for format in self.subset_format_items if
-                           format["disabled"] is False][0]
-            self.subset_format.selected = good_format
-            raise ValueError(f"Cannot export {self.subset.selected} in {event['new']}"
-                             f" format, reverting selection to {self.subset_format.selected}")
-
     def _set_subset_not_supported_msg(self, msg=None):
         """
         Check if selected subset is spectral or composite, and warn and
@@ -346,6 +382,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             else:
                 if self.subset.selected == '':
                     self.subset_invalid_msg = ''
+                    self.subset_format_invalid_msg = ''
                 elif self.app._is_subset_spectral(subset[0]):
                     self.subset_invalid_msg = ''
                 elif len(subset) > 1:
@@ -354,6 +391,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                     self.subset_invalid_msg = ''
         else:  # no subset selected (can be '' instead of None if previous selection made)
             self.subset_invalid_msg = ''
+            self.subset_format_invalid_msg = ''
 
     def _set_dataset_not_supported_msg(self, msg=None):
         if self.dataset.selected_obj is not None:
@@ -361,7 +399,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                 # NOTE: should not be a valid choice due to dataset filters, but we'll include
                 # another check here.
                 self.data_invalid_msg = "Data export is only available for plugin generated data."
-            elif not isinstance(self.dataset.selected_obj, (Spectrum1D, CCDData)):
+            elif not isinstance(self.dataset.selected_obj, (Spectrum, CCDData)):
                 self.data_invalid_msg = "Export is not yet implemented for this type of data"
             elif (data_unit := self.dataset.selected_obj.unit) == u.Unit('DN/s'):
                 self.data_invalid_msg = f'Export Disabled: The unit {data_unit} could not be saved in native FITS format.'  # noqa: E501
@@ -424,7 +462,6 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             If `True`, raise exception when ``overwrite=False`` but
             output file already exists. Otherwise, a message will be sent
             to application snackbar instead.
-
         """
         if self.multiselect:
             raise NotImplementedError("batch export not yet supported")
@@ -465,9 +502,13 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                 restores.append(restore)
 
             if filetype == "mp4":
-                self.save_movie(viewer, filename, filetype)
+                self.save_movie(viewer, filename, filetype,
+                                width=f"{self.image_width}px" if self.image_custom_size else None,
+                                height=f"{self.image_height}px" if self.image_custom_size else None)
             else:
-                self.save_figure(viewer, filename, filetype, show_dialog=show_dialog)
+                self.save_figure(viewer, filename, filetype, show_dialog=show_dialog,
+                                 width=f"{self.image_width}px" if self.image_custom_size else None,
+                                 height=f"{self.image_height}px" if self.image_custom_size else None)  # noqa
 
             # restore marks to their original state
             for restore, mark in zip(restores, viewer.figure.marks):
@@ -509,7 +550,10 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             filetype = self.subset_format.selected
             filename = self._normalize_filename(filename, filetype, overwrite=overwrite)
             if self.subset_invalid_msg != '':
-                raise NotImplementedError(f'Subset can not be exported - {self.subset_invalid_msg}')
+                raise NotImplementedError(f'Subset cannot be exported - {self.subset_invalid_msg}')
+            elif self.subset_format_invalid_msg:
+                raise ValueError(self.subset_format_invalid_msg)
+
             if self.overwrite_warn and not overwrite:
                 if raise_error_for_overwrite:
                     raise FileExistsError(f"{filename} exists but overwrite=False")
@@ -519,6 +563,8 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                 self.save_subset_as_region(selected_subset_label, filename)
             elif self.subset_format.selected == 'ecsv':
                 self.save_subset_as_table(filename)
+            elif self.subset_format.selected == 'stcs':
+                self.save_subset_as_stcs(filename)
 
         elif len(self.dataset.selected):
             filetype = self.dataset_format.selected
@@ -560,35 +606,97 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                     f"Exported to {filename} (overwrite)", sender=self, color="success"))
             self.overwrite_warn = False
 
-    def save_figure(self, viewer, filename=None, filetype="png", show_dialog=False):
-        if filetype == "png":
+    def save_figure(self, viewer, filename=None, filetype="png", show_dialog=False,
+                    width=None, height=None):
+        if filename is None:
+            filename = self.filename_default
 
-            if filename is None:
-                filename = self.filename_default
+        # viewers in plugins will have viewer.app, other viewers have viewer.jdaviz_app
+        if hasattr(viewer, 'jdaviz_app'):
+            app = viewer.jdaviz_app
+        else:
+            app = viewer.app
 
-            def on_img_received(data):
-                try:
-                    with filename.open(mode='bw') as f:
-                        f.write(data)
-                except Exception as e:
-                    self.hub.broadcast(SnackbarMessage(
-                        f"{self.viewer.selected} failed to export to {str(filename)}: {e}",
-                        sender=self, color="error"))
-                finally:
-                    self.hub.broadcast(SnackbarMessage(
-                        f"{self.viewer.selected} exported to {str(filename)}",
-                        sender=self, color="success"))
+        def on_img_received(data):
+            try:
+                with filename.open(mode='bw') as f:
+                    f.write(data)
+            except Exception as e:
+                self.hub.broadcast(SnackbarMessage(
+                    f"{self.viewer.selected} failed to export to {str(filename)}: {e}",
+                    sender=self, color="error"))
+            finally:
+                self.hub.broadcast(SnackbarMessage(
+                    f"{self.viewer.selected} exported to {str(filename)}",
+                    sender=self, color="success"))
 
-            if viewer.figure._upload_png_callback is not None:
+        def get_png(figure):
+            if figure._upload_png_callback is not None:
                 raise ValueError("previous png export is still in progress. Wait to complete before making another call to save_figure")  # noqa: E501 # pragma: no cover
 
-            viewer.figure.get_png_data(on_img_received)
+            figure.get_png_data(on_img_received)
 
-        elif filetype == "svg":
-            viewer.figure.save_svg(str(filename) if filename is not None else None)
+        if (width is not None or height is not None):
+            assert width is not None and height is not None, \
+                "Both width and height must be provided"
+            import ipywidgets as widgets
+            from typing import Callable
+
+            def _show_hidden(widget: widgets.Widget, width: str, height: str):
+                import ipyvuetify as v
+                wrapper_widget = v.Html(
+                    children=[
+                        v.Html(children=[
+                            widget
+                        ], tag="div", style_=f"width: {width}; height: {height};")
+                    ],
+                    tag="div",
+                    style_="overflow: hidden; width: 0px; height: 0px"
+                    )
+                # TODO: we might want to remove it from the DOM
+                app.invisible_children = [*app.invisible_children, wrapper_widget]
+
+            def _widget_after_first_display(widget: widgets.Widget, callback: Callable):
+                if widget._view_count is None:
+                    widget._view_count = 0
+                called_callback = False
+
+                def view_count_changed(change):
+                    nonlocal called_callback
+                    if change["new"] == 1 and not called_callback:
+                        called_callback = True
+                        callback()
+                widget.observe(view_count_changed, "_view_count")
+
+            cloned_viewer = viewer._clone_viewer_outside_app()
+            # make sure we will the size of our container which defines the
+            # size of the figure
+            cloned_viewer.figure.layout.width = "100%"
+            cloned_viewer.figure.layout.height = "100%"
+
+            def on_figure_displayed():
+                # we need a bit of a delay to ensure the figure is fully displayed
+                # maybe this can be fixed on the bqplot side in the future
+                def wait_in_other_thread():
+                    import time
+                    time.sleep(0.2)
+                    get_png(cloned_viewer.figure)
+                # wait in other thread to avoid blocking the main thread (widgets can update)
+                threading.Thread(target=wait_in_other_thread).start()
+            _widget_after_first_display(cloned_viewer.figure, on_figure_displayed)
+            _show_hidden(cloned_viewer.figure, width, height)
+        elif filetype == 'png':
+            # NOTE: get_png already check if _upload_png_callback is not None
+            get_png(viewer.figure)
+        elif filetype == 'svg':
+            if viewer.figure._upload_svg_callback is not None:
+                raise ValueError("previous svg export is still in progress. Wait to complete before making another call to save_figure") # noqa
+            viewer.figure.get_svg_data(on_img_received)
+        else:
+            raise ValueError(f"Unsupported filetype={filetype} for save_figure")
 
     @with_spinner('movie_recording')
-    def _save_movie(self, viewer, i_start, i_end, fps, filename, rm_temp_files):
+    def _save_movie(self, viewer, i_start, i_end, fps, filename, rm_temp_files, width, height):
         # NOTE: All the stuff here has to be in the same thread but
         #       separate from main app thread to work.
 
@@ -602,6 +710,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
         temp_png_files = []
         i = i_start
         video = None
+        slice_plg.value = slice_plg.valid_values_sorted[i_start]
 
         # TODO: Expose to users?
         i_step = 1  # Need n_frames check if we allow tweaking
@@ -614,7 +723,8 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
                 slice_plg.vue_play_next()
                 cur_pngfile = Path(f"._cubeviz_movie_frame_{i}.png")
                 # TODO: skip success snackbars when exporting temp movie frames?
-                self.save_figure(viewer, filename=cur_pngfile, filetype="png", show_dialog=False)
+                self.save_figure(viewer, filename=cur_pngfile, filetype="png", show_dialog=False,
+                                 width=width, height=height)
                 temp_png_files.append(cur_pngfile)
                 i += i_step
 
@@ -637,7 +747,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             cv2.destroyAllWindows()
             if video:
                 video.release()
-            slice_plg._on_slider_updated({'new': orig_slice})
+            slice_plg.value = slice_plg.valid_values_sorted[orig_slice]
 
         if rm_temp_files or self.movie_interrupt:
             for cur_pngfile in temp_png_files:
@@ -650,7 +760,7 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             self.movie_interrupt = False
 
     def save_movie(self, viewer, filename, filetype, i_start=None, i_end=None, fps=None,
-                   rm_temp_files=True):
+                   rm_temp_files=True, width=None, height=None):
         """Save selected slices as a movie.
 
         This method creates a PNG file per frame (``._cubeviz_movie_frame_<n>.png``)
@@ -683,6 +793,12 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
         rm_temp_files : bool
             Remove temporary PNG files after movie creation. Default is `True`.
 
+        width : str, optional
+            Width of the exported image. Required if height is provided.
+
+        height : str, optional
+            Height of the exported image. Required if width is provided.
+
         Returns
         -------
         out_filename : str
@@ -723,11 +839,40 @@ class Export(PluginTemplateMixin, ViewerSelectMixin, SubsetSelectMixin,
             raise ValueError(f"No frames to write: i_start={i_start}, i_end={i_end}")
 
         threading.Thread(
-            target=lambda: self._save_movie(viewer, i_start, i_end, fps, filename, rm_temp_files)
+            target=lambda: self._save_movie(viewer, i_start, i_end, fps, filename, rm_temp_files,
+                                            width, height)
         ).start()
 
         return filename
 
+    def save_subset_as_stcs(self, filename):
+        """
+        Save a subset region to a text file in STC-S format.
+
+        Currently implemented for Circle and Ellipse sky regions only.
+
+        Parameters
+        ----------
+        filename : str
+            Write the STC-S region to a text file with this name.
+
+        Raises
+        ------
+        RuntimeError
+            If data is not aligned by WCS, which is required for STC-S export.
+        """
+        align_by = getattr(self.app, '_align_by', None)
+        if align_by != 'wcs':
+            raise RuntimeError("Please link datasets by WCS first using the Orientation plugin.")
+
+        region = self.app.get_subsets(subset_name=self.subset.selected,
+                                      include_sky_region=True)[0]['sky_region']
+
+        stcs_str = region2stcs_string(region)
+
+        with open(filename, 'w') as f:
+            f.write(stcs_str)
+
     def save_subset_as_region(self, selected_subset_label, filename):
         """
         Save a subset to file as a Region object in the working directory.
