@@ -1,7 +1,7 @@
 import numpy as np
 
 from astropy.convolution import convolve, Gaussian2DKernel
-from specutils import Spectrum1D
+from specutils import Spectrum
 from specutils.manipulation import gaussian_smooth
 from traitlets import List, Unicode, Bool, observe
 
@@ -16,8 +16,7 @@ from jdaviz.core.user_api import PluginUserApi
 __all__ = ['GaussianSmooth']
 
 
-@tray_registry('g-gaussian-smooth', label="Gaussian Smooth",
-               viewer_requirements=['spectrum', 'flux'])
+@tray_registry('g-gaussian-smooth', label="Gaussian Smooth", category="data:manipulation")
 class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
     """
     See the :ref:`Gaussian Smooth Plugin Documentation <gaussian-smooth>` for more details.
@@ -60,7 +59,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
             self.dataset._viewers = [self._default_spectrum_viewer_reference_name]
             self.dataset._clear_cache()
 
-        self.dataset.add_filter('not_from_this_plugin')
+        self.dataset.add_filter('not_from_this_plugin', 'is_spectrum_or_cube')
 
         self.mode = SelectPluginComponent(self,
                                           items='mode_items',
@@ -73,6 +72,9 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
         # description displayed under plugin title in tray
         self._plugin_description = 'Smooth data with a Gaussian kernel.'
 
+        if self.app.config == 'deconfigged':
+            self.observe_traitlets_for_relevancy(traitlets_to_observe=['dataset_items'])
+
     @property
     def _default_spectrum_viewer_reference_name(self):
         return getattr(
@@ -151,7 +153,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         Returns
         -------
-        spec : `~specutils.Spectrum1D`
+        spec : `~specutils.Spectrum`
             The smoothed spectrum or data cube
         """
         if self.mode_selected == 'Spatial':
@@ -190,18 +192,18 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         Returns
         -------
-        spec : `~specutils.Spectrum1D`
+        spec : `~specutils.Spectrum`
             The smoothed spectrum
         """
         # Testing inputs to make sure putting smoothed spectrum into
         # datacollection works
         # input_flux = Quantity(np.array([0.2, 0.3, 2.2, 0.3]), u.Jy)
         # input_spaxis = Quantity(np.array([1, 2, 3, 4]), u.micron)
-        # spec1 = Spectrum1D(input_flux, spectral_axis=input_spaxis)
+        # spec1 = Spectrum(input_flux, spectral_axis=input_spaxis)
 
         # Takes the user input from the dialog (stddev) and uses it to
         # define a standard deviation for gaussian smoothing
-        cube = self.dataset.get_object(cls=Spectrum1D, statistic=None)
+        cube = self.dataset.get_object(cls=Spectrum, statistic=None)
         spec_smoothed = gaussian_smooth(cube, stddev=self.stddev)
 
         return spec_smoothed
@@ -215,7 +217,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         Returns
         -------
-        cube : `~specutils.Spectrum1D`
+        cube : `~specutils.Spectrum`
             The smoothed cube
         """
         cube = self.dataset.selected_obj
@@ -223,7 +225,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         # Extend the 2D kernel to have a length 1 spectral dimension, so that
         # we can do "3d" convolution to the whole cube
-        kernel = np.expand_dims(Gaussian2DKernel(self.stddev), 2)
+        kernel = np.expand_dims(Gaussian2DKernel(self.stddev), cube.meta['spectral_axis_index'])
 
         convolved_data = convolve(cube, kernel)
 
@@ -237,6 +239,6 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         # Create a new cube with the old metadata. Note that astropy
         # convolution generates values for masked (NaN) data.
-        newcube = Spectrum1D(flux=convolved_data * flux_unit, wcs=w)
+        newcube = Spectrum(flux=convolved_data * flux_unit, wcs=w)
 
         return newcube
