@@ -8,10 +8,11 @@ from jdaviz.utils import PRIHDR_KEY, COMMENTCARD_KEY
 __all__ = ['MetadataViewer']
 
 
-@tray_registry('g-metadata-viewer', label="Metadata")
+@tray_registry('g-metadata-viewer', label="Metadata",
+               category='core', sidebar='info', subtab=0)
 class MetadataViewer(PluginTemplateMixin, DatasetSelectMixin):
     """
-    See the :ref:`Metadata Viewer Plugin Documentation <imviz_metadata-viewer>` for more details.
+    See the :ref:`Metadata Viewer Plugin Documentation <imviz-metadata-viewer>` for more details.
 
     Only the following attributes and methods are available through the
     :ref:`public plugin API <plugin-apis>`:
@@ -45,6 +46,9 @@ class MetadataViewer(PluginTemplateMixin, DatasetSelectMixin):
         # description displayed under plugin title in tray
         self._plugin_description = 'View metadata.'
 
+        if self.config == "deconfigged":
+            self.observe_traitlets_for_relevancy(traitlets_to_observe=['dataset_items'])
+
     @property
     def user_api(self):
         return PluginUserApi(self, expose=('dataset', 'show_primary'),
