@@ -3,7 +3,7 @@ import numpy as np
 from copy import deepcopy
 
 import astropy.units as u
-from specutils import Spectrum1D
+from specutils import Spectrum
 from specutils.utils import QuantityModel
 from traitlets import Bool, List, Unicode, observe
 
@@ -40,7 +40,7 @@ class _EmptyParam:
                                    self.unit if self.unit is not None else u.dimensionless_unscaled)
 
 
-@tray_registry('g-model-fitting', label="Model Fitting", viewer_requirements='spectrum')
+@tray_registry('g-model-fitting', label="Model Fitting", category="data:analysis")
 class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                    SpectralSubsetSelectMixin, DatasetSpectralSubsetValidMixin,
                    NonFiniteUncertaintyMismatchMixin,
@@ -79,6 +79,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
       Label of the residuals to apply when calling :meth:`calculate_fit` if ``residuals_calculate``
       is ``True``.
     * :meth:`calculate_fit`
+    * :meth:`fitted_models`
+    * :meth:`get_models`
+    * :meth:`get_model_parameters`
     """
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "model_fitting.vue"
@@ -88,6 +91,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     model_comp_items = List().tag(sync=True)
     model_comp_selected = Unicode().tag(sync=True)
     poly_order = IntHandleEmpty(0).tag(sync=True)
+    poly_order_invalid_msg = Unicode().tag(sync=True)
 
     comp_label = Unicode().tag(sync=True)
     comp_label_default = Unicode().tag(sync=True)
@@ -115,9 +119,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._units = {}
+        self._units = {}  # string representation of display units
         self._fitted_model = None
         self._fitted_spectrum = None
+        self._fitted_models = {}
         self.component_models = []
         self._initialized_models = {}
         self._display_order = False
@@ -164,6 +169,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self.hub.subscribe(self, GlobalDisplayUnitChanged,
                            handler=self._on_global_display_unit_changed)
 
+        self.parallel_n_cpu = None
+        if self.config == "deconfigged":
+            self.observe_traitlets_for_relevancy(traitlets_to_observe=['dataset_items'])
+
     @property
     def _default_spectrum_viewer_reference_name(self):
         return getattr(
@@ -181,13 +190,16 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         expose = ['dataset']
         if self.config == "cubeviz":
             expose += ['cube_fit']
-        expose += ['spectral_subset', 'model_component', 'poly_order', 'model_component_label',
-                   'model_components', 'valid_model_components',
-                   'create_model_component', 'remove_model_component',
-                   'get_model_component', 'set_model_component', 'reestimate_model_parameters',
+        expose += ['spectral_subset', 'model_component',
+                   'poly_order', 'model_component_label', 'model_components',
+                   'valid_model_components', 'create_model_component',
+                   'remove_model_component', 'get_model_component',
+                   'set_model_component', 'reestimate_model_parameters',
                    'equation', 'equation_components',
-                   'add_results', 'residuals_calculate', 'residuals']
-        expose += ['calculate_fit', 'clear_table', 'export_table']
+                   'add_results', 'residuals_calculate',
+                   'residuals']
+        expose += ['calculate_fit', 'clear_table', 'export_table',
+                   'fitted_models', 'get_models', 'get_model_parameters']
         return PluginUserApi(self, expose=expose)
 
     def _param_units(self, param, model_type=None):
@@ -320,11 +332,11 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         sb_unit = self.app._get_display_unit('sb')
         spectral_y_unit = self.app._get_display_unit('spectral_y')
         if event.get('new'):
-            self._units['y'] = sb_unit
+            self._units['y'] = str(sb_unit)
             self.dataset.add_filter('is_flux_cube')
             self.dataset.remove_filter('layer_in_spectrum_viewer')
         else:
-            self._units['y'] = spectral_y_unit
+            self._units['y'] = str(spectral_y_unit)
             self.dataset.add_filter('layer_in_spectrum_viewer')
             self.dataset.remove_filter('is_flux_cube')
 
@@ -358,7 +370,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         if selected_spec is None:
             return
 
-        # Replace NaNs from collapsed Spectrum1D in Cubeviz
+        # Replace NaNs from collapsed Spectrum in Cubeviz
         # (won't affect calculations because these locations are masked)
         selected_spec.flux[np.isnan(selected_spec.flux)] = 0.0
 
@@ -382,6 +394,41 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self.comp_label_default = self._default_comp_label(self.model_comp_selected,
                                                            self.poly_order)
 
+    @observe('poly_order')
+    def _check_poly_order(self, event={}, model_comp=None, poly_order=None):
+        """
+        Check that the polynomial order is valid if the model component is
+        Polynomial1D.  If not, set the invalid message and return None.
+
+        Parameters
+        ----------
+        event : dict
+            IPyWidget callback event object.
+        model_comp : str
+            Type of model component to check.  If not provided, will default
+            to the object attribute ``model_comp_selected``.
+        poly_order : int
+            Order of the polynomial to check.  If not provided, will default
+            to the object attribute ``poly_order``.
+
+        Returns
+        -------
+        int or None
+            The polynomial order if valid, otherwise None.
+        """
+        self.poly_order_invalid_msg = ""
+
+        model_comp = model_comp if model_comp is not None else self.model_comp_selected
+        if model_comp != 'Polynomial1D':
+            return None
+
+        poly_order = poly_order if poly_order is not None else self.poly_order
+        if not isinstance(poly_order, int) or poly_order < 0:
+            self.poly_order_invalid_msg = "Order must be an integer >= 0"
+            return None
+
+        return poly_order
+
     @observe('comp_label')
     def _comp_label_changed(self, event={}):
         if not len(self.comp_label.strip()):
@@ -445,7 +492,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         if model_comp != "Polynomial1D" and poly_order is not None:
             raise ValueError("poly_order should only be passed if model_component is Polynomial1D")
-        poly_order = poly_order if poly_order is not None else self.poly_order
+
+        poly_order = self._check_poly_order(model_comp=model_comp, poly_order=poly_order)
+        if self.poly_order_invalid_msg:
+            raise ValueError(f"poly_{self.poly_order_invalid_msg.lower()}")
 
         # if model_component was passed and different than the one set in the traitlet, AND
         # model_component_label is not passed, AND the auto is enabled on the label, then
@@ -478,12 +528,12 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # Need to set the units the first time we initialize a model component, after this
         # we listen for display unit changes
         if self._units.get('x', '') == '':
-            self._units['x'] = self.app._get_display_unit('spectral')
+            self._units['x'] = str(self.app._get_display_unit('spectral'))
         if self._units.get('y', '') == '':
             if self.cube_fit:
-                self._units['y'] = self.app._get_display_unit('sb')
+                self._units['y'] = str(self.app._get_display_unit('sb'))
             else:
-                self._units['y'] = self.app._get_display_unit('spectral_y')
+                self._units['y'] = str(self.app._get_display_unit('spectral_y'))
 
         if model_comp == "Polynomial1D":
             # self.poly_order is the value in the widget for creating
@@ -512,6 +562,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                 if default_param.unit != default_units:
                     pixar_sr = self.app.data_collection[0].meta.get('PIXAR_SR', 1)
                     viewer = self.app.get_viewer("spectrum-viewer")
+                    # TODO: I suspect this doesn't actually work, but never gets called -Ricky
                     cube_wave = viewer.slice_value * u.Unit(self.app._get_display_unit('spectral'))
                     equivs = all_flux_unit_conversion_equivs(pixar_sr, cube_wave)
 
@@ -529,23 +580,35 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                 data = self.app.data_collection[self.dataset_selected].get_object(statistic=None)
             else:  # User selected some subset from spectrum viewer, just use original cube
                 data = self.app.data_collection[0].get_object(statistic=None)
-            masked_spectrum = self._apply_subset_masks(data, self.spectral_subset)
+            spatial_axes = [0, 1, 2]
+            spatial_axes.remove(data.spectral_axis_index)
+            masked_spectrum = self._apply_subset_masks(data, self.spectral_subset,
+                                                       spatial_axes=tuple(spatial_axes))
         else:
             masked_spectrum = self._apply_subset_masks(self.dataset.selected_spectrum,
                                                        self.spectral_subset)
+
         mask = masked_spectrum.mask
         if mask is not None:
             if mask.ndim == 3:
-                spectral_mask = mask.all(axis=(0, 1))
+                if masked_spectrum.spectral_axis_index in [2, -1]:
+                    spectral_mask = mask.all(axis=(0, 1))
+                elif masked_spectrum.spectral_axis_index == 0:
+                    spectral_mask = mask.all(axis=(1, 2))
             else:
                 spectral_mask = mask
             init_x = masked_spectrum.spectral_axis[~spectral_mask]
             orig_flux_shape = masked_spectrum.flux.shape
             init_y = masked_spectrum.flux[~mask]
             if mask.ndim == 3:
-                init_y = init_y.reshape(orig_flux_shape[0],
-                                        orig_flux_shape[1],
-                                        len(init_x))
+                if masked_spectrum.spectral_axis_index in [2, -1]:
+                    init_y = init_y.reshape(orig_flux_shape[0],
+                                            orig_flux_shape[1],
+                                            len(init_x))
+                elif masked_spectrum.spectral_axis_index == 0:
+                    init_y = init_y.reshape(len(init_x),
+                                            orig_flux_shape[1],
+                                            orig_flux_shape[2])
         else:
             init_x = masked_spectrum.spectral_axis
             init_y = masked_spectrum.flux
@@ -553,18 +616,21 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         if init_y.unit != self._units['y']:
             # equivs for spectral density and flux<>sb
             pixar_sr = masked_spectrum.meta.get('_pixel_scale_factor', 1.0)
-            equivs = all_flux_unit_conversion_equivs(pixar_sr, init_x)
+            equivs = all_flux_unit_conversion_equivs(pixar_sr, init_x.mean())
 
-            init_y = flux_conversion_general([init_y.value],
+            init_y = flux_conversion_general(init_y.value,
                                              init_y.unit,
                                              self._units['y'],
                                              equivs)
 
+        # We need this for models where we average over the spatial axes to initialize
+        spectral_axis_index = masked_spectrum.spectral_axis_index
+
         initialized_model = initialize(
             MODELS[model_comp](name=comp_label,
                                **initial_values,
                                **new_model.get("model_kwargs", {})),
-            init_x, init_y)
+            init_x, init_y, spectral_axis_index=spectral_axis_index)
 
         # need to loop over parameters again as the initializer may have overridden
         # the original default value. However, if we toggled cube_fit, we may need to override
@@ -602,26 +668,42 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self._check_model_equation_invalid()
 
     def _on_global_display_unit_changed(self, msg):
-        if msg.axis == 'spectral_y':
+        if msg.axis in ('spectral_y', 'sb', 'flux'):
             axis = 'y'
         elif msg.axis == 'spectral':
             axis = 'x'
         else:
             return
 
-        if axis == 'y' and self.cube_fit:
-            # The units have to be in surface brightness for a cube fit.
-            uc = self.app._jdaviz_helper.plugins['Unit Conversion']
+        unit = msg.unit
 
-            if msg.unit != uc._obj.sb_unit_selected:
-                self._units[axis] = uc._obj.sb_unit_selected
-                self._check_model_component_compat([axis], [u.Unit(uc._obj.sb_unit_selected)])
+        if axis == 'y':
+            # The units have to be in surface brightness for a cube fit.
+            uc = self.app._jdaviz_helper.plugins.get('Unit Conversion', None)
+            if uc is None:
                 return
+            if not self.cube_fit:
+                # use spectrum viewer y axis units
+                if hasattr(uc, 'spectral_y_unit'):
+                    # NOTE, spectral_y_unit should probably be set and exposed
+                    # in the unit conversion plugin for specviz2d
+                    # and specviz so this check doesn't have to be done
+                    unit = u.Unit(uc.spectral_y_unit)
+                else:
+                    unit = u.Unit(self.app._get_display_unit('spectral_y'))
+
+            else:
+                if unit != uc._obj.sb_unit_selected:
+                    self._units[axis] = uc._obj.sb_unit_selected
+                    self._check_model_component_compat([axis], [u.Unit(uc._obj.sb_unit_selected)])
+                    return
+                elif msg.axis == 'flux' and uc._obj.has_sb:
+                    unit = u.Unit(self.app._get_display_unit('sb'))
 
         # update internal tracking of current units
-        self._units[axis] = str(msg.unit)
+        self._units[axis] = str(unit)
 
-        self._check_model_component_compat([axis], [msg.unit])
+        self._check_model_component_compat([axis], [unit])
 
     def remove_model_component(self, model_component_label):
         """
@@ -779,6 +861,238 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         """
         return re.split(r'[+*/-]', self.equation.value.replace(' ', ''))
 
+    @property
+    def fitted_models(self):
+        """
+        Dictionary of all previously fitted models.
+        """
+        return self._fitted_models
+
+    def get_models(self, models=None, model_label=None, x=None, y=None):
+        """
+        Loop through all models and output models of the label model_label.
+        If x or y is set, return model_labels of those (x, y) coordinates.
+        If x and y are None, print all models regardless of coordinates.
+
+        Parameters
+        ----------
+        models : dict
+            A dict of models, with the key being the label name and the value
+            being an `astropy.modeling.CompoundModel` object. Defaults to
+            `fitted_models` if no parameter is provided.
+        model_label : str
+            The name of the model that will be found and returned. If it
+            equals default, every model present will be returned.
+        x : int
+            The x coordinate of the model spaxels that will be returned.
+        y : int
+            The y coordinate of the model spaxels that will be returned.
+
+        Returns
+        -------
+        selected_models : dict
+            Dictionary of the selected models.
+        """
+        selected_models = {}
+        # If models is not provided, use the app's fitted models
+        if not models:
+            models = self.fitted_models
+
+        # Loop through all keys in the dict models
+        for label in models:
+            # Prevent "Model 2" from being returned when model_label is "Model"
+            if model_label is not None:
+                if label.split(" (")[0] != model_label:
+                    continue
+
+            # If no label was provided, use label name without coordinates.
+            if model_label is None and " (" in label:
+                find_label = label.split(" (")[0]
+            # If coordinates are not present, just use the label.
+            elif model_label is None:
+                find_label = label
+            else:
+                find_label = model_label
+
+            # If x and y are set, return keys that match the model plus that
+            # coordinate pair. If only x or y is set, return keys that fit
+            # that value for the appropriate coordinate.
+            if x is not None and y is not None:
+                find_label = r"{} \({}, {}\)".format(find_label, x, y)
+            elif x:
+                find_label = r"{} \({}, .+\)".format(find_label, x)
+            elif y:
+                find_label = r"{} \(.+, {}\)".format(find_label, y)
+
+            if re.search(find_label, label):
+                selected_models[label] = models[label]
+
+        return selected_models
+
+    def _get_model_parameters(self, models=None, model_label=None, x=None, y=None):
+        """
+        Convert each parameter of model inside models into a coordinate that
+        maps the model name and parameter name to a `astropy.units.Quantity`
+        object.
+
+        Parameters
+        ----------
+        models : dict
+            A dictionary where the key is a model name and the value is an
+            `astropy.modeling.CompoundModel` object.
+        model_label : str
+            Get model parameters for a particular model by inputting its label.
+        x : int
+            The x coordinate of the model spaxels that will be returned from
+            get_models.
+        y : int
+            The y coordinate of the model spaxels that will be returned from
+            get_models.
+
+        Returns
+        -------
+        :dict: a dictionary of the form
+            {model name: {parameter name: [[`astropy.units.Quantity`]]}}
+            for 3d models or
+            {model name: {parameter name: `astropy.units.Quantity`}} where the
+            Quantity object represents the parameter value and unit of one of
+            spaxel models or the 1d models, respectively.
+        """
+        if models and model_label:
+            models = self.get_models(models=models, model_label=model_label, x=x, y=y)
+        elif models is None and model_label:
+            models = self.get_models(model_label=model_label, x=x, y=y)
+        elif models is None:
+            models = self.fitted_models
+
+        data_shapes = {}
+        for label in models:
+            data_label = label.split(" (")[0]
+            if data_label not in data_shapes:
+                data_shapes[data_label] = self.app.data_collection[data_label].data.shape
+
+        param_dict = {}
+        parameters_cube = {}
+        param_x_y = {}
+        param_units = {}
+
+        for label in models:
+            # 3d models take the form of "Model (1,2)" so this if statement
+            # looks for that style and separates out the pertinent information.
+            if " (" in label:
+                label_split = label.split(" (")
+                model_name = label_split[0]
+                x = int(label_split[1].split(", ")[0])
+                y = int(label_split[1].split(", ")[1][:-1])
+
+                # x and y values are added to this dict where they will be used
+                # to convert the models of each spaxel into a single
+                # coordinate in the parameters_cube dictionary.
+                if model_name not in param_x_y:
+                    param_x_y[model_name] = {'x': [], 'y': []}
+                if x not in param_x_y[model_name]['x']:
+                    param_x_y[model_name]['x'].append(x)
+                if y not in param_x_y[model_name]['y']:
+                    param_x_y[model_name]['y'].append(y)
+
+            # 1d models will be handled by this else statement.
+            else:
+                model_name = label
+
+            if model_name not in param_dict:
+                param_dict[model_name] = list(models[label].param_names)
+
+        # This adds another dictionary as the value of
+        # parameters_cube[model_name] where the key is the parameter name
+        # and the value is either a 2d array of zeros or a single value, depending
+        # on whether the model in question is 3d or 1d, respectively.
+        for model_name in param_dict:
+            if model_name in param_x_y:
+                parameters_cube[model_name] = {x: np.zeros(shape=data_shapes[model_name][:2])
+                                               for x in param_dict[model_name]}
+            else:
+                parameters_cube[model_name] = {x: 0
+                                               for x in param_dict[model_name]}
+
+        # This loop handles the actual placement of param.values and
+        # param.units into the parameter_cubes dictionary.
+        for label in models:
+            if " (" in label:
+                label_split = label.split(" (")
+                model_name = label_split[0]
+
+                # If the get_models method is used to build a dictionary of
+                # models and a value is set for the x or y parameters, that
+                # will mean that only one x or y value is present in the
+                # models.
+                if len(param_x_y[model_name]['x']) == 1:
+                    x = 0
+                else:
+                    x = int(label_split[1].split(", ")[0])
+
+                if len(param_x_y[model_name]['y']) == 1:
+                    y = 0
+                else:
+                    y = int(label_split[1].split(", ")[1][:-1])
+
+                param_units[model_name] = {}
+
+                for name in param_dict[model_name]:
+                    param = getattr(models[label], name)
+                    parameters_cube[model_name][name][x][y] = param.value
+                    param_units[model_name][name] = param.unit
+            else:
+                model_name = label
+                param_units[model_name] = {}
+
+                # 1d models do not have anything set of param.unit, so the
+                # return_units and input_units properties need to be used
+                # instead, depending on the type of parameter `name` is.
+                for name in param_dict[model_name]:
+                    param = getattr(models[label], name)
+                    parameters_cube[model_name][name] = param.value
+                    param_units[model_name][name] = param.unit
+
+        # Convert values of parameters_cube[key][param_name] into u.Quantity
+        # objects that contain the appropriate unit set in
+        # param_units[key][param_name]
+        for key in parameters_cube:
+            for param_name in parameters_cube[key]:
+                parameters_cube[key][param_name] = u.Quantity(
+                    parameters_cube[key][param_name],
+                    param_units[key].get(param_name, None))
+
+        return parameters_cube
+
+    def get_model_parameters(self, model_label=None, x=None, y=None):
+        """
+        Convert each parameter of model inside models into a coordinate that
+        maps the model name and parameter name to a `astropy.units.Quantity`
+        object.
+
+        Parameters
+        ----------
+        model_label : str
+            Get model parameters for a particular model by inputting its label.
+        x : int
+            The x coordinate of the model spaxels that will be returned from
+            get_models.
+        y : int
+            The y coordinate of the model spaxels that will be returned from
+            get_models.
+
+        Returns
+        -------
+        :dict: a dictionary of the form
+            {model name: {parameter name: [[`astropy.units.Quantity`]]}}
+            for 3d models or
+            {model name: {parameter name: `astropy.units.Quantity`}} where the
+            Quantity object represents the parameter value and unit of one of
+            spaxel models or the 1d models, respectively.
+        """
+        return self._get_model_parameters(models=None, model_label=model_label,
+                                          x=x, y=y)
+
     def vue_add_model(self, event):
         self.create_model_component()
 
@@ -905,15 +1219,19 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             return
         models_to_fit = self._reinitialize_with_fixed()
 
-        masked_spectrum = self._apply_subset_masks(self.dataset.selected_spectrum,
+        spec = self.dataset.get_selected_spectrum(use_display_units=True)
+
+        masked_spectrum = self._apply_subset_masks(spec,
                                                    self.spectral_subset)
+
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
                 masked_spectrum,
                 models_to_fit,
                 self.model_equation,
                 run_fitter=True,
-                window=None
+                window=None,
+                n_cpu=self.parallel_n_cpu,
             )
         except AttributeError:
             msg = SnackbarMessage("Unable to fit: model equation may be invalid",
@@ -929,7 +1247,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self._fitted_spectrum = fitted_spectrum
 
         if add_data:
-            self.app.fitted_models[self.results_label] = fitted_model
+            self._fitted_models[self.results_label] = fitted_model
             self.add_results.add_results_from_plugin(fitted_spectrum)
 
             if self.residuals_calculate:
@@ -979,14 +1297,17 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         if "_orig_spec" in data.meta:
             spec = data.meta["_orig_spec"]
         else:
-            spec = data.get_object(cls=Spectrum1D, statistic=None)
+            spec = data.get_object(cls=Spectrum, statistic=None)
+
+        spatial_axes = [0, 1, 2]
+        spatial_axes.remove(spec.spectral_axis_index)
 
         sb_unit = self.app._get_display_unit('sb')
         if spec.flux.unit != sb_unit:
             # ensure specutils has access to jdaviz custom unit equivalencies
             pixar_sr = spec.meta.get('_pixel_scale_factor', None)
             equivalencies = all_flux_unit_conversion_equivs(pixar_sr=pixar_sr,
-                                                            cube_wave=spec.spectral_axis)
+                                                            cube_wave=spec.spectral_axis.mean())
 
             pix2_in_flux = 'pix2' in spec.flux.unit.to_string()
             pix2_in_sb = 'pix2' in sb_unit
@@ -1010,7 +1331,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         models_to_fit = self._reinitialize_with_fixed()
 
         # Apply masks from selected spectral subset
-        spec = self._apply_subset_masks(spec, self.spectral_subset)
+        spec = self._apply_subset_masks(spec, self.spectral_subset, spatial_axes=spatial_axes)
         # Also mask out NaNs for fitting. Simply adding filter_non_finite to the cube fit
         # didn't work out of the box, so doing this for now.
         if spec.mask is None:
@@ -1024,7 +1345,8 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                 models_to_fit,
                 self.model_equation,
                 run_fitter=True,
-                window=None
+                window=None,
+                n_cpu=self.parallel_n_cpu
             )
         except ValueError:
             snackbar_message = SnackbarMessage(
@@ -1038,9 +1360,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         if add_data:
             for m in fitted_model:
                 temp_label = "{} ({}, {})".format(self.results_label, m["x"], m["y"])
-                self.app.fitted_models[temp_label] = m["model"]
+                self._fitted_models[temp_label] = m["model"]
 
-        output_cube = Spectrum1D(flux=fitted_spectrum.flux, wcs=fitted_spectrum.wcs)
+        output_cube = Spectrum(flux=fitted_spectrum.flux, wcs=fitted_spectrum.wcs)
 
         selected_spec = self.dataset.selected_obj
         if '_pixel_scale_factor' in selected_spec.meta:
@@ -1058,7 +1380,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         return fitted_model, output_cube
 
-    def _apply_subset_masks(self, spectrum, subset_component):
+    def _apply_subset_masks(self, spectrum, subset_component, spatial_axes=None):
         """
         For a spectrum/spectral cube ``spectrum``, add a mask attribute
         if none exists. Mask excludes non-selected spectral subsets.
@@ -1076,13 +1398,15 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                     # if subset mask is 3D and the `spectrum` mask is 1D, which
                     # happens when `spectrum` has been collapsed from 3D->1D,
                     # then also collapse the 3D mask in the spatial
-                    # dimensions (0, 1) so that slices in the spectral axis that
+                    # dimensions so that slices in the spectral axis that
                     # are masked in all pixels become masked in the spectral subset:
-                    subset_mask = np.all(subset_mask, axis=(0, 1))
+                    subset_mask = np.all(subset_mask, axis=spatial_axes)
             spectrum.mask |= subset_mask
         else:
             if subset_mask.ndim < spectrum.flux.ndim:
                 # correct the shape of spectral/spatial axes when they're different:
+                if spectrum.spectral_axis_index == 0:
+                    subset_mask = np.expand_dims(subset_mask, (1, 2))
                 subset_mask = np.broadcast_to(subset_mask, spectrum.flux.shape)
 
             elif (subset_mask.ndim == spectrum.flux.ndim and
