@@ -1,5 +1,4 @@
 import warnings
-
 import numpy as np
 import pytest
 from astropy import units as u
@@ -10,7 +9,9 @@ from astropy.tests.helper import assert_quantity_allclose
 from astropy.wcs import WCS
 from glue.core.roi import XRangeROI
 from numpy.testing import assert_allclose, assert_array_equal
-from specutils.spectra import Spectrum1D
+from specutils.spectra import Spectrum
+from specutils import SpectralRegion
+from traitlets import TraitError
 
 from jdaviz.configs.default.plugins.model_fitting import fitting_backend as fb
 from jdaviz.configs.default.plugins.model_fitting import initializers
@@ -57,8 +58,99 @@ def test_model_params():
         assert np.all([p in expected_params for p in params])
 
 
+def test_check_poly_order_function(specviz_helper, spectrum1d):
+    specviz_helper.load_data(spectrum1d)
+    plugin = specviz_helper.plugins["Model Fitting"]._obj
+    plugin.dataset_selected = '1D Spectrum'
+
+    result = plugin._check_poly_order()
+    assert result is None
+
+    result = plugin._check_poly_order(model_comp='fake')
+    assert result is None
+
+    result = plugin._check_poly_order(poly_order=-1.5)
+    assert plugin.poly_order_invalid_msg == ""
+    assert result is None
+
+    result = plugin._check_poly_order(model_comp='fake', poly_order=-1.5)
+    assert plugin.poly_order_invalid_msg == ""
+    assert result is None
+
+    poly_order_default = 0
+    assert plugin.poly_order == poly_order_default  # stays default
+
+    plugin.poly_order = poly_order_default
+    plugin.model_comp_selected = 'Polynomial1D'
+
+    result = plugin._check_poly_order()
+    assert result == poly_order_default
+
+    result = plugin._check_poly_order(model_comp='Polynomial1D')
+    assert result == poly_order_default
+
+    result = plugin._check_poly_order(poly_order=4)
+    assert result == 4
+    # doesn't change actual value of plugin.poly_order
+    assert plugin.poly_order == poly_order_default
+
+    result = plugin._check_poly_order(model_comp='Polynomial1D', poly_order=5)
+    assert result == 5
+    # doesn't change actual value of plugin.poly_order
+    assert plugin.poly_order == poly_order_default
+
+    for poly_order in [-2, -1, 2.5]:
+        result = plugin._check_poly_order(poly_order=poly_order)
+        assert plugin.poly_order_invalid_msg == "Order must be an integer >= 0"
+        assert result is None
+
+
+def test_check_poly_order_observer(specviz_helper, spectrum1d):
+    specviz_helper.load_data(spectrum1d)
+    plugin = specviz_helper.plugins["Model Fitting"]._obj
+    plugin.dataset_selected = '1D Spectrum'
+
+    # poly_order default
+    assert plugin.poly_order == 0
+    # No model components added yet
+    assert len(plugin.model_components) == 0
+
+    # Make sure that the observer doesn't interfere with the other models
+    non_poly_model_comps = [m for m in initializers.MODELS.keys() if m != "Polynomial1D"]
+    for i, model_comp in enumerate(non_poly_model_comps):
+        plugin.model_comp_selected = model_comp
+        # Run the observer by setting poly_order
+        plugin.poly_order -= 1
+        plugin.vue_add_model({})
+        # Check that the model component exists (i.e. was added successfully)
+        assert len(plugin.model_components) == i + 1
+        assert plugin.model_components[i]
+
+    plugin.model_comp_selected = "Polynomial1D"
+
+    vue_err_msg = "Order must be an integer >= 0"
+    err_msg = f"poly_{vue_err_msg.lower()}"
+    with pytest.raises(ValueError, match=err_msg):
+        plugin.vue_add_model({})
+
+    for poly_order in range(-3, 0):
+        plugin.poly_order = poly_order
+        assert plugin.poly_order_invalid_msg == vue_err_msg
+        with pytest.raises(ValueError, match=err_msg):
+            plugin.vue_add_model({})
+
+
+    with pytest.raises(TraitError, match="The 'poly_order' trait of a ModelFitting instance expected an int, not the float 2.5."): # noqa
+        plugin.poly_order = 2.5
+
+    plugin.poly_order = 3
+    plugin.model_comp_selected = "Polynomial1D"
+    plugin.vue_add_model({})
+    assert plugin.model_components[-1] == 'P3'
+
+
 def test_model_ids(cubeviz_helper, spectral_cube_wcs):
-    cubeviz_helper.load_data(Spectrum1D(flux=np.ones((3, 4, 5)) * u.nJy, wcs=spectral_cube_wcs),
+    cubeviz_helper.load_data(Spectrum(flux=np.ones((3, 4, 5)) * u.nJy, wcs=spectral_cube_wcs),
                              data_label='test')
     plugin = cubeviz_helper.plugins["Model Fitting"]._obj
     plugin.dataset_selected = 'Spectrum (sum)'
@@ -86,7 +178,7 @@ def test_parameter_retrieval(cubeviz_helper, spectral_cube_wcs):
 
     flux = np.ones((3, 4, 5))
     flux[2, 2, :] = [1, 2, 3, 4, 5]
-    cubeviz_helper.load_data(Spectrum1D(flux=flux * flux_unit, wcs=spectral_cube_wcs),
+    cubeviz_helper.load_data(Spectrum(flux=flux * flux_unit, wcs=spectral_cube_wcs),
                              data_label='test')
 
     plugin = cubeviz_helper.plugins["Model Fitting"]
@@ -100,11 +192,14 @@ def test_parameter_retrieval(cubeviz_helper, spectral_cube_wcs):
     assert cubeviz_helper.app._get_display_unit('sb') == sb_unit
 
     plugin.create_model_component("Linear1D", "L")
+    # NOTE: Hardcoding n_cpu=1 to run in serial, it's slower to spool up
+    # multiprocessing for the size of the cube
+    plugin._obj.parallel_n_cpu = 1
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         plugin.calculate_fit()
 
-    params = cubeviz_helper.get_model_parameters()
+    params = cubeviz_helper.plugins['Model Fitting'].get_model_parameters()
     slope_res = np.zeros((3, 4))
     slope_res[2, 2] = 1.0
 
@@ -118,8 +213,13 @@ def test_parameter_retrieval(cubeviz_helper, spectral_cube_wcs):
                              atol=1e-10 * sb_unit)
 
 
-@pytest.mark.parametrize('unc', ('zeros', None))
-def test_fitting_backend(unc):
+@pytest.mark.parametrize(
+    ('n_cpu', 'unc'), [
+        (1, "zeros"),
+        (1, None),
+        (None, "zeros"),
+        (None, None),])
+def test_fitting_backend(n_cpu, unc):
     np.random.seed(42)
 
     x, y = build_spectrum()
@@ -129,7 +229,7 @@ def test_fitting_backend(unc):
         uncertainties = StdDevUncertainty(np.zeros(y.shape)*u.Jy)
     elif unc is None:
         uncertainties = None
-    spectrum = Spectrum1D(flux=y*u.Jy, spectral_axis=x*u.um, uncertainty=uncertainties)
+    spectrum = Spectrum(flux=y*u.Jy, spectral_axis=x*u.um, uncertainty=uncertainties)
 
     g1f = models.Gaussian1D(0.7*u.Jy, 4.65*u.um, 0.3*u.um, name='g1')
     g2f = models.Gaussian1D(2.0*u.Jy, 5.55*u.um, 0.3*u.um, name='g2')
@@ -141,7 +241,7 @@ def test_fitting_backend(unc):
 
     # Returns the initial model
     fm, fitted_spectrum = fb.fit_model_to_spectrum(spectrum, model_list, expression,
-                                                   run_fitter=False)
+                                                   run_fitter=False, n_cpu=n_cpu)
 
     parameters_expected = np.array([0.7, 4.65, 0.3, 2., 5.55, 0.3, -2.,
                                     8.15, 0.2, 1.])
@@ -149,7 +249,7 @@ def test_fitting_backend(unc):
 
     # Returns the fitted model
     fm, fitted_spectrum = fb.fit_model_to_spectrum(spectrum, model_list, expression,
-                                                   run_fitter=True)
+                                                   run_fitter=True, n_cpu=n_cpu)
 
     parameters_expected = np.array([1.0104705, 4.58956282, 0.19590464, 2.39892026,
                                     5.49867754, 0.10834472, -1.66902953, 8.19714439,
@@ -157,8 +257,14 @@ def test_fitting_backend(unc):
     assert_allclose(fm.parameters, parameters_expected, atol=1e-5)
 
 
-@pytest.mark.parametrize('unc', ('zeros', None))
-def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
+# For coverage of serial vs multiprocessing and unc
+@pytest.mark.parametrize(
+    ('n_cpu', 'unc'), [
+        (1, "zeros"),
+        (1, None),
+        (None, "zeros"),
+        (None, None),])
+def test_cube_fitting_backend(cubeviz_helper, unc, n_cpu, tmp_path):
     np.random.seed(42)
 
     SIGMA = 0.1  # noise in data
@@ -166,7 +272,7 @@ def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
     IMAGE_SIZE_X = 15
     IMAGE_SIZE_Y = 14
 
-    # Flux cube oriented as in JWST data. To build a Spectrum1D
+    # Flux cube oriented as in JWST data. To build a Spectrum
     # instance with this, one need to transpose it so the spectral
     # axis direction corresponds to the last index.
     flux_cube = np.zeros((SPECTRUM_SIZE, IMAGE_SIZE_X, IMAGE_SIZE_Y))
@@ -181,7 +287,7 @@ def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
     for spx in spaxels:
         flux_cube[:, spx[0], spx[1]] = build_spectrum(sigma=SIGMA)[1]
 
-    # Transpose so it can be packed in a Spectrum1D instance.
+    # Transpose so it can be packed in a Spectrum instance.
     flux_cube = flux_cube.transpose(1, 2, 0)  # (15, 14, 200)
     cube_wcs = WCS({
         'WCSAXES': 3, 'RADESYS': 'ICRS', 'EQUINOX': 2000.0,
@@ -201,8 +307,8 @@ def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
     elif unc is None:
         uncertainties = None
 
-    spectrum = Spectrum1D(flux=flux_cube*u.Jy, wcs=cube_wcs,
-                          uncertainty=uncertainties, mask=mask)
+    spectrum = Spectrum(flux=flux_cube*u.Jy, wcs=cube_wcs,
+                        uncertainty=uncertainties, mask=mask)
 
     # Initial model for fit.
     g1f = models.Gaussian1D(0.7*u.Jy, 4.65*u.um, 0.3*u.um, name='g1')
@@ -213,9 +319,6 @@ def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
     model_list = [g1f, g2f, g3f, zero_level]
     expression = "g1 + g2 + g3 + const1d"
 
-    n_cpu = None
-    # n_cpu = 1  # NOTE: UNCOMMENT TO DEBUG LOCALLY, AS NEEDED
-
     # Fit to all spaxels.
     with warnings.catch_warnings():
         warnings.filterwarnings("ignore", message="The fit may be unsuccessful.*")
@@ -238,7 +341,7 @@ def test_cube_fitting_backend(cubeviz_helper, unc, tmp_path):
     assert fitted_model[0].mean.unit == u.um
 
     # Check that spectrum result is formatted as expected.
-    assert isinstance(fitted_spectrum, Spectrum1D)
+    assert isinstance(fitted_spectrum, Spectrum)
     assert len(fitted_spectrum.shape) == 3
     assert fitted_spectrum.shape == (IMAGE_SIZE_X, IMAGE_SIZE_Y, SPECTRUM_SIZE)
 
@@ -312,7 +415,10 @@ def test_results_table(specviz_helper, spectrum1d):
     mf = specviz_helper.plugins['Model Fitting']
     mf.create_model_component('Linear1D')
 
+    uc = specviz_helper.plugins['Unit Conversion']
+
     mf.add_results.label = 'linear model'
+    mf._obj.parallel_n_cpu = 1
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         mf.calculate_fit(add_data=True)
@@ -325,6 +431,9 @@ def test_results_table(specviz_helper, spectrum1d):
                                  'L:intercept_0', 'L:intercept_0:unit',
                                  'L:intercept_0:fixed', 'L:intercept_0:std']
 
+    # verify units in table match the current display unit
+    assert mf_table['L:intercept_0:unit'][-1] == uc.flux_unit
+
     mf.create_model_component('Gaussian1D')
     mf.add_results.label = 'composite model'
     with warnings.catch_warnings():
@@ -346,6 +455,19 @@ def test_results_table(specviz_helper, spectrum1d):
                                  'G:stddev_1', 'G:stddev_1:unit',
                                  'G:stddev_1:fixed', 'G:stddev_1:std']
 
+    mf.remove_model_component('G')
+    assert len(mf_table) == 2
+
+    # verify Spectrum model fitting plugin and table can handle spectral density conversions
+    uc.flux_unit = 'erg / (Angstrom s cm2)'
+    mf.reestimate_model_parameters()
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
+        mf.calculate_fit(add_data=True)
+
+    assert mf_table['L:intercept_0:unit'][-1] == uc.flux_unit
+
 
 def test_equation_validation(specviz_helper, spectrum1d):
     data_label = 'test'
@@ -382,6 +504,7 @@ def test_incompatible_units(specviz_helper, spectrum1d):
     mf.create_model_component('Linear1D')
 
     mf.add_results.label = 'model native units'
+    mf._obj.parallel_n_cpu = 1
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         mf.calculate_fit(add_data=True)
@@ -405,12 +528,14 @@ def test_incompatible_units(specviz_helper, spectrum1d):
 def test_cube_fit_with_nans(cubeviz_helper):
     flux = np.ones((7, 8, 9)) * u.nJy
     flux[:, :, 0] = np.nan
-    spec = Spectrum1D(flux=flux)
+    spec = Spectrum(flux=flux, spectral_axis_index=2)
     cubeviz_helper.load_data(spec, data_label="test")
 
     mf = cubeviz_helper.plugins["Model Fitting"]
     mf.cube_fit = True
     mf.create_model_component("Const1D")
+    mf._obj.parallel_n_cpu = 1
+
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         mf.calculate_fit()
@@ -426,7 +551,7 @@ def test_cube_fit_with_subset_and_nans(cubeviz_helper):
     # Also test with existing mask
     flux = np.ones((7, 8, 9)) * u.nJy
     flux[:, :, 0] = np.nan
-    spec = Spectrum1D(flux=flux)
+    spec = Spectrum(flux=flux, spectral_axis_index=2)
     spec.flux[5, 5, 7] = 10 * u.nJy
     cubeviz_helper.load_data(spec, data_label="test")
 
@@ -437,6 +562,8 @@ def test_cube_fit_with_subset_and_nans(cubeviz_helper):
     mf.cube_fit = True
     mf.spectral_subset = 'Subset 1'
     mf.create_model_component("Const1D")
+    mf._obj.parallel_n_cpu = 1
+
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         mf.calculate_fit()
@@ -448,14 +575,15 @@ def test_fit_with_count_units(cubeviz_helper):
     flux = np.random.random((7, 8, 9)) * u.count
     spectral_axis = np.linspace(4000, 5000, flux.shape[-1]) * u.AA
 
-    spec = Spectrum1D(flux=flux, spectral_axis=spectral_axis)
+    spec = Spectrum(flux=flux, spectral_axis=spectral_axis, spectral_axis_index=2)
     cubeviz_helper.load_data(spec, data_label="test")
 
     mf = cubeviz_helper.plugins["Model Fitting"]
     mf.cube_fit = True
     mf.create_model_component("Const1D")
+    mf._obj.parallel_n_cpu = 1
 
-    # ensures specutils.Spectrum1D.with_flux_unit has access to Jdaviz custom equivalencies for
+    # ensures specutils.Spectrum.with_flux_unit has access to Jdaviz custom equivalencies for
     # PIX^2 unit
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
@@ -480,22 +608,26 @@ def test_cube_fit_after_unit_change(cubeviz_helper, solid_angle_unit):
     mf.cube_fit = True
 
     mf.create_model_component("Const1D")
+    # ensure parameters amplitude unit matches display unit
+    assert mf.get_model_component('C').get('parameters').get('amplitude').get('unit') == f'MJy / {solid_angle_string}'  # noqa
     # Check that the parameter is using the current units when initialized
     assert mf._obj.component_models[0]['parameters'][0]['unit'] == f'MJy / {solid_angle_string}'
 
+    mf._obj.parallel_n_cpu = 1
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
         mf.calculate_fit()
 
+    # It was easier to transpose this for new data shape than rewrite it
     expected_result_slice = np.array([[9.00e-05, 9.50e-05, 1.00e-04, 1.05e-04],
                                       [9.10e-05, 9.60e-05, 1.01e-04, 1.06e-04],
                                       [9.20e-05, 9.70e-05, 1.02e-04, 1.07e-04],
                                       [9.30e-05, 9.80e-05, 1.03e-04, 1.08e-04],
-                                      [9.40e-05, 9.90e-05, 1.04e-04, 1.09e-04]])
+                                      [9.40e-05, 9.90e-05, 1.04e-04, 1.09e-04]]).T
 
     model_flux = cubeviz_helper.app.data_collection[-1].get_component('flux')
     assert model_flux.units == f'MJy / {solid_angle_string}'
-    assert np.allclose(model_flux.data[:, :, 1], expected_result_slice)
+    assert np.allclose(model_flux.data[1, :, :], expected_result_slice)
 
     # Switch back to Jy, see that the component didn't change but the output does
     uc.flux_unit = 'Jy'
@@ -506,7 +638,7 @@ def test_cube_fit_after_unit_change(cubeviz_helper, solid_angle_unit):
 
     model_flux = cubeviz_helper.app.data_collection[-1].get_component('flux')
     assert model_flux.units == f'Jy / {solid_angle_string}'
-    assert np.allclose(model_flux.data[:, :, 1], expected_result_slice * 1e6)
+    assert np.allclose(model_flux.data[1, :, :], expected_result_slice * 1e6)
 
     # ensure conversions that require the spectral axis/translations are handled by the plugin
     uc.spectral_y_type = 'Surface Brightness'
@@ -516,12 +648,14 @@ def test_cube_fit_after_unit_change(cubeviz_helper, solid_angle_unit):
 
     if solid_angle_string == 'sr':
         expected_unit_string = f'erg / (Angstrom s {solid_angle_string} cm2)'
+        assert mf.get_model_component('C').get('parameters').get('amplitude').get('unit') == expected_unit_string  # noqa
     else:
         expected_unit_string = f'erg / (Angstrom s cm2 {solid_angle_string})'
+        assert mf.get_model_component('C').get('parameters').get('amplitude').get('unit') == expected_unit_string  # noqa
 
     assert mf._obj.component_models[0]['parameters'][0]['unit'] == expected_unit_string
 
-    # running this ensures specutils.Spectrum1D.with_flux_unit has Jdaviz custom equivalencies
+    # running this ensures specutils.Spectrum.with_flux_unit has Jdaviz custom equivalencies
     # for spectral axis conversions and scale factor translations
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
@@ -529,3 +663,25 @@ def test_cube_fit_after_unit_change(cubeviz_helper, solid_angle_unit):
 
     model_flux = cubeviz_helper.app.data_collection[-1].get_component('flux')
     assert model_flux.units == expected_unit_string
+
+
+def test_deconf_mf_with_subset(deconfigged_helper):
+    flux = np.ones(9) * u.Jy
+    flux[7] = 10 * u.Jy
+    wavelength = np.arange(9) * u.um
+    spec = Spectrum(flux=flux, spectral_axis=wavelength)
+    deconfigged_helper.load(spec, data_label="1D Spectrum", format='1D Spectrum')
+
+    subset = deconfigged_helper.plugins['Subset Tools']
+    subset.import_region(SpectralRegion(6 * u.Unit('um'), 7 * u.Unit('um')))
+
+    mf = deconfigged_helper.plugins['Model Fitting']
+    # ensure deconfigged app can access subsets in plugin
+    mf.spectral_subset.selected = 'Subset 1'
+    mf.add_results.label = 'linear model'
+    mf._obj.parallel_n_cpu = 1
+    mf.create_model_component()
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', message='Model is linear in parameters.*')
+        mf.calculate_fit()
