@@ -1,41 +1,42 @@
-from echo import delay_callback
+from echo import delay_callback, CallbackProperty
 
 import numpy as np
 
 from glue.config import data_translator
 from glue.core import BaseData
 from glue.core.exceptions import IncompatibleAttribute
-from glue.core.units import UnitConverter
 from glue.core.subset import Subset
 from glue.core.subset_group import GroupedSubset
 from glue.viewers.scatter.state import ScatterLayerState as BqplotScatterLayerState
+from glue.utils import avoid_circular
 
 from glue_astronomy.spectral_coordinates import SpectralCoordinates
 from glue_jupyter.bqplot.profile import BqplotProfileView
 from glue_jupyter.bqplot.image import BqplotImageView
 from glue_jupyter.table import TableViewer
 
+from astropy.utils import deprecated
 from astropy import units as u
 from astropy.nddata import (
     NDDataArray, StdDevUncertainty, VarianceUncertainty, InverseVariance
 )
-from specutils import Spectrum1D
+from specutils import Spectrum
 
 from jdaviz.components.toolbar_nested import NestedJupyterToolbar
 from jdaviz.configs.default.plugins.data_menu import DataMenu
 from jdaviz.core.astrowidgets_api import AstrowidgetsImageViewerMixin
 from jdaviz.core.custom_units_and_equivs import _eqv_sb_per_pixel_to_per_angle
-from jdaviz.core.events import SnackbarMessage
+from jdaviz.core.events import SnackbarMessage, NewViewerMessage, ViewerVisibleLayersChangedMessage
 from jdaviz.core.freezable_state import FreezableProfileViewerState
 from jdaviz.core.marks import LineUncertainties, ScatterMask, OffscreenLinesMarks
 from jdaviz.core.registries import viewer_registry
 from jdaviz.core.template_mixin import WithCache
 from jdaviz.core.user_api import ViewerUserApi
-from jdaviz.core.unit_conversion_utils import check_if_unit_is_per_solid_angle
+from jdaviz.core.unit_conversion_utils import (check_if_unit_is_per_solid_angle,
+                                               flux_conversion_general,
+                                               all_flux_unit_conversion_equivs)
 from jdaviz.utils import (ColorCycler, get_subset_type, _wcs_only_label,
-                          layer_is_image_data, layer_is_not_dq)
-
-uc = UnitConverter()
+                          layer_is_image_data, layer_is_not_dq, layer_is_3d)
 
 uncertainty_str_to_cls_mapping = {
     "std": StdDevUncertainty,
@@ -70,12 +71,13 @@ class JdavizViewerMixin(WithCache):
     def user_api(self):
         # default exposed user APIs.  Can override this method in any particular viewer.
         if not isinstance(self, TableViewer):
-            # TODO: eventually remove data_labels, data_labels_loaded,
+            # TODO: eventually remove data_labels_loaded
             # and data_labels_visible once deprecation period passes
-            # TODO: add data_menu once API is finalized and ready to be made public
-            expose = ['data_labels', 'data_labels_loaded', 'data_labels_visible']
+            expose = ['data_labels_loaded', 'data_labels_visible', 'data_menu']
         else:
             expose = []
+        if self.jdaviz_app.config == 'deconfigged':
+            expose += ['clone_viewer']
         if isinstance(self, BqplotImageView):
             if isinstance(self, AstrowidgetsImageViewerMixin):
                 expose += ['save',
@@ -92,15 +94,20 @@ class JdavizViewerMixin(WithCache):
         elif isinstance(self, TableViewer):
             expose += []
         else:
-            expose += ['set_limits', 'reset_limits']
+            expose += ['set_limits', 'reset_limits', 'set_tick_format']
         return ViewerUserApi(self, expose=expose)
 
     @property
     def data_menu(self):
         return self._data_menu.user_api
 
+    def _deprecated_data_menu(self):
+        # temporary method to allow for opening new data-menu from old button.  This should
+        # be removed anytime after the old button is removed (likely in 4.3)
+        self.data_menu.open_menu()
+
     @property
-    # TODO: deprecate in favor of viewer.data_menu.layers_loaded
+    @deprecated(since="4.1", alternative="viewer.data_menu.data_labels_loaded")
     def data_labels_loaded(self):
         """
         List of data labels loaded in this viewer.
@@ -110,13 +117,10 @@ class JdavizViewerMixin(WithCache):
         data_labels : list
             list of strings
         """
-        viewer_item = self.jdaviz_app._get_viewer_item(self.reference_id)
-        return [self.jdaviz_app._get_data_item_by_id(data_id)['name']
-                for data_id in viewer_item.get('selected_data_items', {}).keys()
-                if self.jdaviz_app._get_data_item_by_id(data_id) is not None]
+        return self.data_menu.data_labels_loaded
 
     @property
-    # TODO: deprecate in favor of viewer.data_menu.layers_visible
+    @deprecated(since="4.1", alternative="viewer.data_menu.data_labels_visible")
     def data_labels_visible(self):
         """
         List of data labels visible in this viewer.
@@ -126,10 +130,56 @@ class JdavizViewerMixin(WithCache):
         data_labels : list
             list of strings
         """
-        viewer_item = self.jdaviz_app._get_viewer_item(self.reference_id)
-        return [self.jdaviz_app._get_data_item_by_id(data_id)['name']
-                for data_id, visibility in viewer_item.get('selected_data_items', {}).items()
-                if visibility != 'hidden']
+        return self.data_menu.data_labels_visible
+
+    def _get_clone_viewer_reference(self):
+        return self.jdaviz_helper._get_clone_viewer_reference(self.reference)
+
+    def _clone_viewer_outside_app(self):
+        new_viewer = type(self)(session=self.session)
+        # TODO: this is needed for jdaviz only, without it it should also
+        # work for glue-jupyter
+        new_viewer._reference_id = self.reference_id + "_TODO_IS_THIS_OK?"
+        d = self.state.as_dict()
+        new_viewer.state.update_from_dict(d)
+
+        for layer in self.layers:
+            layer_state = layer.state
+            new_layer = type(layer)(view=new_viewer, viewer_state=new_viewer.state,
+                                    layer_state=layer_state)
+            new_layer.update()
+        return new_viewer
+
+    def clone_viewer(self):
+        name = self.jdaviz_helper._get_clone_viewer_reference(self.reference)
+
+        self.jdaviz_app._on_new_viewer(NewViewerMessage(self.__class__,
+                                                        data=None,
+                                                        sender=self.jdaviz_app),
+                                       vid=name, name=name)
+
+        new_viewer = self.jdaviz_app.get_viewer(name)
+
+        visible_layers = self.data_menu.data_labels_visible
+        for layer in self.data_menu.data_labels_loaded[::-1]:
+            visible = layer in visible_layers
+            new_viewer.data_menu.add_data(layer)
+            new_viewer.data_menu.set_layer_visibility(layer, visible)
+            # TODO: don't revert color when adding same data to a new viewer
+
+        # allow viewers to set attributes (not in state) on cloned viewers
+        for attr in getattr(self, '_clone_attrs', []):
+            if hasattr(self, attr):
+                setattr(new_viewer, attr, getattr(self, attr))
+        new_viewer.state.update_from_dict(self.state.as_dict())
+
+        for this_layer_state, new_layer_state in zip(self.state.layers, new_viewer.state.layers):
+            for k, v in this_layer_state.as_dict().items():
+                if k in ('layer',):
+                    continue
+                setattr(new_layer_state, k, v)
+
+        return new_viewer.user_api
 
     def reset_limits(self):
         """
@@ -181,6 +231,24 @@ class JdavizViewerMixin(WithCache):
         """
         return self.state.x_min, self.state.x_max, self.state.y_min, self.state.y_max
 
+    def set_tick_format(self, fmt, axis):
+        """
+        Manually set the tick format of one of the axes.
+
+        Parameters
+        ----------
+        fmt : str
+            Format of tick marks.
+            For example, ``'0.1e'`` to set scientific notation or ``'0.2f'`` to turn it off.
+        axis : {x, y}
+            The viewer axis.
+        """
+        if axis not in ('x', 'y'):
+            raise ValueError("axis must be 'x' or 'y'")
+        # Examples of values for fmt are '0.1e' or '0.2f'
+        axis = {'x': 0, 'y': 1}[axis]
+        self.figure.axes[axis].tick_format = fmt
+
     @property
     def native_marks(self):
         """
@@ -219,8 +287,15 @@ class JdavizViewerMixin(WithCache):
             # whenever as_steps changes, we need to redraw the uncertainties (if enabled)
             layer_state.add_callback('as_steps', self._show_uncertainty_changed)
 
+        # use echo-validator to ensure visible sets & updates properly in plot options & data menu
+        if (hasattr(layer_state, 'visible') and get_subset_type(layer_state.layer) != 'spatial'):
+            layer_state.layer.visible = CallbackProperty()
+            layer_state.add_callback('layer',
+                                     self._expected_subset_layer_default,
+                                     validator=True)
+
     def _expected_subset_layer_default(self, layer_state):
-        if self.__class__.__name__ in ('CubevizImageView', 'RampvizImageView'):
+        if self.__class__.__name__ == 'RampvizImageView':
             # Do not override default for subsets as for some reason
             # this isn't getting called when they're first added, but rather when
             # the next state change is made (for example: manually changing the visibility)
@@ -230,6 +305,10 @@ class JdavizViewerMixin(WithCache):
         if self.__class__.__name__ == 'RampvizProfileView':
             # Rampviz doesn't show subset profiles by default:
             layer_state.visible = False
+        elif (self.__class__.__name__ == 'CubevizImageView' and
+              get_subset_type(layer_state.layer) != 'spatial'):
+            # set visibility of spectral subsets to false in Cubeviz image-viewers
+            layer_state.visible = False
         else:
             layer_state.visible = self._get_layer(layer_state.layer.data.label).visible
 
@@ -286,6 +365,7 @@ class JdavizViewerMixin(WithCache):
 
         self._data_menu.visible_layers = visible_layers
 
+    @avoid_circular
     def _on_layers_update(self, layers=None):
         if self.__class__.__name__ == 'MosvizTableViewer':
             # MosvizTableViewer uses this as a mixin, but we do not need any of this layer
@@ -330,6 +410,9 @@ class JdavizViewerMixin(WithCache):
                     self._expected_subset_layers.remove(layer.layer.label)
                 self._expected_subset_layer_default(layer)
 
+        self.hub.broadcast(ViewerVisibleLayersChangedMessage(
+            viewer_reference=self.reference, visible_layers=selected_data_items, sender=self))
+
     def _on_subset_create(self, msg):
         from jdaviz.configs.mosviz.plugins.viewers import MosvizTableViewer
         if isinstance(self, MosvizTableViewer):
@@ -376,6 +459,21 @@ class JdavizViewerMixin(WithCache):
                           if (layer.visible and
                               layer_is_image_data(layer.layer) and
                               layer_is_not_dq(layer.layer) and
+                              (getattr(layer, 'bitmap_visible', False) or
+                               getattr(layer, 'contour_visible', False)))]
+        if len(visible_layers) == 0:
+            return None
+
+        return visible_layers[-1]
+
+    @property
+    def active_cube_layer(self):
+        """Active cube layer in the viewer, if available."""
+        # Find visible layers
+        visible_layers = [layer for layer in self.state.layers
+                          if (layer.visible and
+                              layer_is_3d(layer.layer) and
+                              layer_is_not_dq(layer.layer) and
                               (layer.bitmap_visible or layer.contour_visible))]
         if len(visible_layers) == 0:
             return None
@@ -483,7 +581,7 @@ class JdavizProfileView(JdavizViewerMixin, BqplotProfileView):
                             layer_data = self.jdaviz_app._get_object_cache[cache_key]
                         else:
                             # If spectrum, collapse via the defined statistic
-                            if _class == Spectrum1D:
+                            if _class == Spectrum:
                                 layer_data = lyr.get_object(cls=_class, statistic=statistic)
                             else:
                                 layer_data = lyr.get_object(cls=_class)
@@ -566,9 +664,16 @@ class JdavizProfileView(JdavizViewerMixin, BqplotProfileView):
         else:
             # Check if the new data flux unit is actually compatible since flux not linked.
             try:
-                if self.state.y_display_unit not in ['None', None, 'DN']:
-                    uc.to_unit(data, data.find_component_id("flux"), [1, 1],
-                               u.Unit(self.state.y_display_unit))  # Error if incompatible
+                if (self.state.y_display_unit not in ['None', None, 'DN'] and
+                   hasattr(data.get_component('flux').data, 'units')):
+                    psc = data.meta.get('_pixel_scale_factor', None)
+                    cube_wave = data.get_component('spectral')
+
+                    eqv = all_flux_unit_conversion_equivs(pixar_sr=psc, cube_wave=cube_wave)
+                    flux_conversion_general([1, 1],
+                                            data.get_component('flux').data.units,
+                                            self.state.y_display_unit,
+                                            equivalencies=eqv)
             except Exception as err:
                 # Raising exception here introduces a dirty state that messes up next load_data
                 # but not raising exception also causes weird behavior unless we remove the data
@@ -635,7 +740,7 @@ class JdavizProfileView(JdavizViewerMixin, BqplotProfileView):
 
                 data_obj = lyr.data.get_object(cls=self.default_class)
 
-                if self.default_class == Spectrum1D:
+                if self.default_class == Spectrum:
                     data_x = data_obj.spectral_axis.value
                     data_y = data_obj.flux.value
                 else:
@@ -688,26 +793,34 @@ class JdavizProfileView(JdavizViewerMixin, BqplotProfileView):
                 uncert_cls = uncertainty_str_to_cls_mapping[uncertainty_type_str]
                 error = uncert_cls(error).represent_as(StdDevUncertainty).array
 
-                # Then we assume that last axis is always wavelength.
-                # This may need adjustment after the following
-                # specutils PR is merged: https://github.com/astropy/specutils/pull/1033
-                spectral_axis = -1
+                if 'spectral_axis_index' in lyr.data.meta:
+                    spectral_axis_index = lyr.data.meta['spectral_axis_index']
+                else:
+                    # We have to make an assumption in this case.
+                    # TODO: Should we have other handling for non-spectral data (e.g. light curves?)
+                    spectral_axis_index = -1
+
                 data_obj = lyr.data.get_object(cls=self.default_class, statistic=None)
 
-                if isinstance(lyr.data.coords, SpectralCoordinates):
-                    spectral_wcs = lyr.data.coords
+                lyr_coords = lyr.data.coords
+
+                if isinstance(lyr_coords, SpectralCoordinates):
+                    spectral_wcs = lyr_coords
                     data_x = spectral_wcs.pixel_to_world_values(
-                        np.arange(lyr.data.shape[spectral_axis])
+                        np.arange(lyr.data.shape[spectral_axis_index])
                     )
                     if isinstance(data_x, tuple):
                         data_x = data_x[0]
                 else:
-                    if hasattr(lyr.data.coords, 'spectral_wcs'):
-                        spectral_wcs = lyr.data.coords.spectral_wcs
-                    elif hasattr(lyr.data.coords, 'spectral'):
-                        spectral_wcs = lyr.data.coords.spectral
+                    if hasattr(lyr_coords, 'spectral_wcs'):
+                        spectral_wcs = lyr_coords.spectral_wcs
+                    elif hasattr(lyr_coords, 'spectral'):
+                        spectral_wcs = lyr_coords.spectral
+                    elif hasattr(lyr_coords, "world_n_dim") and lyr_coords.world_n_dim == 1:
+                        # 1D GWCS in this case, just use the coords
+                        spectral_wcs = lyr_coords
                     data_x = spectral_wcs.pixel_to_world(
-                        np.arange(lyr.data.shape[spectral_axis])
+                        np.arange(lyr.data.shape[spectral_axis_index])
                     )
 
                 data_y = data_obj.data
