@@ -12,13 +12,14 @@ from glue.core.message import SubsetUpdateMessage
 from ipywidgets import widget_serialization
 from photutils.aperture import (ApertureStats, CircularAperture, EllipticalAperture,
                                 RectangularAperture)
+from photutils.profiles import CurveOfGrowth, RadialProfile
 from traitlets import Any, Bool, Integer, List, Unicode, observe
 
 from jdaviz.core.custom_traitlets import FloatHandleEmpty
 from jdaviz.core.custom_units_and_equivs import PIX2
 from jdaviz.core.events import (GlobalDisplayUnitChanged, SnackbarMessage,
                                 LinkUpdatedMessage, SliceValueUpdatedMessage)
-from jdaviz.core.region_translators import regions2aperture, _get_region_from_spatial_subset
+from jdaviz.core.region_translators import regions2aperture
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin, DatasetMultiSelectMixin,
                                         SubsetSelect, ApertureSubsetSelectMixin,
@@ -27,12 +28,14 @@ from jdaviz.core.unit_conversion_utils import (all_flux_unit_conversion_equivs,
                                                check_if_unit_is_per_solid_angle,
                                                flux_conversion_general,
                                                handle_squared_flux_unit_conversions)
+from jdaviz.core.user_api import PluginUserApi
 from jdaviz.utils import PRIHDR_KEY
 
 __all__ = ['SimpleAperturePhotometry']
 
 
-@tray_registry('imviz-aper-phot-simple', label="Aperture Photometry")
+@tray_registry('imviz-aper-phot-simple', label="Aperture Photometry",
+               category="data:analysis")
 class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                                DatasetMultiSelectMixin, TableMixin, PlotMixin, MultiselectMixin):
     """
@@ -42,9 +45,37 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
     Only the following attributes and methods are available through the
     :ref:`public plugin API <plugin-apis>`:
 
-    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
-    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
+    * :meth:`~jdaviz.core.template_mixin.TableMixin.clear_table`
+    * :meth:`~jdaviz.core.template_mixin.TableMixin.export_table`
+    * :meth:`calculate_batch_photometry`
+    * :meth:`calculate_photometry`
+    * :meth:`fitted_models`
+    * :meth:`fit_radial_profile`
+    * :meth:`unpack_batch_options`
+    * ``aperture`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
+    * ``background`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
+    * ``background_value``
+      Fixed value to use as background level.
+    * ``counts_factor``
+      Factor to convert data unit to counts, in unit of flux/counts.
+    * ``current_plot_type``
+      Choice of Curve of Growth, Radial Profile, or Radial Profile (Raw).
+      Only applicable when ``multiselect=False``.
+    * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
+    * ``flux_scaling``
+      Flux scaling factor for calculation of magnitudes in output table.
+    * ``multiselect``
+      Enable multiselect mode to select multiple datasets for aperture photometry.
+    * ``pixel_area``
+        In arcsec squared, only used if data is in units of surface brightness.
+    * ``plot`` (:class:`~jdaviz.core.template_mixin.Plot`):
+        Plot, based on selection of ``current_plot_type``. Only applicable when
+        ``multiselect=False``
+    * ``table`` (:class:`~jdaviz.core.template_mixin.Table`):
+        Table with photometry results.
     """
     template_file = __file__, "aper_phot_simple.vue"
     uses_active_status = Bool(True).tag(sync=True)
@@ -91,6 +122,8 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
         # description displayed under plugin title in tray
         self._plugin_description = 'Perform aperture photometry for drawn regions.'
 
+        self.dataset.add_filter('is_cube_or_image')
+
         self.background = SubsetSelect(self,
                                        'background_items',
                                        'background_selected',
@@ -112,6 +145,7 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
 
         self.plot_types = ["Curve of Growth", "Radial Profile", "Radial Profile (Raw)"]
         self.current_plot_type = self.plot_types[0]
+        self._fitted_models = {}
         self._fitted_model_name = 'phot_radial_profile'
 
         # override default plot styling
@@ -150,14 +184,23 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
             self.hub.subscribe(self, GlobalDisplayUnitChanged,
                                handler=self._on_display_units_changed)
 
-# TODO: expose public API once finalized
-#    @property
-#    def user_api(self):
-#        return PluginUserApi(self, expose=('multiselect', 'dataset', 'aperture',
-#                                           'background', 'background_value',
-#                                           'pixel_area', 'counts_factor', 'flux_scaling',
-#                                           'calculate_photometry',
-#                                           'unpack_batch_options', 'calculate_batch_photometry'))
+        if self.config == "deconfigged":
+            self.observe_traitlets_for_relevancy(traitlets_to_observe=['dataset_items'])
+
+    @property
+    def user_api(self):
+        expose = ('multiselect', 'dataset', 'aperture', 'background',
+                  'background_value', 'pixel_area', 'counts_factor', 'flux_scaling',
+                  'calculate_photometry', 'unpack_batch_options',
+                  'calculate_batch_photometry', 'table', 'clear_table',
+                  'export_table', 'fitted_models', 'current_plot_type',
+                  'fit_radial_profile', 'plot')
+
+        return PluginUserApi(self, expose=expose)
+
+    @property
+    def fitted_models(self):
+        return self._fitted_models
 
     def _on_slice_changed(self, msg):
         if self.config != "cubeviz":
@@ -170,7 +213,7 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
         if self.config != "cubeviz":
             return
         # self.dataset might not exist when app is setting itself up.
-        if hasattr(self, "dataset"):
+        if hasattr(self, "dataset") and self.dataset.choices != []:
             if isinstance(self.dataset.selected_dc_item, list):
                 datasets = self.dataset.selected_dc_item
             else:
@@ -491,9 +534,14 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                 data = self.dataset.selected_dc_item
 
         comp = data.get_component(data.main_components[0])
+        if data.ndim > 2:
+            spectral_axis_index = getattr(data, "meta", {}).get("spectral_axis_index", 0)
 
         if self.config == "cubeviz" and data.ndim > 2:
-            comp_data = comp.data[:, :, self._cubeviz_slice_ind].T  # nx, ny --> ny, nx
+            if spectral_axis_index == 0:
+                comp_data = comp.data[self._cubeviz_slice_ind, :, :]
+            else:
+                comp_data = comp.data[:, :, self._cubeviz_slice_ind].T  # nx, ny --> ny, nx
             # Similar to coords_info logic.
             if '_orig_spec' in getattr(data, 'meta', {}):
                 w = data.meta['_orig_spec'].wcs.celestial
@@ -536,7 +584,10 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
             return
 
         try:
-            reg = _get_region_from_spatial_subset(self, self.background.selected_subset_state)
+            type = 'sky_region' if self.app.config == 'imviz' and self.app._align_by == 'wcs' else 'region'  # noqa: E501
+            reg = self.app.get_subsets(subset_name=self.background_selected,
+                                       include_sky_region=type == 'sky_region',
+                                       spatial_only=True)[0][type]
             self.background_value = self._calc_background_median(reg)
         except Exception as e:
             self.background_value = 0
@@ -592,6 +643,12 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
             # we can use the pre-cached value
             data = self.dataset.selected_dc_item
 
+        if data.ndim > 2:
+            if "spectral_axis_index" in getattr(data, "meta", {}):
+                spectral_axis_index = data.meta["spectral_axis_index"]
+            else:
+                spectral_axis_index = 0
+
         if aperture is not None:
             if aperture not in self.aperture.choices:
                 raise ValueError(f"aperture must be one of {self.aperture.choices}")
@@ -612,8 +669,8 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
         # Reset last fitted model
         fit_model = None
         # TODO: remove _fitted_model_name cache?
-        if self._fitted_model_name in self.app.fitted_models:
-            del self.app.fitted_models[self._fitted_model_name]
+        if self._fitted_model_name in self._fitted_models:
+            del self._fitted_models[self._fitted_model_name]
 
         comp = data.get_component(data.main_components[0])
         if comp.units:
@@ -675,7 +732,10 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
             raise ValueError('Missing or invalid background value')
 
         if self.config == "cubeviz" and data.ndim > 2:
-            comp_data = comp.data[:, :, self._cubeviz_slice_ind].T  # nx, ny --> ny, nx
+            if spectral_axis_index == 0:
+                comp_data = comp.data[self._cubeviz_slice_ind, :, :]
+            else:
+                comp_data = comp.data[:, :, self._cubeviz_slice_ind].T  # nx, ny --> ny, nx
             # Similar to coords_info logic.
             if '_orig_spec' in getattr(data, 'meta', {}):
                 w = data.meta['_orig_spec'].wcs
@@ -696,8 +756,11 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
             ycenter = reg.center.y
             if data.coords is not None:
                 if self.config == "cubeviz" and data.ndim > 2:
-                    sky_center = w.pixel_to_world(self._cubeviz_slice_ind,
-                                                  ycenter, xcenter)[1]
+                    if spectral_axis_index == 0:
+                        sky = w.pixel_to_world(xcenter, ycenter, self._cubeviz_slice_ind)
+                    else:
+                        sky = w.pixel_to_world(self._cubeviz_slice_ind, ycenter, xcenter)
+                    sky_center = [coord for coord in sky if hasattr(coord, "icrs")][0]
                 else:  # "imviz"
                     sky_center = w.pixel_to_world(xcenter, ycenter)
             else:
@@ -775,15 +838,19 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
         if include_pixarea_fac:
             # convert pixarea, which is in arcsec2/pix2 to the display solid angle unit / pix2
 
-            if self.config == 'imviz':
+            if self.config in ('imviz', 'deconfigged'):
                 # can remove once unit conversion implemented in imviz and
                 # display_solid_angle_unit traitlet is set, for now it will always be the data units
                 display_solid_angle_unit = check_if_unit_is_per_solid_angle(comp.units,
                                                                             return_unit=True)
 
-            elif self.config == 'cubeviz':
+            elif self.config == "cubeviz":
                 display_solid_angle_unit = u.Unit(self.display_solid_angle_unit)
 
+            else:
+                raise NotImplementedError(
+                    f"Unsupported config {self.config} for aperture photometry.")
+
             # if angle unit is pix2, pixarea should be 1 pixel2 per pixel2
             if display_solid_angle_unit == PIX2:
                 pixarea_fac = 1 * PIX2
@@ -913,9 +980,9 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                     self.plot.figure.title = 'Curve of growth from aperture center'
                     eqv = []
                 x_arr, sum_arr, x_label, y_label = _curve_of_growth(
-                    comp_data, (xcenter, ycenter), aperture, phot_table['sum'][0],
-                    wcs=data.coords, background=bg, pixarea_fac=pixarea_fac,
-                    display_unit=plot_display_unit, equivalencies=eqv)
+                    comp_data, (xcenter, ycenter), aperture, wcs=data.coords, background=bg,
+                    pixarea_fac=pixarea_fac, image_unit=img_unit, display_unit=plot_display_unit,
+                    equivalencies=eqv)
                 self.plot._update_data('profile', x=x_arr, y=sum_arr, reset_lims=True)
                 self.plot.update_style('profile', line_visible=True, color='gray', size=32)
                 self.plot.update_style('fit', visible=False)
@@ -937,9 +1004,9 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                         self.plot.figure.title = 'Radial profile from aperture center'
                         eqv = []
                     x_data, y_data = _radial_profile(
-                        phot_aperstats.data_cutout, phot_aperstats.bbox, (xcenter, ycenter),
-                        raw=False, display_unit=plot_display_unit, image_unit=img_unit,
-                        equivalencies=eqv)
+                        comp_data, phot_aperstats.bbox, (xcenter, ycenter),
+                        raw=False, image_unit=img_unit, display_unit=plot_display_unit,
+                        equivalencies=eqv, background=bg)
                     self.plot._update_data('profile', x=x_data, y=y_data, reset_lims=True)
                     self.plot.update_style('profile', line_visible=True, color='gray', size=32)
 
@@ -949,13 +1016,15 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                     else:
                         self.plot.figure.title = 'Raw radial profile from aperture center'
                     x_data, y_data = _radial_profile(
-                        phot_aperstats.data_cutout, phot_aperstats.bbox, (xcenter, ycenter),
-                        raw=True, display_unit=plot_display_unit, image_unit=img_unit)
+                        comp_data, phot_aperstats.bbox, (xcenter, ycenter), raw=True,
+                        image_unit=img_unit, display_unit=plot_display_unit, background=bg)
 
                     self.plot._update_data('profile', x=x_data, y=y_data, reset_lims=True)
                     self.plot.update_style('profile', line_visible=False, color='gray', size=10)
 
                 # Fit Gaussian1D to radial profile data.
+                # Even though photutils radial profile has gaussian fit option, the fit is done
+                # before Jdaviz unit conversion, so we do our own fit here after unit conversion.
                 if self.fit_radial_profile:
                     fitter = TRFLSQFitter()
                     y_max = np.nanmax(y_data)
@@ -974,7 +1043,7 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
                         self.hub.broadcast(SnackbarMessage(
                             f"Radial profile fitting: {msg}", color='warning', sender=self))
                     y_fit = fit_model(x_data)
-                    self.app.fitted_models[self._fitted_model_name] = fit_model
+                    self._fitted_models[self._fitted_model_name] = fit_model
                     self.plot._update_data('fit', x=x_data, y=y_fit, reset_lims=True)
                     self.plot.update_style('fit', color='magenta',
                                            markers_visible=False, line_visible=True)
@@ -1220,14 +1289,14 @@ class SimpleAperturePhotometry(PluginTemplateMixin, ApertureSubsetSelectMixin,
 # NOTE: These are hidden because the APIs are for internal use only
 # but we need them as a separate functions for unit testing.
 
-def _radial_profile(radial_cutout, reg_bb, centroid, raw=False,
-                    image_unit=None, display_unit=None, equivalencies=[]):
+def _radial_profile(data, reg_bb, centroid, raw=False,
+                    image_unit=None, display_unit=None, equivalencies=[], background=0):
     """Calculate radial profile.
 
     Parameters
     ----------
-    radial_cutout : ndarray
-        Cutout image from ``ApertureStats``.
+    data : ndarray or `~astropy.units.Quantity`
+        Data for radial profile.
 
     reg_bb : obj
         Bounding box from ``ApertureStats``.
@@ -1241,7 +1310,7 @@ def _radial_profile(radial_cutout, reg_bb, centroid, raw=False,
 
     image_unit : str or None
         (For cubeviz only to deal with display unit conversion). Unit of input
-        'radial cutout', used with `display_unit` to convert output to desired
+        data, used with `display_unit` to convert output to desired
         display unit.
 
     display_unit : str or None
@@ -1253,29 +1322,29 @@ def _radial_profile(radial_cutout, reg_bb, centroid, raw=False,
         to display unit selected in the unit conversion plugin, if it differs
         from the native data unit.
 
+    background : float or `~astropy.units.Quantity`
+        Background to subtract, if any. Unit must match ``data``.
+
+    Returns
+    -------
+    x_arr, y_arr : ndarray
+        X and Y data to plot. Y should already be background subtracted.
+
     """
-    reg_ogrid = np.ogrid[reg_bb.iymin:reg_bb.iymax, reg_bb.ixmin:reg_bb.ixmax]
-    radial_dx = reg_ogrid[1] - centroid[0]
-    radial_dy = reg_ogrid[0] - centroid[1]
-    radial_r = np.hypot(radial_dx, radial_dy)
+    if isinstance(data, u.Quantity):
+        data = data.value
+    if isinstance(background, u.Quantity):
+        background = background.value  # data unit
 
-    # Sometimes the mask is smaller than radial_r
-    if radial_cutout.shape != reg_bb.shape:
-        radial_r = radial_r[:radial_cutout.shape[0], :radial_cutout.shape[1]]
-
-    radial_r = radial_r[~radial_cutout.mask].ravel()  # pix
-    radial_img = radial_cutout.compressed()  # data unit
+    edge_radii = np.arange(max(reg_bb.shape) // 2 + 2)  # Edges need one extra
+    rp = RadialProfile(data, centroid, edge_radii, error=None, mask=None)
 
     if raw:
-        i_arr = np.argsort(radial_r)
-        x_arr = radial_r[i_arr]
-        y_arr = radial_img[i_arr]
+        x_arr = rp.data_radius  # pix
+        y_arr = rp.data_profile - background  # data unit
     else:
-        # This algorithm is from the imexam package,
-        # see licenses/IMEXAM_LICENSE.txt for more details
-        radial_r = np.rint(radial_r).astype(int)
-        y_arr = np.bincount(radial_r, radial_img) / np.bincount(radial_r)
-        x_arr = np.arange(y_arr.size)
+        x_arr = rp.radius  # pix
+        y_arr = rp.profile - background  # data unit
 
     if display_unit is not None:
         if image_unit is None:
@@ -1292,8 +1361,8 @@ def _radial_profile(radial_cutout, reg_bb, centroid, raw=False,
     return x_arr, y_arr
 
 
-def _curve_of_growth(data, centroid, aperture, final_sum, wcs=None, background=0,
-                     n_datapoints=10, pixarea_fac=None, display_unit=None, equivalencies=[]):
+def _curve_of_growth(data, centroid, aperture, wcs=None, background=0, n_datapoints=10,
+                     pixarea_fac=None, image_unit=None, display_unit=None, equivalencies=[]):
     """Calculate curve of growth for aperture photometry.
 
     Parameters
@@ -1305,14 +1374,8 @@ def _curve_of_growth(data, centroid, aperture, final_sum, wcs=None, background=0
         ``ApertureStats`` centroid or desired center in ``(x, y)``.
 
     aperture : obj
-        ``photutils`` aperture to use, except its center will be
-        changed to the given ``centroid``. This is because the aperture
-        might be hand-drawn and a more accurate centroid has been
-        recalculated separately.
-
-    final_sum : float or `~astropy.units.Quantity`
-        Aperture sum that is already calculated in the
-        main plugin above.
+        ``photutils`` aperture used to set radii for the curve.
+        Actual aperture used is always circular.
 
     wcs : obj or `None`
         Supported WCS objects or `None`.
@@ -1323,14 +1386,24 @@ def _curve_of_growth(data, centroid, aperture, final_sum, wcs=None, background=0
     n_datapoints : int
         Number of data points in the curve.
 
-    pixarea_fac : float or `None`
+    pixarea_fac : `~astropy.units.Quantity` or `None`
         For ``flux_unit/sr`` to ``flux_unit`` conversion.
 
+    image_unit : str or None
+        (For cubeviz only to deal with display unit conversion). Unit of input
+        data, used with `display_unit` to convert output to desired
+        display unit.
+
     display_unit : str or None
         (For cubeviz only to deal with display unit conversion). Desired unit
         for output. If unit is a surface brightness, a Flux unit will be
         returned if pixarea_fac is provided.
 
+    equivalencies : list or None
+        Optional, equivalencies for unit conversion to convert radial profile
+        to display unit selected in the unit conversion plugin, if it differs
+        from the native data unit.
+
     Returns
     -------
     x_arr : ndarray
@@ -1350,62 +1423,59 @@ def _curve_of_growth(data, centroid, aperture, final_sum, wcs=None, background=0
     """
     n_datapoints += 1  # n + 1
 
-    # determined desired unit for output sum array and y label
-    # cubeviz only to handle unit conversion display unit changes
-    if display_unit is not None:
-        sum_unit = u.Unit(display_unit)
-    else:
-        if isinstance(data, u.Quantity):
-            sum_unit = data.unit
-        else:
-            sum_unit = None
-    if sum_unit and pixarea_fac is not None:
-        # multiply data unit by its solid angle to convert sum in sb to sum in flux
-        sum_unit *= check_if_unit_is_per_solid_angle(sum_unit, return_unit=True)
+    if isinstance(data, u.Quantity):
+        data = data.value
+    if isinstance(background, u.Quantity):
+        background = background.value  # data unit
 
     if hasattr(aperture, 'to_pixel'):
         aperture = aperture.to_pixel(wcs)
 
     if isinstance(aperture, CircularAperture):
-        x_label = 'Radius (pix)'
-        x_arr = np.linspace(0, aperture.r, num=n_datapoints)[1:]
-        aper_list = [CircularAperture(centroid, cur_r) for cur_r in x_arr[:-1]]
+        r = aperture.r
     elif isinstance(aperture, EllipticalAperture):
-        x_label = 'Semimajor axis (pix)'
-        x_arr = np.linspace(0, aperture.a, num=n_datapoints)[1:]
-        a_arr = x_arr[:-1]
-        b_arr = aperture.b * a_arr / aperture.a
-        aper_list = [EllipticalAperture(centroid, cur_a, cur_b, theta=aperture.theta)
-                     for (cur_a, cur_b) in zip(a_arr, b_arr)]
+        r = max(aperture.a, aperture.b)
     elif isinstance(aperture, RectangularAperture):
-        x_label = 'Width (pix)'
-        x_arr = np.linspace(0, aperture.w, num=n_datapoints)[1:]
-        w_arr = x_arr[:-1]
-        h_arr = aperture.h * w_arr / aperture.w
-        aper_list = [RectangularAperture(centroid, cur_w, cur_h, theta=aperture.theta)
-                     for (cur_w, cur_h) in zip(w_arr, h_arr)]
+        r = max(aperture.w, aperture.h) * 0.5
     else:
         raise TypeError(f'Unsupported aperture: {aperture}')
 
-    sum_arr = [ApertureStats(data, cur_aper, wcs=wcs, local_bkg=background).sum
-               for cur_aper in aper_list]
-    if isinstance(sum_arr[0], u.Quantity):
-        sum_arr = u.Quantity(sum_arr)
-    else:
-        sum_arr = np.array(sum_arr)
-    if pixarea_fac is not None:
-        sum_arr = sum_arr * pixarea_fac
-    if isinstance(final_sum, u.Quantity):
-        final_sum = flux_conversion_general(final_sum.value, final_sum.unit,
-                                            sum_arr.unit, equivalencies)
-    sum_arr = np.append(sum_arr, final_sum)
+    radii = np.linspace(0, r, num=n_datapoints)[1:]
+    cog = CurveOfGrowth(data, centroid, radii, error=None, mask=None)
+    x_arr = cog.radius
+    sum_arr = cog.profile - (cog.radii ** 2 * np.pi * background)
 
-    if sum_unit is None:
+    # Determine desired unit for output sum array and y label.
+    # Cubeviz only: To handle unit conversion when display unit changes.
+
+    if image_unit is not None:
+        sum_unit = u.Unit(image_unit)
+        if (display_unit is not None) and (image_unit != display_unit):
+            disp_unit = u.Unit(display_unit)
+            do_flux_conv = True
+        else:
+            do_flux_conv = False
+
+        # multiply data unit by its solid angle to convert sum in sb to sum in flux
+        if pixarea_fac is not None:
+            sa = check_if_unit_is_per_solid_angle(sum_unit, return_unit=True)
+            sum_unit *= sa
+            sum_arr = sum_arr * pixarea_fac.value
+            if do_flux_conv:
+                disp_unit *= sa
+
+        # convert array from native data unit to display unit, if they differ
+        if do_flux_conv:
+            sum_arr = flux_conversion_general(sum_arr,
+                                              sum_unit,
+                                              disp_unit,
+                                              equivalencies=equivalencies,
+                                              with_unit=False)
+            y_label = disp_unit.to_string()
+        else:
+            y_label = sum_unit.to_string()
+
+    else:
         y_label = 'Value'
-    else:
-        y_label = sum_unit.to_string()
-        # bqplot does not like Quantity
-        sum_arr = flux_conversion_general(sum_arr.value, sum_arr.unit, sum_unit,
-                                          equivalencies, with_unit=False)
 
-    return x_arr, sum_arr, x_label, y_label
+    return x_arr, sum_arr, 'Radius (pix)', y_label
