@@ -1,8 +1,10 @@
+from copy import deepcopy
+
 import numpy as np
-import numpy.ma as ma
 from astropy import units as u
-from astropy.table import QTable
+from astropy.table import QTable, Table as AstropyTable
 from astropy.coordinates import SkyCoord
+from echo import delay_callback
 from traitlets import List, Unicode, Bool, Int, observe
 
 from jdaviz.core.events import SnackbarMessage
@@ -11,14 +13,17 @@ from jdaviz.core.template_mixin import (PluginTemplateMixin, ViewerSelectMixin,
                                         FileImportSelectPluginComponent, HasFileImportSelect,
                                         with_spinner)
 from jdaviz.core.custom_traitlets import IntHandleEmpty
+from jdaviz.core.events import CatalogResultsChangedMessage, CatalogSelectClickEventMessage
 from jdaviz.core.marks import CatalogMark
-from jdaviz.core.template_mixin import TableMixin
+from jdaviz.core.template_mixin import Table, TableMixin
 from jdaviz.core.user_api import PluginUserApi
+from jdaviz.utils import get_top_layer_index
 
 __all__ = ['Catalogs']
 
 
-@tray_registry('imviz-catalogs', label="Catalog Search")
+@tray_registry('imviz-catalogs', label="Catalog Search",
+               category="data:analysis")
 class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, TableMixin):
     """
     See the :ref:`Catalog Search Plugin Documentation <imviz-catalogs>` for more details.
@@ -29,7 +34,15 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
+    * :meth:`import_catalog`
     * :meth:`zoom_to_selected`
+    * :meth:`search`
+    * :attr:`max_sources`
+    * ``catalog`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
+    * ``table`` (:class:`~jdaviz.core.template_mixin.Table`):
+      Table containing all search results.
+    * ``table_selected`` (:class:`~jdaviz.core.template_mixin.Table`):
+      Table containing all selected search results.
     """
     template_file = __file__, "catalogs.vue"
     uses_active_status = Bool(True).tag(sync=True)
@@ -46,12 +59,13 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
             'Object ID': np.nan,
             'id': np.nan,
             'x_coord': np.nan,
-            'y_coord': np.nan}
+            'y_coord': np.nan
+            }
 
-    @property
-    def user_api(self):
-        return PluginUserApi(self, expose=('clear_table', 'export_table',
-                                           'zoom_to_selected'))
+    headers = ['Right Ascension (degrees)', 'Declination (degrees)',
+               'Object ID', 'x_coord', 'y_coord']
+
+    table_selected_widget = Unicode().tag(sync=True)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -59,6 +73,8 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
         # description displayed under plugin title in tray
         self._plugin_description = 'Query catalog for objects within region on sky.'
 
+        self.viewer.add_filter('is_image_viewer')
+
         cat_options = ['SDSS', 'Gaia']
         if not self.app.state.settings.get('server_is_remote', False):
             cat_options.append('From File...')
@@ -72,31 +88,88 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
         self._marker_name = 'catalog_results'
 
         # initializing the headers in the table that is displayed in the UI
-        headers = ['Right Ascension (degrees)', 'Declination (degrees)',
-                   'Object ID', 'x_coord', 'y_coord']
-
-        self.table.headers_avail = headers
-        self.table.headers_visible = headers
+        self.table.headers_avail = self.headers
+        self.table.headers_visible = self.headers
         self.table._default_values_by_colname = self._default_table_values
-        self.table._selected_rows_changed_callback = lambda msg: self.plot_selected_points()
+        self.table._selected_rows_changed_callback = self._table_selection_changed
+        # note: `item_key` is the name of a column in the
+        # table that has unique values for in row
         self.table.item_key = 'id'
         self.table.show_rowselect = True
 
+        def clear_table_callback():
+            # gets the current viewer
+            viewer = self.viewer.selected_obj
+
+            # resetting values
+            self.results_available = False
+            self.number_of_results = 0
+
+            if self._marker_name in self.app.data_collection.labels:
+                # all markers are removed from the viewer
+                viewer.remove_markers(marker_name=self._marker_name)
+
+        self.table._clear_callback = clear_table_callback
+
+        self.table_selected = Table(self, name='table_selected')
+        self.table_selected.clear_btn_lbl = 'Clear Selection'
+        self.table_selected.show_if_empty = False
+
+        def clear_selected_table_callback():
+            self.table.select_none()
+
+        self.table_selected._clear_callback = clear_selected_table_callback
+        self.table_selected_widget = 'IPY_MODEL_'+self.table_selected.model_id
+
         self.docs_description = "Queries an area encompassed by the viewer using\
                                  a specified catalog and marks all the objects\
                                  found within the area."
 
+        self.session.hub.subscribe(self, CatalogSelectClickEventMessage,
+                                   self._on_catalog_select_click_event)
+
+        self.observe_traitlets_for_relevancy(traitlets_to_observe=['viewer_items'])
+
+    @property
+    def user_api(self):
+        return PluginUserApi(self, expose=('clear_table', 'export_table', 'import_catalog',
+                                           'zoom_to_selected', 'select_rows',
+                                           'select_all', 'select_none',
+                                           'catalog', 'max_sources', 'search',
+                                           'table', 'table_selected'))
+
     @staticmethod
     def _file_parser(path):
+        if isinstance(path, AstropyTable):  # includes QTable
+            path = deepcopy(path)  # Avoid overwriting original input
+            from_file_string = f'API: {path.__class__.__name__} object'
+            return '', {from_file_string: path, "_orig_colnames_for_jdaviz_export": path.colnames}
+
         try:
             table = QTable.read(path)
         except Exception:
             return 'Could not parse file with astropy.table.QTable.read', {}
 
+        if not table.colnames:  # Ensure the file has columns
+            return "File contains no columns", {}
+
         if 'sky_centroid' not in table.colnames:
             return 'Table does not contain required sky_centroid column', {}
 
-        return '', {path: table}
+        table.meta["_orig_colnames_for_jdaviz_export"] = table.colnames
+        return '', {path: table, "_orig_colnames_for_jdaviz_export": table.colnames}
+
+    def _on_catalog_select_click_event(self, msg):
+        xs, ys = self.table._qtable['x_coord'], self.table._qtable['y_coord']
+        # nearest point
+        distsq = (xs - msg.x)**2 + (ys - msg.y)**2
+        ind = np.argmin(distsq)
+        item = self.table.items[ind]
+        if item in self.table.selected_rows:
+            self.table.selected_rows = [sr for sr in self.table.selected_rows if sr != item]
+        else:
+            self.table.selected_rows += [item]
+        self._table_selection_changed()
 
     @with_spinner()
     def search(self, error_on_fail=False):
@@ -110,12 +183,12 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
 
         Returns
         -------
-        skycoord_table : `~astropy.coordinates.SkyCoord` or `None`
+        skycoords : `~astropy.coordinates.SkyCoord` or `None`
             Sky coordinates (or None if no results available).
 
         """
         # calling clear in the case the user forgot after searching
-        self.clear()
+        self.clear_table()
 
         # gets the current viewer
         viewer = self.viewer.selected_obj
@@ -185,9 +258,6 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
             # TODO: Filter this table the same way as the actual displayed markers.
             # attach the table to the app for Python extraction
             self.app._catalog_source_table = query_region_result
-            skycoord_table = SkyCoord(query_region_result['ra'],
-                                      query_region_result['dec'],
-                                      unit='deg')
 
         elif self.catalog_selected == 'Gaia':
             from astroquery.gaia import Gaia
@@ -203,18 +273,20 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
             if len(sources) == self.max_sources:
                 max_sources_used = True
             self.app._catalog_source_table = sources
-            skycoord_table = SkyCoord(sources['ra'], sources['dec'], unit='deg')
 
         elif self.catalog_selected == 'From File...':
             # all exceptions when going through the UI should have prevented setting this path
             # but this exceptions might be raised here if setting from_file from the UI
             table = self.catalog.selected_obj
-            self.app._catalog_source_table = table
+            column_names = table.colnames
+            self.table.headers_avail = self.headers + [
+                col for col in column_names if col not in self.headers]
+            self.table.headers_visible = self.headers
             if len(table['sky_centroid']) > self.max_sources:
-                skycoord_table = table['sky_centroid'][:self.max_sources]
+                self.app._catalog_source_table = table[:self.max_sources]
                 max_sources_used = True
             else:
-                skycoord_table = table['sky_centroid']
+                self.app._catalog_source_table = table
         else:
             self.results_available = False
             self.number_of_results = 0
@@ -222,7 +294,7 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
             raise NotImplementedError(f"{self.catalog_selected} not a supported catalog")
 
         self.results_available = True
-        if not len(skycoord_table):
+        if not len(self.app._catalog_source_table):
             self.number_of_results = 0
             self.app._catalog_source_table = None
             return
@@ -234,68 +306,82 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
                     sender=self)
             self.hub.broadcast(snackbar_message)
 
-        # coordinates found are converted to pixel coordinates
-        pixel_table = viewer.state.reference_data.coords.world_to_pixel(skycoord_table)
-        # coordinates are filtered out (using a mask) if outside the zoom range
-        pair_pixel_table = np.dstack((pixel_table[0], pixel_table[1]))
-        # ma.masked_outside removes the coordinates outside the zoom range
-        # ma.compress_rows removes any row that has a mask mark
-        filtered_table = ma.compress_rows(
-            ma.masked_outside(pair_pixel_table, [zoom_x_min, zoom_y_min], [zoom_x_max, zoom_y_max])
-            [0])
-        # coordinates are split into their respective x and y values
-        # then they are converted to sky coordinates
-        filtered_pair_pixel_table = np.array(np.hsplit(filtered_table, 2))
-        x_coordinates = np.squeeze(filtered_pair_pixel_table[0])
-        y_coordinates = np.squeeze(filtered_pair_pixel_table[1])
+        # Convert to pixel coordinates and filter results to be within viewer bounds
+        if self.catalog_selected in ["SDSS", "Gaia"]:
+            skycoords = SkyCoord(self.app._catalog_source_table['ra'],
+                                 self.app._catalog_source_table['dec'],
+                                 unit='deg')
+        elif self.catalog_selected in ["From File..."]:
+            skycoords = self.app._catalog_source_table['sky_centroid']
+
+        pixel_table = viewer.state.reference_data.coords.world_to_pixel(skycoords)
+        self.app._catalog_source_table['x_coord'] = pixel_table[0]
+        self.app._catalog_source_table['y_coord'] = pixel_table[1]
+        x_coordinates = []
+        y_coordinates = []
+        source_table = self.app._catalog_source_table
+        mask = ((source_table['x_coord'] < zoom_x_min) |
+                (source_table['x_coord'] > zoom_x_max) |
+                (source_table['y_coord'] < zoom_y_min) |
+                (source_table['y_coord'] > zoom_y_max))
+        self.app._catalog_source_table = self.app._catalog_source_table[~mask]
+        skycoords = skycoords[~mask]
 
         if self.catalog_selected in ["SDSS", "Gaia"]:
-            # for single source convert table information to lists for zipping
-            if len(self.app._catalog_source_table) == 1 or self.max_sources == 1:
-                x_coordinates = [x_coordinates]
-                y_coordinates = [y_coordinates]
-
-            for row, x_coord, y_coord in zip(self.app._catalog_source_table,
-                                             x_coordinates, y_coordinates):
+            for row in self.app._catalog_source_table:
+                x_coordinates.append(row['x_coord'])
+                y_coordinates.append(row['y_coord'])
                 row_id = row[src_id_colname]
                 # Check if the row contains the required keys
                 row_info = {'Right Ascension (degrees)': row['ra'],
                             'Declination (degrees)': row['dec'],
                             'Object ID': row_id.astype(str),
                             'id': len(self.table),
-                            'x_coord': x_coord.item() if x_coord.size == 1 else x_coord,
-                            'y_coord': y_coord.item() if y_coord.size == 1 else y_coord}
+                            'x_coord': row['x_coord'],
+                            'y_coord': row['y_coord']}
                 self.table.add_item(row_info)
+
         # NOTE: If performance becomes a problem, see
         # https://docs.astropy.org/en/stable/table/index.html#performance-tips
         elif self.catalog_selected in ["From File..."]:
-            # for single source convert table information to lists for zipping
-            if len(self.app._catalog_source_table) == 1 or self.max_sources == 1:
-                x_coordinates = [x_coordinates]
-                y_coordinates = [y_coordinates]
+            for row in self.app._catalog_source_table:  # noqa:E501
+                row_info = {
+                    'Right Ascension (degrees)': row['sky_centroid'].ra.deg,
+                    'Declination (degrees)': row['sky_centroid'].dec.deg,
+                    'Object ID': str(row.get('label', f"{len(self.table) + 1}")),
+                    'id': len(self.table),
+                    'x_coord': row['x_coord'],
+                    'y_coord': row['y_coord'],
+                }
+                x_coordinates.append(row['x_coord'])
+                y_coordinates.append(row['y_coord'])
+                # Add sky_centroid and label explicitly to row_info
+                row_info['sky_centroid'] = row['sky_centroid']
+                row_info['label'] = row_info['Object ID']
+                for col in table.colnames:
+                    if col not in self.headers:  # Skip already processed columns
+                        row_info[col] = row[col]
 
-            for row, x_coord, y_coord in zip(self.app._catalog_source_table,
-                                             x_coordinates, y_coordinates):
-                # Check if the row contains the required keys
-                row_info = {'Right Ascension (degrees)': row['sky_centroid'].ra.deg,
-                            'Declination (degrees)': row['sky_centroid'].dec.deg,
-                            'Object ID': str(row.get('label', 'N/A')),
-                            'id': len(self.table),
-                            'x_coord': x_coord.item() if x_coord.size == 1 else x_coord,
-                            'y_coord': y_coord.item() if y_coord.size == 1 else y_coord}
                 self.table.add_item(row_info)
 
-        filtered_skycoord_table = viewer.state.reference_data.coords.pixel_to_world(x_coordinates,
-                                                                                    y_coordinates)
+        filtered_skycoords = viewer.state.reference_data.coords.pixel_to_world(x_coordinates,
+                                                                               y_coordinates)
 
         # QTable stores all the filtered sky coordinate points to be marked
-        catalog_results = QTable({'coord': filtered_skycoord_table})
+        catalog_results = QTable({'coord': filtered_skycoords})
 
         self.number_of_results = len(catalog_results)
         # markers are added to the viewer based on the table
         viewer.marker = {'color': 'blue', 'alpha': 0.8, 'markersize': 30, 'fill': False}
         viewer.add_markers(table=catalog_results, use_skycoord=True, marker_name=self._marker_name)
-        return skycoord_table
+
+        if "_orig_colnames_for_jdaviz_export" in self.catalog._cached_obj:
+            self.table._qtable.meta["_orig_colnames_for_jdaviz_export"] = self.catalog._cached_obj["_orig_colnames_for_jdaviz_export"]  # noqa: E501
+
+        msg = CatalogResultsChangedMessage(sender=self)
+        self.session.hub.broadcast(msg)
+
+        return skycoords
 
     def _get_mark(self, viewer):
         matches = [mark for mark in viewer.figure.marks if isinstance(mark, CatalogMark)]
@@ -319,16 +405,23 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
         for mark in self.marks.values():
             mark.visible = self.is_active
 
-    def plot_selected_points(self):
+    def _table_selection_changed(self, msg={}):
         selected_rows = self.table.selected_rows
 
+        self.table_selected._clear_table()
+        for selected_row in selected_rows:
+            self.table_selected.add_item(selected_row)
+
+        if (self.table_selected._qtable and
+                "_orig_colnames_for_jdaviz_export" in self.catalog._cached_obj):
+            self.table_selected._qtable.meta["_orig_colnames_for_jdaviz_export"] = self.catalog._cached_obj["_orig_colnames_for_jdaviz_export"]  # noqa: E501
+
         x = [float(coord['x_coord']) for coord in selected_rows]
         y = [float(coord['y_coord']) for coord in selected_rows]
         self._get_mark(self.viewer.selected_obj).update_xy(getattr(x, 'value', x),
                                                            getattr(y, 'value', y))
 
     def vue_zoom_in(self, *args, **kwargs):
-
         self.zoom_to_selected()
 
     def zoom_to_selected(self, padding=0.02, return_bounding_box=False):
@@ -348,42 +441,68 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
 
         Returns
         -------
-        list of float or None
+        bb : list of float or None
             If there are activley selected rows, and ``return_bounding_box`` is
             True, returns a list containing the bounding
-            box coordinates: [x_min, x_max, y_min, y_max].
-            Otherwise, returns None.
+            box coordinates: ``[x_min, x_max, y_min, y_max]``.
+            Otherwise, returns `None`.
 
         """
-
-        viewer = self.app._jdaviz_helper._default_viewer
+        viewer = self.viewer.selected_obj  # gets the current viewer
 
         selected_rows = self.table.selected_rows
-
-        if not len(selected_rows):
+        if not selected_rows:  # Check if no rows are selected
             return
 
         if padding <= 0 or padding > 1:
-            raise ValueError("`padding` must be between 0 and 1.")
+            raise ValueError("padding must be between 0 (exclusive) and 1 (inclusive).")
 
-        x = [float(coord['x_coord']) for coord in selected_rows]
-        y = [float(coord['y_coord']) for coord in selected_rows]
+        i_top = get_top_layer_index(viewer)
+        image = viewer.layers[i_top].layer
+        x_min = 99999
+        x_max = -99999
+        y_min = 99999
+        y_max = -99999
+        for coord in selected_rows:  # list of dict
+            cur_x, cur_y = viewer._get_real_xy(
+                image, float(coord['x_coord']), float(coord['y_coord']))[:2]
+            if cur_x < x_min:
+                x_min = cur_x
+            if cur_x > x_max:
+                x_max = cur_x
+            if cur_y < y_min:
+                y_min = cur_y
+            if cur_y > y_max:
+                y_max = cur_y
 
-        limits = viewer.state._get_reset_limits()
-        max_dim = max((limits[1] - limits[0]), (limits[3] - limits[2]))
-        padding = max_dim * padding
+        if x_min == x_max and y_min == y_max:  # Only one selected
+            pass
+        elif x_min >= x_max or y_min >= y_max:
+            raise ValueError(
+                f"Zoom failed: x_min={x_min}, x_max={x_max}, y_min={y_min}, y_max={y_max}")
 
-        # this works with single selected points
-        # zooming when the range is too large is not performing correctly
-        x_min = min(x) - padding
-        x_max = max(x) + padding
-        y_min = min(y) - padding
-        y_max = max(y) + padding
+        pix_pad = padding * max(x_max, y_max)
+        x_min -= pix_pad
+        x_max += pix_pad
+        y_min -= pix_pad
+        y_max += pix_pad
+        new_y_min = viewer._get_real_xy(image, x_min, y_min, reverse=True)[1]
+        new_y_max = viewer._get_real_xy(image, x_max, y_max, reverse=True)[1]
 
-        viewer.set_limits(x_min=x_min, x_max=x_max, y_min=y_min, y_max=y_max)
+        # First, we center using image's coordinates.
+        viewer.center_on((0.5 * (x_min + x_max), 0.5 * (y_min + y_max)))
+
+        # Then, we zoom using reference data's coordinates. This is important when WCS linked.
+        # We cannot use viewer.zoom_level because it is wonky when WCS linked.
+        # Given most displays are wider in X, we make sure Y coordinates all fit first
+        # and X will naturally all fit within after aspect ratio is taken into account.
+        with delay_callback(viewer.state, 'x_min', 'x_max', 'y_min', 'y_max'):
+            viewer.state.y_min = new_y_min
+            viewer.state.y_max = new_y_max
+        viewer.state._adjust_limits_aspect()
 
         if return_bounding_box:
-            return [x_min, x_max, y_min, y_max]
+            return [viewer.state.x_min, viewer.state.x_max, viewer.state.y_min, viewer.state.y_max]
 
     def import_catalog(self, catalog):
         """
@@ -394,39 +513,13 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, Tabl
         catalog : str
           Path to a file that can be parsed by astropy QTable
         """
-        # TODO: self.catalog.import_obj for a QTable directly (see footprints implementation)
         if isinstance(catalog, str):
             self.catalog.import_file(catalog)
+        elif isinstance(catalog, AstropyTable):  # includes QTable
+            self.catalog.import_obj(catalog)
         else:  # pragma: no cover
-            raise ValueError("catalog must be a string (file path)")
+            raise ValueError("catalog must be a string (file path) or Table object")
 
     def vue_do_search(self, *args, **kwargs):
         # calls self.search() which handles all of the searching logic
         self.search()
-
-    def clear(self, hide_only=True):
-        # gets the current viewer
-        viewer = self.viewer.selected_obj
-
-        if not hide_only and self._marker_name in self.app.data_collection.labels:
-            # resetting values
-            self.results_available = False
-            self.number_of_results = 0
-
-            # all markers are removed from the viewer
-            viewer.remove_markers(marker_name=self._marker_name)
-
-        elif self.results_available:
-            from jdaviz.utils import layer_is_table_data
-
-            # resetting values
-            self.results_available = False
-            self.number_of_results = 0
-
-            # markers still there, just hidden
-            for lyr in viewer.layers:
-                if layer_is_table_data(lyr.layer) and lyr.layer.label == self._marker_name:
-                    lyr.visible = False
-
-    def vue_do_clear(self, *args, **kwargs):
-        self.clear()
