@@ -7,9 +7,11 @@ from glue.core.message import DataCollectionAddMessage, DataCollectionDeleteMess
 from glue_jupyter.common.toolbar_vuetify import read_icon
 
 from jdaviz.core.custom_traitlets import FloatHandleEmpty
-from jdaviz.core.events import LinkUpdatedMessage, ChangeRefDataMessage
+from jdaviz.core.events import (LinkUpdatedMessage, ChangeRefDataMessage,
+                                FootprintSelectClickEventMessage,
+                                FootprintMarkVisibilityChangedMessage)
 from jdaviz.core.marks import FootprintOverlay
-from jdaviz.core.region_translators import regions2roi
+from jdaviz.core.region_translators import is_stcs_string, regions2roi, stcs_string2region
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin, ViewerSelectMixin,
                                         EditableSelectPluginComponent, SelectPluginComponent,
@@ -23,7 +25,96 @@ from jdaviz.configs.imviz.plugins.footprints import preset_regions
 __all__ = ['Footprints']
 
 
-@tray_registry('imviz-footprints', label="Footprints")
+# These functions help us determine which polygon overlay is spatially closest
+# to a user's click on the viewer. The math is based on projecting a point
+# onto line segments and computing the nearest one across all polygons.
+#
+# This is useful when multiple regions are displayed and we want to know
+# which one the user meant to interact with.
+def closest_point_on_segment(px, py, x1, y1, x2, y2):
+    """
+    Find the closest points on a line segment to a given point.
+
+    Parameters
+    ----------
+    px : float
+        X coordinate of the reference point.
+    py : float
+        Y coordinate of the reference point.
+    x1, y1 : numpy.ndarray
+        Coordinates of the starting points of the segments.
+    x2, y2 : numpy.ndarray
+        Coordinates of the ending points of the segments.
+
+    Returns
+    -------
+    closest_xs : numpy.ndarray
+        The x coordinates of the closest points on the segments.
+    closest_ys : numpy.ndarray
+        The y coordinates of the closest points on the segments.
+
+    """
+
+    dx, dy = x2 - x1, y2 - y1
+    denominator = dx ** 2 + dy ** 2
+    # Calculate t: how far along the segment the projection of the point falls
+    t = ((px - x1) * dx + (py - y1) * dy) / np.where(denominator == 0, 1, denominator)
+    t = np.where(denominator == 0, 0, t)  # t = 0 if denominator was actually 0
+    t = np.clip(t, 0, 1)
+    closest_xs = x1 + t * dx
+    closest_ys = y1 + t * dy
+    return closest_xs, closest_ys
+
+
+def find_closest_polygon_point(px, py, polygons):
+    """
+    Return the closest point on a polygon AND its overlay label from a list of polygons.
+
+    Parameters
+    ----------
+    px : float
+        X coordinate of the reference point.
+    py : float
+        Y coordinate of the reference point.
+    polygons : list of dict
+        List of polygons to compare against the given point.
+        Each dict contains 'x', 'y' coordinates and an 'overlay' label.
+
+    Returns
+    -------
+    closest_overlay : str
+        Label of the closest overlay.
+    closest_point : tuple of float
+        The (x, y) coordinates of the closest point.
+    """
+    min_dist = float('inf')
+    closest_point = None
+    closest_overlay = None
+
+    for polygon in polygons:
+        x_coords, y_coords = np.array(polygon['x']), np.array(polygon['y'])
+        x1 = x_coords
+        x2 = np.roll(x_coords, -1)
+        y1 = y_coords
+        y2 = np.roll(y_coords, -1)
+        label = polygon['overlay']
+
+        closest_xs, closest_ys = closest_point_on_segment(px, py, x1, y1, x2, y2)
+        dist = (closest_xs - px)**2 + (closest_ys - py)**2
+
+        min_idx = np.argmin(dist)
+        min_dist_for_this_polygon = dist[min_idx]
+
+        if min_dist_for_this_polygon < min_dist:
+            min_dist = min_dist_for_this_polygon
+            closest_point = (closest_xs[min_idx], closest_ys[min_idx])
+            closest_overlay = label
+
+    return closest_overlay, closest_point
+
+
+@tray_registry('imviz-footprints', label="Footprints",
+               category='data:analysis')
 class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
     """
     See the :ref:`Footprints Plugin Documentation <imviz-footprints>` for more details.
@@ -113,7 +204,7 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
 
         self.viewer.multiselect = True  # multiselect always enabled
         # require a viewer's reference data to have WCS so that footprints can be mapped to sky
-        self.viewer.add_filter('reference_has_wcs')
+        self.viewer.add_filter('is_image_viewer', 'reference_has_wcs')
 
         self.overlay = EditableSelectPluginComponent(self,
                                                      name='overlay',
@@ -160,8 +251,39 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
         self.hub.subscribe(self, DataCollectionAddMessage, handler=self._on_link_type_updated)
         self.hub.subscribe(self, DataCollectionDeleteMessage, handler=self._on_link_type_updated)
         self.hub.subscribe(self, ChangeRefDataMessage, handler=self._on_rotation)  # noqa
+        self.hub.subscribe(self, FootprintSelectClickEventMessage,
+                           handler=self._on_select_footprint_overlay)
         self._on_link_type_updated()
 
+        self.observe_traitlets_for_relevancy(traitlets_to_observe=['viewer_items'])
+
+    def _highlight_overlay(self, overlay_label, viewers=None):
+        """
+        Visually highlight one overlay by thickening its stroke.
+        """
+        for viewer in viewers:
+            for mark in self._get_marks(viewer):
+                mark.set_selected_style(is_selected=mark.overlay == overlay_label)
+
+    def _on_select_footprint_overlay(self, data):
+        click_x, click_y = data.x, data.y
+        viewers = self.viewer.selected_obj if isinstance(self.viewer.selected_obj, list) else [
+                                                     self.viewer.selected_obj]
+
+        overlay_data = []
+        for viewer in viewers:
+            overlays = self._get_marks(viewer)
+            for overlay in overlays:
+                overlay_data.append({
+                    "overlay": overlay.overlay,
+                    "x": np.array(overlay.x),
+                    "y": np.array(overlay.y),
+                })
+        closest_overlay_label, closest_point = find_closest_polygon_point(
+            click_x, click_y, overlay_data)
+        self.overlay_selected = closest_overlay_label
+        self._highlight_overlay(closest_overlay_label, viewers=self.viewer.selected_obj)
+
     @property
     def user_api(self):
         return PluginUserApi(self, expose=('overlay',
@@ -302,6 +424,8 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
                 continue
             viewer.figure.marks = [m for m in viewer.figure.marks
                                    if getattr(m, 'overlay', None) != lbl]
+            self.hub.broadcast(FootprintMarkVisibilityChangedMessage(
+                viewer_id=viewer.reference, sender=self))
 
     @observe('is_active', 'viewer_items')
     # NOTE: intentionally not using skip_if_no_updates_since_last_active since this only controls
@@ -327,6 +451,9 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
                 visible = self._mark_visible(viewer_id, overlay)
                 for mark in marks:
                     mark.visible = visible
+                if marks:
+                    self.hub.broadcast(FootprintMarkVisibilityChangedMessage(
+                        viewer_id=viewer_id, sender=self))
 
     def center_on_viewer(self, viewer_ref=None):
         """
@@ -415,6 +542,8 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
                 fp[key] = getattr(self, attr)
         self._ignore_traitlet_change = False
         self._preset_args_changed(overlay_selected=overlay_selected)
+        # Highlight when selection changes
+        self._highlight_overlay(self.overlay_selected, viewers=self.viewer.selected_obj)
 
     def _mark_visible(self, viewer_id, overlay=None):
         if not self.is_active:
@@ -461,10 +590,14 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
                 mark.visible = visible
                 mark.colors = [self.color]
                 mark.fill_opacities = [self.fill_opacity]
+                self._highlight_overlay(self.overlay_selected, viewers=[viewer])
+                self.hub.broadcast(FootprintMarkVisibilityChangedMessage(
+                    viewer_id=viewer.reference, sender=self))
 
     def import_region(self, region):
         """
-        Import an Astropy regions object (or file).
+        Import an Astropy regions object or if a string is provided, attempt to parse it as a
+        STC-S string or region file.
 
         Parameters
         ----------
@@ -473,10 +606,13 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
         self._ensure_first_overlay()
         if isinstance(region, (regions.Region, regions.Regions)):
             self.preset.import_obj(region)
-        elif isinstance(region, str):  # TODO: support path objects?
-            self.preset.import_file(region)
+        elif isinstance(region, str):
+            if is_stcs_string(region):
+                self.preset.import_obj(stcs_string2region(region))
+            else:  # TODO: support path objects?
+                self.preset.import_file(region)
         else:
-            raise TypeError("region must be a regions.Regions object or string (file path)")
+            raise TypeError("region must be a regions.Regions object, STC-S string or file path")
         # _preset_args_changed was probably already triggered by from_file traitlet changing, but
         # that may have been before the file was fully parsed and available from preset.selected_obj
         self._preset_args_changed()
@@ -617,3 +753,8 @@ class Footprints(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect):
 
             if not update_existing and len(new_marks):
                 viewer.figure.marks = viewer.figure.marks + new_marks
+            self.hub.broadcast(FootprintMarkVisibilityChangedMessage(
+                viewer_id=viewer.reference, sender=self))
+
+            if overlay_selected == self.overlay_selected:
+                self._highlight_overlay(self.overlay_selected, viewers=[viewer])
