@@ -1,22 +1,25 @@
 import os
-
+import warnings
 import asdf
 import numpy as np
 from astropy import units as u
+from astropy.utils.exceptions import AstropyWarning
 from astropy.io import fits
 from astropy.nddata import NDData
 from astropy.wcs import WCS
 from astropy.utils.data import cache_contents
+from urllib.parse import urlparse
 
 from glue.core.data import Component, Data
 from gwcs.wcs import WCS as GWCS
 from stdatamodels import asdf_in_fits
 
-from jdaviz.core.registries import data_parser_registry
 from jdaviz.core.events import SnackbarMessage
+from jdaviz.core.registries import data_parser_registry
 from jdaviz.utils import (
-    standardize_metadata, standardize_roman_metadata,
-    PRIHDR_KEY, _wcs_only_label, download_uri_to_path
+    PRIHDR_KEY, _try_gwcs_to_fits_sip, _wcs_only_label,
+    download_uri_to_path, get_cloud_fits,
+    standardize_metadata, standardize_roman_metadata
 )
 
 try:
@@ -47,7 +50,8 @@ def prep_data_layer_as_dq(data):
 
 @data_parser_registry("imviz-data-parser")
 def parse_data(app, file_obj, ext=None, data_label=None,
-               parent=None, cache=None, local_path=None, timeout=None):
+               parent=None, cache=None, local_path=None, timeout=None,
+               gwcs_to_fits_sip=False):
     """Parse a data file into Imviz.
 
     Parameters
@@ -80,8 +84,24 @@ def parse_data(app, file_obj, ext=None, data_label=None,
         remote requests in seconds (passed to
         `~astropy.utils.data.download_file` or
         `~astroquery.mast.Conf.timeout`).
+
+    gwcs_to_fits_sip : bool, optional
+        Try to convert GWCS coordinates into an approximate FITS SIP solution. Typical
+        precision loss due to this approximation is of order 0.1 pixels. This may
+        improve image rendering performance for images with expensive GWCS
+        transformations.
     """
+
     if isinstance(file_obj, str):
+        if urlparse(file_obj).scheme == 's3':
+            file_obj = get_cloud_fits(
+                file_obj, ext=ext)
+            _parse_image(
+                app, file_obj, data_label, ext=ext, parent=parent,
+                try_gwcs_to_fits_sip=gwcs_to_fits_sip
+            )
+            return
+
         if data_label is None:
             data_label = os.path.splitext(os.path.basename(file_obj))[0]
 
@@ -118,11 +138,17 @@ def parse_data(app, file_obj, ext=None, data_label=None,
             try:
                 if HAS_ROMAN_DATAMODELS:
                     with rdd.open(file_obj) as pf:
-                        _parse_image(app, pf, data_label, ext=ext, parent=parent)
+                        _parse_image(
+                            app, pf, data_label, ext=ext, parent=parent,
+                            try_gwcs_to_fits_sip=gwcs_to_fits_sip
+                        )
             except TypeError:
                 # if roman_datamodels cannot parse the file, load it with asdf:
                 with asdf.open(file_obj) as af:
-                    _parse_image(app, af, data_label, ext=ext, parent=parent)
+                    _parse_image(
+                        app, af, data_label, ext=ext, parent=parent,
+                        try_gwcs_to_fits_sip=gwcs_to_fits_sip
+                    )
 
         elif file_obj_lower.endswith('.reg'):
             # This will load DS9 regions as Subset but only if there is already data.
@@ -130,6 +156,7 @@ def parse_data(app, file_obj, ext=None, data_label=None,
                                                                         combination_mode='new')
 
         else:  # Assume FITS
+
             with fits.open(file_obj) as pf:
                 available_extensions = [hdu.name for hdu in pf]
 
@@ -144,12 +171,18 @@ def parse_data(app, file_obj, ext=None, data_label=None,
                                             if label.endswith('[DATA]')][-1]
                     parent = latest_sci_extension
 
-                _parse_image(app, pf, data_label, ext=ext, parent=parent)
+                _parse_image(
+                    app, pf, data_label, ext=ext, parent=parent,
+                    try_gwcs_to_fits_sip=gwcs_to_fits_sip
+                )
     else:
-        _parse_image(app, file_obj, data_label, ext=ext, parent=parent)
+        _parse_image(
+            app, file_obj, data_label, ext=ext, parent=parent,
+            try_gwcs_to_fits_sip=gwcs_to_fits_sip
+        )
 
 
-def get_image_data_iterator(app, file_obj, data_label, ext=None):
+def get_image_data_iterator(app, file_obj, data_label, ext=None, try_gwcs_to_fits_sip=False):
     """This function is for internal use, so other viz can also extract image data
     like Imviz does.
     """
@@ -157,11 +190,17 @@ def get_image_data_iterator(app, file_obj, data_label, ext=None):
         if 'ASDF' in file_obj:  # JWST ASDF-in-FITS
             # Load multiple extensions
             if ext == '*' or isinstance(ext, (tuple, list)):
-                data_iter = _jwst_all_to_glue_data(file_obj, data_label, load_extensions=ext)
+                data_iter = _jwst_all_to_glue_data(
+                    file_obj, data_label, load_extensions=ext,
+                    try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+                )
 
             # Load only specified extension
             else:
-                data_iter = _jwst_to_glue_data(file_obj, ext, data_label)
+                data_iter = _jwst_to_glue_data(
+                    file_obj, ext, data_label,
+                    try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+                )
 
                 # if more than one viewable extension is found in the file,
                 # issue info message.
@@ -207,11 +246,13 @@ def get_image_data_iterator(app, file_obj, data_label, ext=None):
 
     # load Roman 2D datamodels:
     elif HAS_ROMAN_DATAMODELS and isinstance(file_obj, rdd.DataModel):
-        data_iter = _roman_2d_to_glue_data(file_obj, data_label, ext=ext)
+        data_iter = _roman_asdf_2d_to_glue_data(file_obj, data_label, ext=ext)
 
     # load ASDF files that may not validate as Roman datamodels:
     elif isinstance(file_obj, asdf.AsdfFile):
-        data_iter = _roman_asdf_2d_to_glue_data(file_obj, data_label, ext=ext)
+        data_iter = _roman_asdf_2d_to_glue_data(
+            file_obj, data_label, ext=ext, try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+        )
 
     else:
         raise NotImplementedError(f'Imviz does not support {file_obj}')
@@ -219,12 +260,15 @@ def get_image_data_iterator(app, file_obj, data_label, ext=None):
     return data_iter
 
 
-def _parse_image(app, file_obj, data_label, ext=None, parent=None):
+def _parse_image(app, file_obj, data_label, ext=None, parent=None, try_gwcs_to_fits_sip=False):
     if app is None:
         raise ValueError("app is None, cannot proceed")
     if data_label is None:
         data_label = app.return_data_label(file_obj, ext, alt_name="image_data")
-    data_iter = get_image_data_iterator(app, file_obj, data_label, ext=ext)
+    data_iter = get_image_data_iterator(
+        app, file_obj, data_label, ext=ext,
+        try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+    )
 
     # Save the SCI extension to this list:
     sci_ext = None
@@ -242,7 +286,7 @@ def _parse_image(app, file_obj, data_label, ext=None, parent=None):
             # NOTE: if extending this beyond GWCS, the mouseover logic
             # for outside_*_bounding_box should also be updated.
             data.coords._orig_bounding_box = data.coords.bounding_box
-            data.coords.bounding_box = None
+
         if not data.meta.get(_wcs_only_label, False):
             data_label = app.return_data_label(data_label, alt_name="image_data")
 
@@ -310,7 +354,7 @@ def _validate_bunit(bunit, raise_error=True):
 
 # ---- Functions that handle input from JWST FITS files -----
 
-def _jwst_all_to_glue_data(file_obj, data_label, load_extensions='*'):
+def _jwst_all_to_glue_data(file_obj, data_label, load_extensions='*', try_gwcs_to_fits_sip=False):
     for hdu in file_obj:
         if (
             _validate_fits_image2d(hdu, raise_error=False) and
@@ -321,12 +365,14 @@ def _jwst_all_to_glue_data(file_obj, data_label, load_extensions='*'):
             if ext == 'sci':
                 ext = 'data'
 
-            data, new_data_label = _jwst2data(file_obj, ext, data_label)
+            data, new_data_label = _jwst2data(
+                file_obj, ext, data_label, try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+            )
 
             yield data, new_data_label
 
 
-def _jwst_to_glue_data(file_obj, ext, data_label):
+def _jwst_to_glue_data(file_obj, ext, data_label, try_gwcs_to_fits_sip=False):
     # Translate FITS extension into JWST ASDF convention.
     if ext is None:
         ext = 'data'
@@ -342,12 +388,14 @@ def _jwst_to_glue_data(file_obj, ext, data_label):
             fits_ext = ext
 
     _validate_fits_image2d(file_obj[fits_ext])
-    data, new_data_label = _jwst2data(file_obj, ext, data_label)
+    data, new_data_label = _jwst2data(
+        file_obj, ext, data_label, try_gwcs_to_fits_sip=try_gwcs_to_fits_sip
+    )
 
     yield data, new_data_label
 
 
-def _jwst2data(file_obj, ext, data_label):
+def _jwst2data(file_obj, ext, data_label, try_gwcs_to_fits_sip=False):
     comp_label = ext.upper()
     new_data_label = f'{data_label}[{comp_label}]'
     data = Data(label=new_data_label)
@@ -367,14 +415,25 @@ def _jwst2data(file_obj, ext, data_label):
 
             # This is instance of gwcs.WCS, not astropy.wcs.WCS
             if 'wcs' in dm_meta:
-                data.coords = dm_meta['wcs']
+                gwcs = dm_meta['wcs']
 
+                if try_gwcs_to_fits_sip:
+                    data.coords = _try_gwcs_to_fits_sip(gwcs)
+                else:
+                    data.coords = gwcs
             imdata = dm[ext]
             component = Component.autotyped(imdata, units=bunit)
 
             # Might have bad GWCS. If so, we exclude it.
             try:
-                data.add_component(component=component, label=comp_label)
+                # we catch an astropy warning here for non-standard FITS
+                # keywords arising from the FITS SIP approximation. These
+                # warnings don't affect the result, and there's a pending
+                # glue PR with a fix: https://github.com/glue-viz/glue/pull/2540
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', category=AstropyWarning)
+                    data.add_component(component=component, label=comp_label)
+
             except Exception:  # pragma: no cover
                 data.coords = None
                 data.add_component(component=component, label=comp_label)
@@ -392,8 +451,7 @@ def _jwst2data(file_obj, ext, data_label):
 
 # ---- Functions that handle input from Roman ASDF files -----
 
-def _roman_2d_to_glue_data(file_obj, data_label, ext=None):
-
+def _roman_asdf_2d_to_glue_data(file_obj, data_label, ext=None, try_gwcs_to_fits_sip=False):
     if ext == '*':
         # NOTE: Update as needed. Should cover all the image extensions available.
         ext_list = ('data', 'dq', 'err', 'var_poisson', 'var_rnoise')
@@ -405,7 +463,12 @@ def _roman_2d_to_glue_data(file_obj, data_label, ext=None):
         ext_list = (ext, )
 
     meta = standardize_roman_metadata(file_obj)
-    coords = getattr(getattr(file_obj, 'meta', {}), 'wcs', None)
+    gwcs = meta.get('wcs', None)
+
+    if try_gwcs_to_fits_sip and gwcs is not None:
+        coords = _try_gwcs_to_fits_sip(gwcs)
+    else:
+        coords = gwcs
 
     for cur_ext in ext_list:
         comp_label = cur_ext.upper()
@@ -413,51 +476,20 @@ def _roman_2d_to_glue_data(file_obj, data_label, ext=None):
         data = Data(coords=coords, label=new_data_label)
 
         # This could be a quantity or a ndarray:
-        ext_values = getattr(file_obj, cur_ext)
+        if HAS_ROMAN_DATAMODELS and isinstance(file_obj, rdd.DataModel):
+            ext_values = getattr(file_obj, cur_ext)
+        else:
+            ext_values = file_obj['roman'][cur_ext]
         bunit = getattr(ext_values, 'unit', '')
-        component = Component.autotyped(np.array(ext_values), units=bunit)
+        component = Component(np.array(ext_values), units=bunit)
         data.add_component(component=component, label=comp_label)
-        data.meta.update(meta)
-
-        if comp_label == 'dq':
+        data.meta.update(standardize_metadata(dict(meta)))
+        if comp_label == 'DQ':
             prep_data_layer_as_dq(data)
 
         yield data, new_data_label
 
 
-def _roman_asdf_2d_to_glue_data(file_obj, data_label, ext=None):
-    if ext == '*':
-        # NOTE: Update as needed. Should cover all the image extensions available.
-        ext_list = ('data', 'dq', 'err', 'var_poisson', 'var_rnoise')
-    elif ext is None:
-        ext_list = ('data', )
-    elif isinstance(ext, (list, tuple)):
-        ext_list = ext
-    else:
-        ext_list = (ext, )
-
-    roman = file_obj.tree.get('roman')
-    meta = roman.get('meta', {})
-    coords = meta.get('wcs', None)
-
-    for cur_ext in ext_list:
-        if cur_ext in roman:
-            comp_label = cur_ext.upper()
-            new_data_label = f'{data_label}[{comp_label}]'
-            data = Data(coords=coords, label=new_data_label)
-
-            # This could be a quantity or a ndarray:
-            ext_values = roman.get(cur_ext)
-            bunit = getattr(ext_values, 'unit', '')
-            component = Component(np.array(ext_values), units=bunit)
-            data.add_component(component=component, label=comp_label)
-            data.meta.update(standardize_metadata(dict(meta)))
-            if comp_label == 'DQ':
-                prep_data_layer_as_dq(data)
-
-            yield data, new_data_label
-
-
 # ---- Functions that handle input from non-JWST, non-Roman FITS files -----
 
 def _hdu_to_glue_data(hdu, data_label, hdulist=None):
@@ -465,7 +497,8 @@ def _hdu_to_glue_data(hdu, data_label, hdulist=None):
     yield data, data_label
 
 
-def _hdus_to_glue_data(file_obj, data_label, ext=None):
+def _hdus_to_glue_data(file_obj, data_label, ext=None, try_gwcs_to_fits_sip=False):
+
     for hdu in file_obj:
         if ext is None or ext == '*' or hdu.name in ext:
             if _validate_fits_image2d(hdu, raise_error=False):
