@@ -108,6 +108,37 @@ def test_user_api(imviz_helper, image_2d_wcs, tmp_path):
         plugin._obj.vue_file_import_cancel()
         assert plugin.preset.selected == preset
 
+        # test that importing a proper STC-S string works
+        stc_s = 'POLYGON ICRS 5.023 4.992 5.024 4.991 5.029 4.995 5.026 4.998'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
+        stc_s = 'POLYGON 5.023 4.992 5.024 4.991 5.029 4.995 5.026 4.998'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
+        stc_s = 'CIRCLE ICRS 5.029 4.992 0.000314'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
+        stc_s = 'CIRCLE 5.029 4.992 0.000314'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
+        stc_s = 'ELLIPSE ICRS 5.029 4.992 0.0003143 0.00027 45.0'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
+        stc_s = 'ELLIPSE 5.029 4.992 0.0003143 0.00027 45.0'
+        plugin.import_region(stc_s)
+        viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
+        assert len(viewer_marks) == 1
+
         tmp_file = str(tmp_path / 'test_region.reg')
         reg.write(tmp_file, format='ds9')
         plugin.import_region(tmp_file)
@@ -124,12 +155,18 @@ def test_user_api(imviz_helper, image_2d_wcs, tmp_path):
                                               height=3 * u.deg, width=2 * u.deg)
         plugin.import_region(valid_region_sky)
 
+        # test invalid input
+        with pytest.raises(TypeError):
+            plugin.import_region(5)
+
+        # test invalid file path input
         tmp_invalid_path = str(tmp_path / 'invalid_path.reg')
         with pytest.raises(ValueError):
             plugin.import_region(tmp_invalid_path)
         with pytest.raises(TypeError):
             plugin.import_region(5)
 
+        # test invalid region input
         invalid_region = CirclePixelRegion(PixCoord(x=8, y=7), radius=3.5)
         with pytest.raises(ValueError):
             plugin.import_region(invalid_region)
@@ -139,6 +176,22 @@ def test_user_api(imviz_helper, image_2d_wcs, tmp_path):
             plugin.import_region(tmp_invalid_file)
         assert plugin.preset.selected == preset
 
+        # test invalid STC-S string input
+        invalid_stc_s = 'RECTANGLE 5.029 4.992 0.000314'
+        with pytest.raises(ValueError):
+            plugin.import_region(invalid_stc_s)
+        assert plugin.preset.selected == preset
+
+        invalid_stc_s = 'CIRCLE FK4 5.029 4.992 0.000314'
+        with pytest.raises(ValueError):
+            plugin.import_region(invalid_stc_s)
+        assert plugin.preset.selected == preset
+
+        invalid_stc_s = 'CIRCLE ICRS 5.029 4 0.000314'
+        with pytest.raises(ValueError):
+            plugin.import_region(invalid_stc_s)
+        assert plugin.preset.selected == preset
+
     # with the plugin no longer active, marks should not be visible
     assert plugin._obj.is_active is False
     viewer_marks = _get_markers_from_viewer(imviz_helper.default_viewer)
@@ -249,3 +302,32 @@ def test_footprint_updates_on_rotation(imviz_helper):
     # at the top of the viewer, and this test will fail.
     marks = _get_markers_from_viewer(imviz_helper.default_viewer)
     assert np.concatenate([marks[0].y, marks[1].y]).min() < -3
+
+
+def test_footprint_select(imviz_helper):
+    wcs = WCS({'CTYPE1': 'RA---TAN', 'CUNIT1': 'deg', 'CDELT1': -0.0002777777778,
+               'CRPIX1': 1, 'CRVAL1': 9.423508457380343,
+               'CTYPE2': 'DEC--TAN', 'CUNIT2': 'deg', 'CDELT2': 0.0002777777778,
+               'CRPIX2': 1, 'CRVAL2': -33.71313112382379})
+    arr = np.arange(40000).reshape(200, 200)
+    ndd = NDData(arr, wcs=wcs)
+    imviz_helper.load_data(ndd)
+    fp = imviz_helper.plugins["Footprints"]
+    toolbar = imviz_helper.viewers['imviz-0']._obj.toolbar
+    tool = toolbar.tools['jdaviz:selectfootprint']
+    assert tool.is_visible() is False
+
+    with fp.as_active():
+        current_overlay = fp.overlay.selected
+        assert current_overlay == "default"
+        fp.preset = "MIRI"
+        tool.on_mouse_event({'domain': {'x': 95, 'y': 140}})
+        assert current_overlay == "default"
+        assert tool.is_visible() is True
+
+        # Add a new overlay
+        fp.add_overlay("layer1")
+        fp.overlay = "layer1"
+        fp.v3_offset = 10
+        tool.on_mouse_event({'domain': {'x': 95, 'y': 140}})
+        assert fp.overlay.selected == "layer1"
