@@ -54,7 +54,7 @@ class TestDefaultOrientation(BaseImviz_WCS_WCS):
         if self.viewer.top_visible_data_label != "has_wcs_2[SCI,1]":
             self.viewer.blink_once()
 
-        label_mouseover = self.imviz.app.session.application._tools['g-coords-info']
+        label_mouseover = self.imviz._coords_info
         mp = self.imviz.plugins['Markers']
 
         with mp.as_active():
@@ -90,7 +90,7 @@ class TestDefaultOrientation(BaseImviz_WCS_WCS):
         if self.viewer.top_visible_data_label != "has_wcs_2[SCI,1]":
             self.viewer.blink_once()
 
-        label_mouseover = self.imviz.app.session.application._tools['g-coords-info']
+        label_mouseover = self.imviz._coords_info
         mp = self.imviz.plugins['Markers']
 
         with mp.as_active():
@@ -113,8 +113,8 @@ class TestDefaultOrientation(BaseImviz_WCS_WCS):
             #        https://jira.stsci.edu/browse/JDAT-4256
             with pytest.raises(AssertionError):
                 assert_allclose(mp._obj.marks["imviz-0"].x, [1, 0])
-            with pytest.raises(AssertionError):
-                assert_allclose(mp._obj.marks["imviz-0"].y, 0)
+
+            assert_allclose(mp._obj.marks["imviz-0"].y, 0, atol=1e-08)
 
             mp.clear_table()
 
@@ -177,7 +177,7 @@ class TestDeleteOrientation(BaseImviz_WCS_WCS):
         lc_plugin.viewer = "imviz-1"
         lc_plugin.orientation = "North-up, East-left"
 
-        self.imviz.app.vue_data_item_remove({"item_name": "North-up, East-left"})
+        self.imviz.app.data_item_remove("North-up, East-left")
 
         assert self.viewer.state.reference_data.label == "Default orientation"
         assert viewer_2.state.reference_data.label == "Default orientation"
@@ -202,7 +202,7 @@ class TestDeleteOrientation(BaseImviz_WCS_WCS):
         # Switch to N-up E-right
         lc_plugin.set_north_up_east_right()
 
-        self.imviz.app.vue_data_item_remove({"item_name": "North-up, East-left"})
+        self.imviz.app.data_item_remove("North-up, East-left")
 
         # Check that E-right still linked to default
         assert len(self.imviz.app.data_collection.external_links) == 3
