@@ -7,9 +7,10 @@ from astropy.nddata import NDData, StdDevUncertainty
 from astropy.tests.helper import assert_quantity_allclose
 from astropy.utils.data import download_file
 from astropy.wcs import WCS
+
 from gwcs import WCS as GWCS
 from numpy.testing import assert_allclose, assert_array_equal
-from regions import CirclePixelRegion, PixCoord, RectanglePixelRegion
+from regions import CirclePixelRegion, EllipsePixelRegion, PixCoord, RectanglePixelRegion
 from skimage.io import imsave
 from stdatamodels import asdf_in_fits
 
@@ -236,14 +237,13 @@ class TestParseImage:
 
     @pytest.mark.remote_data
     def test_parse_jwst_nircam_level2(self, imviz_helper):
+
         # Default behavior: Science image
-        with pytest.warns(UserWarning, match='You may be querying for a remote file'):
-            # if you don't pass a `cache` value, a warning should be raised:
-            imviz_helper.load_data(self.jwst_asdf_url_1, timeout=100)
+        imviz_helper.load_data(self.jwst_asdf_url_1, timeout=100, gwcs_to_fits_sip=False)
 
         data = imviz_helper.app.data_collection[0]
-        comp = data.get_component('DATA')
-        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_1))[0] + '[DATA]'
+        comp = data.get_component('data')
+        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_1))[0] + '[SCI,1]'
         assert data.label == expected_label
         assert data.shape == (2048, 2048)
         assert isinstance(data.coords, GWCS)
@@ -252,13 +252,12 @@ class TestParseImage:
 
         # --- Since download is expensive, we attach GWCS-specific tests here. ---
 
-        # Ensure interactive region supports GWCS. Also see test_regions.py
-        imviz_helper._apply_interactive_region('bqplot:truecircle',
-                                               (965, 1122),
-                                               (976.9, 1110.1))  # Star
-        imviz_helper._apply_interactive_region('bqplot:rectangle',
-                                               (982, 1088),
-                                               (1008, 1077))  # Background
+        # Ensure region supports GWCS. Also see test_regions.py
+        imviz_helper.plugins['Subset Tools'].import_region([
+            CirclePixelRegion(center=PixCoord(x=970.95, y=1116.05), radius=5.95),
+            RectanglePixelRegion(center=PixCoord(x=995.0, y=1082.5), width=26, height=11)
+        ], combination_mode="new")
+
         subsets = imviz_helper.plugins['Subset Tools'].get_regions()
         assert list(subsets.keys()) == ['Subset 1', 'Subset 2'], subsets
         # check that retrieved subsets-as-regions from subset plugin match what was loaded.
@@ -286,7 +285,7 @@ class TestParseImage:
         assert_allclose(phot_plugin.counts_factor, 0.0036385915646798953)
         assert_allclose(phot_plugin.flux_scaling, 0.003631)
         phot_plugin.vue_do_aper_phot()
-        tbl = imviz_helper.get_aperture_photometry_results()
+        tbl = imviz_helper.plugins['Aperture Photometry'].export_table()
         assert_quantity_allclose(tbl[0]['xcenter'], 970.95 * u.pix)
         assert_quantity_allclose(tbl[0]['ycenter'], 1116.05 * u.pix)
         sky = tbl[0]['sky_center']
@@ -321,12 +320,12 @@ class TestParseImage:
         # --- Back to parser testing below. ---
 
         # Request specific extension (name + ver, but ver is not used), use given label
-        imviz_helper.load_data(self.jwst_asdf_url_1, cache=True, ext=('DQ', 42),
+        imviz_helper.load_data(self.jwst_asdf_url_1, cache=True, ext='DQ',
                                data_label='jw01072001001_01101_00001_nrcb1_cal',
                                show_in_viewer=False)
         data = imviz_helper.app.data_collection[1]
-        comp = data.get_component('DQ')
-        assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[DQ]'
+        comp = data.get_component('dq')
+        assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[DQ,1]'
         assert data.meta['aperture']['name'] == 'NRCB5_FULL'
         assert comp.units == ''
 
@@ -337,8 +336,8 @@ class TestParseImage:
                                    data_label='jw01072001001_01101_00001_nrcb1_cal',
                                    show_in_viewer=False)
             data = imviz_helper.app.data_collection[2]
-            comp = data.get_component('DATA')  # SCI = DATA
-            assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[DATA]'
+            comp = data.get_component('data')  # SCI = DATA
+            assert data.label == 'jw01072001001_01101_00001_nrcb1_cal[SCI,1]'
             assert isinstance(data.coords, GWCS)
             assert comp.units == 'MJy/sr'
 
@@ -346,24 +345,24 @@ class TestParseImage:
             imviz_helper.app.data_collection.clear()
             imviz_helper.load_data(pf, ext='SCI', data_label='TEST', show_in_viewer=False)
             data = imviz_helper.app.data_collection[0]
-            assert data.label.endswith('[DATA]')
+            assert data.label.endswith('[SCI,1]')
 
             imviz_helper.load_data(pf, ext='SCI', data_label='TEST', show_in_viewer=False)
             data = imviz_helper.app.data_collection[1]
-            assert data.label.endswith('[DATA] (1)')
+            assert data.label.endswith('[SCI,1] (1)')
 
             # Load all extensions
             imviz_helper.app.data_collection.clear()
             imviz_helper.load_data(pf, ext='*', show_in_viewer=False)
             data = imviz_helper.app.data_collection
             assert len(data.labels) == 7
-            assert data.labels[0].endswith('[DATA]')
-            assert data.labels[1].endswith('[ERR]')
-            assert data.labels[2].endswith('[DQ]')
-            assert data.labels[3].endswith('[AREA]')
-            assert data.labels[4].endswith('[VAR_POISSON]')
-            assert data.labels[5].endswith('[VAR_RNOISE]')
-            assert data.labels[6].endswith('[VAR_FLAT]')
+            assert data.labels[0].endswith('[SCI,1]')
+            assert data.labels[1].endswith('[ERR,1]')
+            assert data.labels[2].endswith('[DQ,1]')
+            assert data.labels[3].endswith('[AREA,1]')
+            assert data.labels[4].endswith('[VAR_POISSON,1]')
+            assert data.labels[5].endswith('[VAR_RNOISE,1]')
+            assert data.labels[6].endswith('[VAR_FLAT,1]')
 
         # Invalid ASDF attribute (extension)
         with pytest.raises(KeyError, match='does_not_exist'):
@@ -371,11 +370,10 @@ class TestParseImage:
 
     @pytest.mark.remote_data
     def test_parse_jwst_niriss_grism(self, imviz_helper):
-        """No valid image GWCS for Imviz, will fall back to loading without WCS."""
         imviz_helper.load_data(self.jwst_asdf_url_2, cache=True, show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
-        comp = data.get_component('DATA')
-        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_2))[0] + '[DATA]'
+        comp = data.get_component('data')
+        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_2))[0] + '[SCI,1]'
         assert data.label == expected_label
         assert data.shape == (2048, 2048)
         assert data.coords is None
@@ -393,18 +391,20 @@ class TestParseImage:
         comp = data.get_component('SCI,1')
         expected_label = os.path.splitext(os.path.basename(url))[0] + '[SCI,1]'
         assert data.label == expected_label
-        assert data.shape == (4299, 4219)
+        assert data.shape[0] == 4299
+        # This file keeps slightly changing shape, it's very annoying.
+        assert data.shape[1] in (4219, 4220)
         assert_allclose(data.meta['PHOTFLAM'], 7.8711728E-20)
         assert isinstance(data.coords, WCS)
         assert comp.units == 'electron/s'
-        assert comp.data.shape == (4299, 4219)
+        assert comp.data.shape[0] == 4299
+        assert comp.data.shape[1] in (4219, 4220)
 
         # --- Since download is expensive, we attach FITS WCS-specific tests here. ---
 
         # Test simple aperture photometry plugin.
-        imviz_helper._apply_interactive_region('bqplot:ellipse',
-                                               (1465, 2541),
-                                               (1512, 2611))  # Galaxy
+        imviz_helper.plugins['Subset Tools'].import_region(
+            EllipsePixelRegion(center=PixCoord(x=1488.5, y=2576), width=47, height=70))  # Galaxy
         phot_plugin = imviz_helper.app.get_tray_item_from_name('imviz-aper-phot-simple')
         phot_plugin.data_selected = 'contents[SCI,1]'
         phot_plugin.aperture_selected = 'Subset 1'
@@ -412,7 +412,7 @@ class TestParseImage:
         phot_plugin.background_value = 0.0014  # Manual entry: Median on whole array
         assert_allclose(phot_plugin.pixel_area, 0.0025)  # Not used but still auto-populated
         phot_plugin.vue_do_aper_phot()
-        tbl = imviz_helper.get_aperture_photometry_results()
+        tbl = imviz_helper.plugins['Aperture Photometry'].export_table()
         assert_quantity_allclose(tbl[0]['xcenter'], 1488.5 * u.pix, atol=2 * u.pix)
         assert_quantity_allclose(tbl[0]['ycenter'], 2576 * u.pix, atol=2 * u.pix)
         sky = tbl[0]['sky_center']
@@ -420,29 +420,24 @@ class TestParseImage:
         assert_allclose(sky.dec.deg, 10.802045612042956, rtol=1e-3)
         data_unit = u.electron / u.s
         assert_quantity_allclose(tbl[0]['background'], 0.0014 * data_unit)
-        assert_quantity_allclose(tbl[0]['sum'], 112.712406 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['sum_aper_area'], 2583.959958 * PIX2, rtol=1e-3)
         assert_array_equal(tbl[0]['pixarea_tot'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts_err'], None)
         assert_array_equal(tbl[0]['counts_fac'], None)
         assert_array_equal(tbl[0]['aperture_sum_mag'], None)
         assert_array_equal(tbl[0]['flux_scaling'], None)
-        assert_quantity_allclose(tbl[0]['min'], -0.02422 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['max'], 1.577081 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mean'], 0.043684 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['median'], 0.02129 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mode'], -0.023497 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['std'], 0.09908 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mad_std'], 0.023219 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['var'], 0.009817 * (data_unit * data_unit), rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['biweight_location'], 0.020978 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['biweight_midvariance'], 0.000606 * (data_unit * data_unit), rtol=1e-3)  # noqa
-        assert_quantity_allclose(tbl[0]['fwhm'], 21.230628 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['semimajor_sigma'], 9.712415 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['semiminor_sigma'], 8.260686 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['orientation'], 80.054019 * u.deg, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['eccentricity'], 0.525929, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['sum_aper_area'], 2583.959958 * PIX2, rtol=1e-3)
+        # This file keeps changing in reprocessing between two shapes, which gives different values
+        if data.shape[1] == 4219:
+            assert_quantity_allclose(tbl[0]['sum'], 112.712406 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['min'], -0.02422 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['max'], 1.577081 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['mean'], 0.043684 * data_unit, rtol=1e-3)
+        elif data.shape[1] == 4220:
+            assert_quantity_allclose(tbl[0]['sum'], 127.233862 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['min'], -0.027572 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['max'], 4.021776 * data_unit, rtol=1e-3)
+            assert_quantity_allclose(tbl[0]['mean'], 0.049325 * data_unit, rtol=1e-3)
 
         # Request specific extension (name only), use given label
         imviz_helper.load_data(filename, ext='CTX', data_label='jclj01010_drz',
@@ -453,8 +448,8 @@ class TestParseImage:
         assert data.meta['EXTNAME'] == 'CTX'
         assert comp.units == ''  # BUNIT is not set
 
-        # Request specific extension (name + ver), use given label
-        imviz_helper.load_data(filename, ext=('WHT', 1), data_label='jclj01010_drz',
+        # Request specific extension and use given label
+        imviz_helper.load_data(filename, ext='WHT', data_label='jclj01010_drz',
                                show_in_viewer=False)
         data = imviz_helper.app.data_collection[2]
         comp = data.get_component('WHT,1')
@@ -467,14 +462,14 @@ class TestParseImage:
             # Default behavior: Load first image
             imviz_helper.load_data(pf, show_in_viewer=False)
             data = imviz_helper.app.data_collection[3]
-            assert data.label.startswith('Unknown HDU object') and data.label.endswith('[SCI,1]')
+            assert data.label.startswith('Image') and data.label.endswith('[SCI,1]')
             assert_allclose(data.meta['PHOTFLAM'], 7.8711728E-20)
             assert 'SCI,1' in data.components
 
             # Request specific extension (name only), use given label
             imviz_helper.load_data(pf, ext='CTX', show_in_viewer=False)
             data = imviz_helper.app.data_collection[4]
-            assert data.label.startswith('Unknown HDU object') and data.label.endswith('[CTX,1]')
+            assert data.label.startswith('Image') and data.label.endswith('[CTX,1]')
             assert data.meta['EXTNAME'] == 'CTX'
             assert 'CTX,1' in data.components
 
@@ -502,6 +497,22 @@ class TestParseImage:
         with pytest.raises(KeyError, match='not found'):
             parse_data(imviz_helper.app, filename, ext='DOES_NOT_EXIST', data_label='foo')
 
+    @pytest.mark.remote_data
+    @pytest.mark.filterwarnings("ignore:Some non-standard WCS keywords were excluded")
+    @pytest.mark.parametrize(
+        ('gwcs_to_fits_sip', 'expected_cls'),
+        ((True, WCS), (False, GWCS),), ids=('True-WCS', 'False-GWCS'))
+    def test_gwcs_to_fits_sip(self, gwcs_to_fits_sip, expected_cls, imviz_helper):
+        """
+        Test gwcs_to_fits_sip as an argument to load_data until it is
+        fully deprecated.
+        """
+        imviz_helper.load_data(self.jwst_asdf_url_1, cache=True,
+                               gwcs_to_fits_sip=gwcs_to_fits_sip)
+
+        data = imviz_helper.app.data_collection[0]
+        assert isinstance(data.coords, expected_cls)
+
 
 def test_load_valid_not_valid(imviz_helper):
     # Load something valid.
@@ -509,7 +520,7 @@ def test_load_valid_not_valid(imviz_helper):
     imviz_helper.load_data(arr, data_label='valid', show_in_viewer=False)
 
     # Load something invalid.
-    with pytest.raises(ValueError, match='Imviz cannot load this array with ndim=1'):
+    with pytest.raises(ValueError, match='no valid loaders found for input'):
         imviz_helper.load_data(np.zeros(2), show_in_viewer=False)
 
     # Make sure valid data is still there.
