@@ -1,5 +1,7 @@
 import numpy as np
 import pytest
+from astropy import units as u
+from astropy.nddata import NDData
 from regions import CirclePixelRegion, PixCoord
 
 from jdaviz.app import Application
@@ -27,7 +29,8 @@ def test_create_destroy_viewer(imviz_helper, desired_name, actual_name):
     assert sorted(imviz_helper.plugins['Compass'].viewer.labels) == viewer_names
 
     po = imviz_helper.plugins['Plot Options']
-    po.multiselect = True
+    po.viewer_multiselect = True
+    po.layer_multiselect = True
     po.viewer = viewer_names
 
     imviz_helper.destroy_viewer(actual_name)
@@ -36,6 +39,36 @@ def test_create_destroy_viewer(imviz_helper, desired_name, actual_name):
     assert po.viewer.labels == ['imviz-0']
 
 
+def test_create_viewer_align_by_wcs(imviz_helper, image_2d_wcs):
+    data = NDData(np.ones((128, 128)) * u.nJy, wcs=image_2d_wcs)
+    imviz_helper.load_data(data, data_label='my_data')
+
+    imviz_helper.create_image_viewer(viewer_name='new-viewer')
+    dm = imviz_helper.viewers['new-viewer'].data_menu
+    dm.add_data('my_data[DATA]')
+
+    assert not dm._obj.orientation_align_by_wcs
+
+    imviz_helper.plugins['Orientation'].align_by = 'WCS'
+
+    assert dm._obj.orientation_align_by_wcs
+    assert dm.orientation.selected == 'Default orientation'
+
+
+def test_align_by_wcs_create_viewer(imviz_helper, image_2d_wcs):
+    data = NDData(np.ones((128, 128)) * u.nJy, wcs=image_2d_wcs)
+    imviz_helper.load_data(data, data_label='my_data')
+
+    imviz_helper.plugins['Orientation'].align_by = 'WCS'
+
+    imviz_helper.create_image_viewer(viewer_name='new-viewer')
+    dm = imviz_helper.viewers['new-viewer'].data_menu
+    dm.add_data('my_data[DATA]')
+
+    assert dm._obj.orientation_align_by_wcs
+    assert dm.orientation.selected == 'Default orientation'
+
+
 def test_get_viewer_created(imviz_helper):
     # This viewer has no reference but has ID.
     viewer1 = imviz_helper.create_image_viewer()
@@ -73,7 +106,7 @@ def test_destroy_viewer_with_subset(imviz_helper):
     imviz_helper.destroy_viewer('second')
 
     # Delete the Subset: Should have no traceback.
-    imviz_helper._delete_region('Subset 1')
+    imviz_helper.app.delete_subsets('Subset 1')
 
 
 def test_mastviz_config():
