@@ -6,18 +6,17 @@
 import base64
 import math
 from io import BytesIO
+import warnings
 
 import matplotlib.pyplot as plt
 import numpy as np
 from astropy import units as u
 from astropy import coordinates as coord
 from astropy.coordinates import SkyCoord
-from astropy.modeling import models
 from astropy.nddata import NDData
 from astropy.wcs import WCS
 from astropy.wcs.utils import proj_plane_pixel_scales
 
-from gwcs import coordinate_frames as cf
 from gwcs.wcs import WCS as GWCS
 
 from matplotlib.patches import Polygon
@@ -73,17 +72,31 @@ def add_offset_radec(ra_deg, dec_deg, delta_deg_ra, delta_deg_dec):
 
 def add_offset_xy(image_wcs, x, y, delta_deg_x, delta_deg_y):
     # calculate ra/dec of x,y pixel
-    c = image_wcs.pixel_to_world(x, y)
+    if isinstance(image_wcs, GWCS):
+        c = image_wcs(x, y, with_bounding_box=False)
+    else:
+        c = image_wcs.pixel_to_world(x, y)
+
     if isinstance(c, SkyCoord):
         ra_deg = c.ra.deg
         dec_deg = c.dec.deg
-    else:  # list of Quantity (e.g., from FITS primary header)
+    elif hasattr(c[0], 'unit'):
+        # list of Quantity (e.g., from FITS primary header)
         ra_deg = c[0].value
         dec_deg = c[1].value
+    else:
+        ra_deg = c[0]
+        dec_deg = c[1]
 
     # add offsets
     ra2_deg, dec2_deg = add_offset_radec(ra_deg, dec_deg, delta_deg_x, delta_deg_y)
 
+    if isinstance(image_wcs, GWCS):
+        pixel_coords = image_wcs.invert(ra2_deg, dec2_deg, with_bounding_box=False)
+        if hasattr(pixel_coords[0], 'unit'):
+            return u.Quantity(pixel_coords).to_value(u.pix)
+        return pixel_coords
+
     # then back to new pixel coords
     return image_wcs.world_to_pixel_values(ra2_deg, dec2_deg)  # x2, y2
 
@@ -105,7 +118,6 @@ def calc_compass_radius(image_wcs, x, y, radius_px):
     # a bounding box set.
 
     delta = 0.1 / 3600  # 0.1 arcsec
-
     xe, ye = add_offset_xy(image_wcs, x, y, delta, 0.0)
     xn, yn = add_offset_xy(image_wcs, x, y, 0.0, delta)
 
@@ -276,10 +288,9 @@ def draw_compass_mpl(image, orig_shape=None, wcs=None, show=True, zoom_limits=No
 def data_outside_gwcs_bounding_box(data, x, y):
     """This is for internal use by Imviz coordinates transformation only."""
     outside_bounding_box = False
-    if hasattr(data.coords, '_orig_bounding_box'):
-        # then coords is a GWCS object and had its bounding box cleared
-        # by the Imviz parser
-        ints = data.coords._orig_bounding_box.intervals
+    if getattr(data.coords, 'bounding_box', None) is not None:
+        # then coords is a GWCS object
+        ints = data.coords.bounding_box.intervals
         if isinstance(ints[0].lower, u.Quantity):
             bb_xmin = ints[0].lower.value
             bb_xmax = ints[0].upper.value
@@ -295,7 +306,7 @@ def data_outside_gwcs_bounding_box(data, x, y):
     return outside_bounding_box
 
 
-def _rotated_gwcs(
+def _rotated_wcs(
     center_world_coord,
     rotation_angle,
     pixel_scales,
@@ -310,63 +321,21 @@ def _rotated_gwcs(
     refdata_extent = image_extent.max()
     pixel_scales = refdata_extent / u.Quantity(refdata_shape, u.pix)
 
-    # multiplying by +/-1 can flip north/south or east/west:
-    flip_direction = (
-        models.Multiply(cdelt_signs[0]) &
-        models.Multiply(cdelt_signs[1])
+    wcs_keywords = dict(
+        wcsaxes=2,
+        ctype1='RA---TAN',
+        ctype2='DEC--TAN',
+        crpix1=refdata_shape[0] / 2 + 1,
+        crpix2=refdata_shape[1] / 2 + 1,
+        crval1=center_world_coord.ra.deg,
+        crval2=center_world_coord.dec.deg,
+        cdelt1=cdelt_signs[0] * pixel_scales[0].to_value(u.deg / u.pix),
+        cdelt2=cdelt_signs[1] * pixel_scales[1].to_value(u.deg / u.pix),
+        lonpole=180 - rotation_angle.to_value(u.deg),
+        cname1='lon',
+        cname2='lat',
     )
-
-    # shift to compensate for the difference between the center and corner:
-    shift = (
-        models.Shift((0.5 - refdata_shape[0])/2 * u.pix) &
-        models.Shift((0.5 - refdata_shape[1])/2 * u.pix)
-    )
-
-    # rotate field of view:
-    rho = rotation_angle
-    sin_rho = np.sin(rho.to_value(u.rad))
-    cos_rho = np.cos(rho.to_value(u.rad))
-    rotation_matrix = np.array([[cos_rho, -sin_rho],
-                                [sin_rho, cos_rho]])
-    rotation = models.AffineTransformation2D(
-        rotation_matrix * u.deg, translation=[0, 0] * u.deg
-    )
-    rotation.input_units_equivalencies = {
-        "x": u.pixel_scale(pixel_scales[0]),
-        "y": u.pixel_scale(pixel_scales[1])
-    }
-    rotation.inverse = models.AffineTransformation2D(
-        np.linalg.inv(rotation_matrix) * u.pix, translation=[0, 0] * u.pix
-    )
-    rotation.inverse.input_units_equivalencies = {
-        "x": u.pixel_scale(1 / pixel_scales[0]),
-        "y": u.pixel_scale(1 / pixel_scales[1])
-    }
-
-    tan = models.Pix2Sky_TAN()
-    celestial_rotation = models.RotateNative2Celestial(
-        center_world_coord.ra, center_world_coord.dec, 180 * u.deg
-    )
-
-    det2sky = shift | flip_direction | rotation | tan | celestial_rotation
-    det2sky.name = "linear_transform"
-
-    detector_frame = cf.Frame2D(
-        name="detector",
-        axes_names=("x", "y"),
-        unit=(u.pix, u.pix)
-    )
-    sky_frame = cf.CelestialFrame(
-        reference_frame=coord.ICRS(),
-        name='icrs',
-        unit=(u.deg, u.deg)
-    )
-    pipeline = [
-        (detector_frame, det2sky),
-        (sky_frame, None)
-    ]
-
-    return GWCS(pipeline)
+    return WCS(wcs_keywords)
 
 
 def _prepare_rotated_nddata(real_image_shape, wcs, rotation_angle, refdata_shape,
@@ -406,6 +375,7 @@ def _prepare_rotated_nddata(real_image_shape, wcs, rotation_angle, refdata_shape
             data.meta.get('wcsinfo', None) or
             data.meta.get('wcs', None)
         )
+        pixel_scales = None
         if wcsinfo is not None and not isinstance(wcsinfo, GWCS):
             crval1 = float(wcsinfo.get('CRVAL1', wcsinfo.get('crval1')))
             crval2 = float(wcsinfo.get('CRVAL2', wcsinfo.get('crval2')))
@@ -420,7 +390,7 @@ def _prepare_rotated_nddata(real_image_shape, wcs, rotation_angle, refdata_shape
                 if 'wcs' not in data.meta else data.meta['wcs'],
                 fiducial, None, 1
             )]) * u.deg / u.pix
-        else:
+        if pixel_scales is None or any(np.isnan(pixel_scales)):
             # fall back on simple approximation:
             compare_pixel_coords = [[0, 0], [0, 1]] * u.pix
             compare_sky_coords = data.coords.pixel_to_world(*compare_pixel_coords)
@@ -434,11 +404,18 @@ def _prepare_rotated_nddata(real_image_shape, wcs, rotation_angle, refdata_shape
     # get the world coordinates of the pixel origin
     center_pixel_coord = np.array(real_image_shape) / 2 * u.pix
     center_world_coord = wcs.pixel_to_world(*center_pixel_coord[::-1])
+
+    if np.isnan(rotation_angle):
+        compare_pixel_coords = [[0, 0], [1, 0]] * u.pix
+        compare_sky_coords = data.coords.pixel_to_world(*compare_pixel_coords)
+        position_angle = compare_sky_coords[0].position_angle(compare_sky_coords[1])
+        rotation_angle = position_angle
+
     rotation_angle = coord.Angle(rotation_angle).wrap_at(360 * u.deg)
 
-    # create a GWCS centered on ``filename``,
+    # create a WCS centered on ``filename``,
     # and rotated by ``rotation_angle``:
-    new_rotated_gwcs = _rotated_gwcs(
+    new_rotated_wcs = _rotated_wcs(
         center_world_coord,
         rotation_angle,
         pixel_scales,
@@ -448,13 +425,13 @@ def _prepare_rotated_nddata(real_image_shape, wcs, rotation_angle, refdata_shape
     )
 
     # create a fake NDData (we use arange so data boundaries show up in Imviz
-    # if it ever is accidentally exposed) with the rotated GWCS:
+    # if it ever is accidentally exposed) with the rotated WCS:
     placeholder_data = np.nan * np.ones(refdata_shape)
 
     ndd = NDData(
         data=placeholder_data,
-        wcs=new_rotated_gwcs,
-        meta={wcs_only_key: True, '_pixel_scales': pixel_scales}
+        wcs=new_rotated_wcs,
+        meta={wcs_only_key: True, '_pixel_scales': pixel_scales, 'plugin': 'orientation'}
     )
     return ndd
 
@@ -464,7 +441,7 @@ def _get_rotated_nddata_from_label(
     cdelt_signs=None, target_wcs_east_left=True, target_wcs_north_up=True
 ):
     """
-    Create a synthetic NDData which stores GWCS that approximate
+    Create a synthetic NDData which stores WCS that approximate
     the WCS in the coords attr of the Data object with label ``data_label``
     loaded into ``app``.
 
@@ -575,14 +552,26 @@ def compute_scale(wcs, fiducial, disp_axis, pscale_ratio=1):
     if spectral and disp_axis is None:  # pragma: no cover
         raise ValueError('If input WCS is spectral, a disp_axis must be given')
 
-    crpix = np.array(wcs.invert(*fiducial))
+    # gwcs will not interally strip units off input if the forward transform
+    # does not use quantities, so they must be removed before checking if points
+    # are in image and computing the inverse
+    if not wcs.forward_transform.uses_quantity and hasattr(fiducial, 'value'):
+        fiducial = fiducial.value
+
+    if wcs.in_image(*fiducial):
+        crpix = np.array(wcs.invert(*fiducial))
+    else:
+        # At this point we know the bounding box is probably defined, so we can
+        # use the central coordinate inside the bounding box
+        warnings.warn("WCS fiducial coordinates not inside bounding box")
+        crpix = np.mean(wcs.pixel_bounds, axis=1)
 
     delta = np.zeros_like(crpix)
     spatial_idx = np.where(np.array(wcs.output_frame.axes_type) == 'SPATIAL')[0]
     delta[spatial_idx[0]] = 1
 
     crpix_with_offsets = np.vstack((crpix, crpix + delta, crpix + np.roll(delta, 1))).T
-    crval_with_offsets = wcs(*crpix_with_offsets, with_bounding_box=False)
+    crval_with_offsets = wcs(*crpix_with_offsets)
 
     coords = SkyCoord(
         ra=crval_with_offsets[spatial_idx[0]],
