@@ -258,7 +258,7 @@ class Mosviz(ConfigHelper, LineListMixin):
 
             if sp1_val is not None and sp1_val != sp2_val:
                 # then there was a conflict
-                msg = f"Warning: value for {attr} in row {row} in disagreement between Spectrum1D and Spectrum2D" # noqa
+                msg = f"Warning: value for {attr} in row {row} in disagreement between Spectrum and Spectrum2D" # noqa
                 msg = SnackbarMessage(msg, color='warning', sender=self)
                 self.app.hub.broadcast(msg)
 
@@ -283,12 +283,12 @@ class Mosviz(ConfigHelper, LineListMixin):
         ----------
         spectra_1d : list or str
             A list of spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
 
         spectra_2d : list or str
             A list of spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
 
         images : list of obj, str, or `None`
@@ -415,9 +415,17 @@ class Mosviz(ConfigHelper, LineListMixin):
             raise NotImplementedError("Please set valid values for the Mosviz.load_data() method")
 
         if allow_link_table:
-            self.link_table_data(None)
+            try:
+                self.link_table_data(None)
+            except KeyError:
+                self.app.hub.broadcast(SnackbarMessage(
+                    "The table data was not linked", color="warning", sender=self))
 
-        self._add_redshift_column()
+        try:
+            self._add_redshift_column()
+        except ValueError:
+            self.app.hub.broadcast(SnackbarMessage(
+                "No data found with label 'MOS Table'", color="warning", sender=self))
 
         # Any subsequently added data will automatically be linked
         # with data already loaded in the app
@@ -432,7 +440,13 @@ class Mosviz(ConfigHelper, LineListMixin):
         ).figure.axes[1].label_offset = "-50"
 
         # Load the first object into the viewers automatically
-        self.app.get_viewer(self._default_table_viewer_reference_name).figure_widget.highlighted = 0
+        try:
+            self.app.get_viewer(
+                self._default_table_viewer_reference_name
+            ).figure_widget.highlighted = 0
+        except ValueError:
+            self.app.hub.broadcast(SnackbarMessage(
+                "No data found with label 'MOS Table'", color="warning", sender=self))
 
         # Notify the user that this all loaded successfully
         self.app.hub.broadcast(SnackbarMessage(
@@ -460,12 +474,12 @@ class Mosviz(ConfigHelper, LineListMixin):
         ----------
         spectra_1d : list or str
             A list of spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
 
         spectra_2d : list or str
             A list of spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
         """
 
@@ -499,7 +513,7 @@ class Mosviz(ConfigHelper, LineListMixin):
         ----------
         data_obj : list or str
             A list of spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
         data_labels : str or list
             String representing the label for the data item loaded via
@@ -527,7 +541,7 @@ class Mosviz(ConfigHelper, LineListMixin):
         ----------
         data_obj : list or str
             A list of 2D spectra as translatable container objects (e.g.
-            ``Spectrum1D``) that can be read by glue-jupyter. Alternatively,
+            ``Spectrum``) that can be read by glue-jupyter. Alternatively,
             can be a string file path.
         data_labels : str or list
             String representing the label for the data item loaded via
@@ -921,7 +935,7 @@ class Mosviz(ConfigHelper, LineListMixin):
 
         Returns
         -------
-        `~specutils.Spectrum1D`
+        `~specutils.Spectrum`
         """
         return self._get_spectrum('1D Spectra', row, apply_slider_redshift)
 
@@ -941,7 +955,7 @@ class Mosviz(ConfigHelper, LineListMixin):
 
         Returns
         -------
-        `~specutils.Spectrum1D`
+        `~specutils.Spectrum`
         """
         return self._get_spectrum('2D Spectra', row, apply_slider_redshift)
 
@@ -956,7 +970,7 @@ class Mosviz(ConfigHelper, LineListMixin):
             Provide a label to retrieve a specific data set from data_collection.
         spectral_subset : str, optional
             Spectral subset applied to data.
-        cls : `~specutils.Spectrum1D`, `~astropy.nddata.CCDData`, optional
+        cls : `~specutils.Spectrum`, `~astropy.nddata.CCDData`, optional
             The type that data will be returned as.
 
         Returns
