@@ -5,7 +5,7 @@ from zipfile import ZipFile
 import numpy as np
 import pytest
 from astropy.nddata import CCDData
-from specutils import Spectrum1D
+from specutils import Spectrum
 
 from jdaviz.utils import PRIHDR_KEY
 
@@ -27,7 +27,7 @@ def test_load_spectrum1d(mosviz_helper, spectrum1d):
                                         ).data()
 
     assert len(data) == 1
-    assert isinstance(data[0], Spectrum1D)
+    assert isinstance(data[0], Spectrum)
 
     with pytest.raises(AttributeError):
         mosviz_helper.load_1d_spectra([1, 2, 3])
@@ -74,7 +74,7 @@ def test_load_spectrum_collection(mosviz_helper, spectrum_collection):
                                         ).data()
 
     assert len(data) == 1
-    assert isinstance(data[0], Spectrum1D)
+    assert isinstance(data[0], Spectrum)
 
 
 def test_load_list_of_spectrum1d(mosviz_helper, spectrum1d):
@@ -96,7 +96,7 @@ def test_load_list_of_spectrum1d(mosviz_helper, spectrum1d):
                                         ).data()
 
     assert len(data) == 1
-    assert isinstance(data[0], Spectrum1D)
+    assert isinstance(data[0], Spectrum)
 
 
 def test_load_mos_spectrum2d(mosviz_helper, mos_spectrum2d):
@@ -117,7 +117,7 @@ def test_load_mos_spectrum2d(mosviz_helper, mos_spectrum2d):
                                         ).data()
 
     assert len(data) == 1
-    assert data[0].shape == (1024, 15)
+    assert data[0].shape == (15, 1024)
 
 
 @pytest.mark.parametrize('label', [None, "Test Label"])
@@ -171,7 +171,7 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
     spec2d_viewer = mosviz_helper.app.get_viewer('spectrum-2d-viewer')
 
     # Coordinates info panel should not crash even when nothing is loaded.
-    label_mouseover = mosviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = mosviz_helper._coords_info
     label_mouseover._viewer_mouse_event(image_viewer, {'event': 'mousemove',
                                                        'domain': {'x': 0, 'y': 0}})
     assert label_mouseover.as_text() == ('', '', '')
@@ -221,9 +221,12 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
     assert label_mouseover.icon == ''
 
     label_mouseover._viewer_mouse_event(spec2d_viewer,
-                                        {'event': 'mousemove', 'domain': {'x': 10, 'y': 100}})
-    assert label_mouseover.as_text() == ('Pixel x=00010.0 y=00100.0 Value +8.12986e-01',
-                                         'Wave 1.10000e-05 m', '')
+                                        {'event': 'mousemove', 'domain': {'x': 100, 'y': 10}})
+
+    # Note: spectra2d Wave loaded in meters, but we respect one spectral unit, so the meters in
+    # converted to Angstrom (the spectra1d spectral unit).
+    assert label_mouseover.as_text() == ('Pixel x=00100.0 y=00010.0 Value +1.26544e-01 Jy',
+                                         'Wave 1.01000e+06 Angstrom', '')
     assert label_mouseover.icon == 'c'
 
     # need to trigger a mouseleave or mouseover to reset the traitlets
