@@ -1,15 +1,18 @@
 from contextlib import nullcontext
-from functools import cached_property
 
 from astropy import units as u
+from glue.core import Data as glue_core_data
 from glue.core.subset_group import GroupedSubset
 from glue_jupyter.bqplot.image import BqplotImageView
-from specutils import Spectrum1D
+from specutils import Spectrum
 from traitlets import List, Unicode, observe, Bool
+from specreduce import tracing
 
 from jdaviz.configs.default.plugins.viewers import JdavizProfileView
+from jdaviz.configs.specviz.plugins.viewers import Spectrum1DViewer
 from jdaviz.core.custom_units_and_equivs import _eqv_flux_to_sb_pixel, _eqv_pixar_sr
-from jdaviz.core.events import GlobalDisplayUnitChanged, AddDataMessage, SliceValueUpdatedMessage
+from jdaviz.core.events import (GlobalDisplayUnitChanged, AddDataMessage,
+                                RemoveDataMessage, SliceValueUpdatedMessage)
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin, UnitSelectPluginComponent,
                                         SelectPluginComponent, PluginUserApi)
@@ -22,14 +25,16 @@ from jdaviz.core.unit_conversion_utils import (create_equivalent_spectral_axis_u
 __all__ = ['UnitConversion']
 
 
-def _valid_glue_display_unit(unit_str, sv, axis='x'):
+def _valid_glue_display_unit(unit_str, viewer, axis='x'):
     # need to make sure the unit string is formatted according to the list of valid choices
     # that glue will accept (may not be the same as the defaults of the installed version of
     # astropy)
-    if not unit_str:
+    if not unit_str or not viewer:
         return unit_str
     unit_u = u.Unit(unit_str)
-    choices_str = getattr(sv.state.__class__, f'{axis}_display_unit').get_choices(sv.state)
+    if not hasattr(viewer.state.__class__, f'{axis}_display_unit'):
+        return unit_str
+    choices_str = getattr(viewer.state.__class__, f'{axis}_display_unit').get_choices(viewer.state)  # noqa
     choices_str = [choice for choice in choices_str if choice is not None]
     choices_u = [u.Unit(choice) for choice in choices_str]
     if unit_u not in choices_u:
@@ -45,12 +50,11 @@ def _flux_to_sb_unit(flux_unit, angle_unit):
         # str > unit > str to remove formatting inconsistencies with
         # parentheses/order of units/etc
         sb_unit = (u.Unit(flux_unit) / u.Unit(angle_unit)).to_string()
-
     return sb_unit
 
 
 @tray_registry('g-unit-conversion', label="Unit Conversion",
-               viewer_requirements='spectrum')
+               category="core", sidebar="settings", subtab=1)
 class UnitConversion(PluginTemplateMixin):
     """
     The Unit Conversion plugin handles global app-wide unit-conversion.
@@ -113,9 +117,8 @@ class UnitConversion(PluginTemplateMixin):
 
         self._cached_properties = ['image_layers']
 
-        if self.config not in ['specviz', 'cubeviz']:
-            # TODO [specviz2d, mosviz] x_display_unit is not implemented in glue for image viewer
-            # used by spectrum-2d-viewer
+        if self.config not in ['specviz', 'specviz2d', 'cubeviz', 'deconfigged']:
+            # TODO [mosviz] x_display_unit is not implemented in glue for image viewer
             # TODO [mosviz]: add to yaml file
             # TODO [cubeviz, slice]: slice indicator broken after changing spectral_unit
             # TODO: support for multiple viewers and handling of mixed state from glue (or does
@@ -124,16 +127,19 @@ class UnitConversion(PluginTemplateMixin):
 
         self.session.hub.subscribe(self, AddDataMessage,
                                    handler=self._on_add_data_to_viewer)
+        self.session.hub.subscribe(self, RemoveDataMessage,
+                                   handler=self._on_remove_data_from_viewer)
         self.session.hub.subscribe(self, SliceValueUpdatedMessage,
                                    handler=self._on_slice_changed)
 
-        self.has_spectral = self.config in ('specviz', 'cubeviz', 'specviz2d', 'mosviz')
+        self.has_spectral = self.config in ('specviz', 'cubeviz', 'specviz2d',
+                                            'mosviz', 'deconfigged')
         self.spectral_unit = UnitSelectPluginComponent(self,
                                                        items='spectral_unit_items',
                                                        selected='spectral_unit_selected')
         self.spectral_unit.choices = create_equivalent_spectral_axis_units_list(u.Hz)
 
-        self.has_flux = self.config in ('specviz', 'cubeviz', 'specviz2d', 'mosviz')
+        self.has_flux = self.config in ('specviz', 'cubeviz', 'specviz2d', 'mosviz', 'deconfigged')
         self.flux_unit = UnitSelectPluginComponent(self,
                                                    items='flux_unit_items',
                                                    selected='flux_unit_selected')
@@ -141,7 +147,8 @@ class UnitConversion(PluginTemplateMixin):
         # initialize flux choices to empty list, will be populated when data is loaded
         self.flux_unit.choices = []
 
-        self.has_angle = self.config in ('cubeviz', 'specviz', 'mosviz')
+        self.has_angle = self.config in ('cubeviz', 'specviz', 'mosviz',
+                                         'specviz2d', 'deconfigged')
         self.angle_unit = UnitSelectPluginComponent(self,
                                                     items='angle_unit_items',
                                                     selected='angle_unit_selected')
@@ -161,6 +168,14 @@ class UnitConversion(PluginTemplateMixin):
                                                      items='spectral_y_type_items',
                                                      selected='spectral_y_type_selected')
 
+        if self.app.config == 'deconfigged':
+            self.observe_traitlets_for_relevancy(
+                traitlets_to_observe=['spectral_unit_selected',
+                                      'flux_unit_selected',
+                                      'angle_unit_selected',
+                                      'time_unit_selected'],
+                irrelevant_msg_callback=self.relevant_if_any_truthy)
+
     @property
     def user_api(self):
         expose = []
@@ -189,13 +204,24 @@ class UnitConversion(PluginTemplateMixin):
     def spectral_y_unit(self):
         return self.sb_unit_selected if self.spectral_y_type_selected == 'Surface Brightness' else self.flux_unit_selected  # noqa
 
-    @cached_property
+    @property
     def image_layers(self):
         return [layer
                 for viewer in self._app._viewer_store.values() if isinstance(viewer, BqplotImageView)  # noqa
                 for layer in viewer.layers]
 
+    def _on_remove_data_from_viewer(self, msg):
+        viewer = msg.viewer
+        if viewer.reference == 'spectrum-viewer' and not len(viewer.layers):
+            self.disabled_msg = 'Unit Conversion unavailable without data loaded in spectrum viewer' # noqa
+
+        elif viewer.reference == 'spectrum-viewer' and len(viewer.layers):
+            xunit = _valid_glue_display_unit(self.spectral_unit.selected, viewer, 'x')
+            viewer.state.x_display_unit = xunit
+            viewer.set_plot_axes()
+
     def _on_add_data_to_viewer(self, msg):
+
         # toggle warning message for cubes without PIXAR_SR defined
         if self.config == 'cubeviz':
             # NOTE: this assumes data_collection[0] is the science (flux/sb) cube
@@ -206,16 +232,36 @@ class UnitConversion(PluginTemplateMixin):
                 self.pixar_sr_exists = False
 
         viewer = msg.viewer
+        # If we were disabled due to having no data loaded, undo that
+        if viewer.reference == 'spectrum-viewer':
+            self.disabled_msg = ''
+        if isinstance(msg.data, glue_core_data):
+            data_obj = None
+
+        # sync viewer and UC plugin units when new data is loaded. this is necessary, for
+        # example, when all data is unloaded but unit selections are set in the UC plugin,
+        # and new data is loaded which should use those units
+        if len(self.spectral_unit_selected) and hasattr(viewer.state, 'x_display_unit'):
+            if viewer.state.x_display_unit != self.spectral_unit_selected:
+                xunit = _valid_glue_display_unit(self.spectral_unit.selected, viewer, 'x')
+                viewer.state.x_display_unit = xunit
+                viewer.set_plot_axes()
+        if len(self.spectral_y_unit) and hasattr(viewer.state, 'y_display_unit'):
+            if viewer.state.y_display_unit != self.spectral_y_unit:
+                self._handle_spectral_y_unit()
 
         if (not len(self.spectral_unit_selected)
                 or not len(self.flux_unit_selected)
                 or not len(self.angle_unit_selected)
-                or (self.config == 'cubeviz' and not len(self.spectral_y_type_selected))):
+                or (self.config in ('cubeviz', 'deconfigged')
+                    and not len(self.spectral_y_type_selected))):
 
-            data_obj = msg.data.get_object()
-            if isinstance(data_obj, Spectrum1D):
+            data_obj = self.app._jdaviz_helper.get_data(msg.data.label)
 
-                self.spectral_unit._addl_unit_strings = self.spectrum_viewer.state.__class__.x_display_unit.get_choices(self.spectrum_viewer.state)  # noqa
+            # if the viewer is spectral and the data is Spectrum, get flux/sb/spectral
+            # axis units from the Spectrum object
+            if isinstance(data_obj, Spectrum) and isinstance(viewer, Spectrum1DViewer):
+                self.spectral_unit._addl_unit_strings = viewer.state.__class__.x_display_unit.get_choices(viewer.state)  # noqa
                 if not len(self.spectral_unit_selected):
                     try:
                         self.spectral_unit.selected = str(data_obj.spectral_axis.unit)
@@ -237,9 +283,13 @@ class UnitConversion(PluginTemplateMixin):
 
                     try:
                         if angle_unit is None:
-                            # default to sr if input spectrum is not in surface brightness units
-                            # TODO: for cubeviz, should we check the cube itself?
-                            self.angle_unit.selected = 'sr'
+                            if self.config in ['specviz', 'specviz2d']:
+                                self.has_angle = False
+                                self.has_sb = False
+                            else:
+                                # default to pix2 if input data is not in surface brightness units
+                                # TODO: for cubeviz, should we check the cube itself?
+                                self.angle_unit.selected = 'pix2'
                         else:
                             self.angle_unit.selected = str(angle_unit)
                     except ValueError:
@@ -278,7 +328,49 @@ class UnitConversion(PluginTemplateMixin):
             # set the attribute display unit (contour and stretch units) for the new layer
             # NOTE: this assumes that all image data is coerced to surface brightness units
             layers = [lyr for lyr in msg.viewer.layers if lyr.layer.data.label == msg.data.label]
-            self._handle_attribute_display_unit(self.sb_unit_selected, layers=layers)
+
+            if not isinstance(data_obj, tracing.Trace):
+
+                if not len(self.spectral_unit_selected) and hasattr(data_obj, 'spectral_axis'):
+                    try:
+                        self.spectral_unit.selected = str(data_obj.spectral_axis.unit)
+                    except ValueError:
+                        self.spectral_unit.selected = ''
+
+                if not self.flux_unit_selected:
+                    flux_unit = data_obj.flux.unit if hasattr(data_obj, 'flux') else data_obj.unit
+                    # get flux/sb unit from data object, and solid angle to turn sb into flux
+                    angle_unit = check_if_unit_is_per_solid_angle(flux_unit,
+                                                                  return_unit=True)
+                    flux_unit = flux_unit if angle_unit is None else flux_unit * angle_unit  # noqa
+
+                    self.flux_unit.choices = create_equivalent_flux_units_list(flux_unit)
+                    try:
+                        self.flux_unit.selected = str(flux_unit)
+                    except ValueError:
+                        self.flux_unit.selected = ''
+
+                if not self.angle_unit_selected:
+                    flux_unit = data_obj.flux.unit if hasattr(data_obj, 'flux') else data_obj.unit
+                    angle_unit = check_if_unit_is_per_solid_angle(flux_unit,
+                                                                  return_unit=True)
+                    self.angle_unit.choices = create_equivalent_angle_units_list(angle_unit)
+                    try:
+                        if angle_unit is None:
+                            if self.config == 'specviz2d':
+                                self.has_angle = False
+                                self.has_sb = False
+                            else:
+                                # default to pix2 if input data is not in surface brightness units
+                                # TODO: for cubeviz, should we check the cube itself?
+                                self.angle_unit.selected = 'pix2'
+                        else:
+                            self.angle_unit.selected = str(angle_unit)
+                    except ValueError:
+                        self.angle_unit.selected = ''
+
+            if self.angle_unit:
+                self._handle_attribute_display_unit(self.sb_unit_selected, layers=layers)
             self._clear_cache('image_layers')
 
     def _on_slice_changed(self, msg):
@@ -301,15 +393,20 @@ class UnitConversion(PluginTemplateMixin):
         axis = msg.get('name').split('_')[0]
 
         if axis == 'spectral':
-            xunit = _valid_glue_display_unit(self.spectral_unit.selected, self.spectrum_viewer, 'x')
-            self.spectrum_viewer.state.x_display_unit = xunit
-            self.spectrum_viewer.set_plot_axes()
+            for sv in self.spectrum_1d_viewers:
+                xunit = _valid_glue_display_unit(self.spectral_unit.selected, sv, 'x')
+                sv.state.x_display_unit = xunit
+                sv.set_plot_axes()
+            for s2dv in self.spectrum_2d_viewers:
+                xunit = _valid_glue_display_unit(self.spectral_unit.selected, s2dv, 'x')
 
         elif axis == 'flux':
             # handle spectral y-unit first since that is a more apparent change to the user
             # and feels laggy if it is done later
             if self.spectral_y_type_selected == 'Flux':
                 self._handle_spectral_y_unit()
+            for sv in self.spectrum_1d_viewers:
+                sv.set_plot_axes()
 
             if len(self.angle_unit_selected):
                 # NOTE: setting sb_unit_selected will call this method again with axis=='sb',
@@ -331,10 +428,12 @@ class UnitConversion(PluginTemplateMixin):
         elif axis == 'sb':
             # handle spectral y-unit first since that is a more apparent change to the user
             # and feels laggy if it is done later
-            if self.spectral_y_type_selected == 'Surface Brightness':
+            if self.spectral_y_type and self.spectral_y_type_selected == 'Surface Brightness':
                 self._handle_spectral_y_unit()
 
             self._handle_attribute_display_unit(self.sb_unit_selected)
+            for sv in self.spectrum_1d_viewers:
+                sv.set_plot_axes()
 
         # custom axes downstream can override _on_unit_selected if anything needs to be
         # processed before the GlobalDisplayUnitChanged message is broadcast
@@ -351,18 +450,32 @@ class UnitConversion(PluginTemplateMixin):
         the spectrum viewer with the new unit, and then emit a
         GlobalDisplayUnitChanged message to notify
         """
-        yunit = _valid_glue_display_unit(self.spectral_y_unit, self.spectrum_viewer, 'y')
-        if self.spectrum_viewer.state.y_display_unit == yunit:
-            self.spectrum_viewer.set_plot_axes()
-            return
-        try:
-            self.spectrum_viewer.state.y_display_unit = yunit
-        except ValueError:
-            # may not be data in the viewer, or unit may be incompatible
-            pass
+        if self.spectral_y_type_selected:
+            yunit = _valid_glue_display_unit(self.spectral_y_unit, self.spectrum_viewer, 'y')
+        elif self.sb_unit_selected:
+            yunit = _valid_glue_display_unit(self.sb_unit_selected, self.spectrum_viewer, 'y')
         else:
-            self.spectrum_viewer.set_plot_axes()
+            yunit = _valid_glue_display_unit(self.flux_unit_selected, self.spectrum_viewer, 'y')
 
+        spectral_y_change = False
+        for sv in self.spectrum_1d_viewers:
+
+            if self.spectral_unit.selected != yunit:
+                spectral_y_change = True
+
+            if sv.state.y_display_unit == yunit:
+                sv.set_plot_axes()
+                continue
+            try:
+                sv.state.y_display_unit = yunit
+            except ValueError:
+                # may not be data in the viewer, or unit may be incompatible
+                pass
+            else:
+                sv.set_plot_axes()
+                spectral_y_change = True
+
+        if spectral_y_change:
             # broadcast that there has been a change in the spectrum viewer y axis,
             self.hub.broadcast(GlobalDisplayUnitChanged('spectral_y',
                                                         yunit,
@@ -376,21 +489,30 @@ class UnitConversion(PluginTemplateMixin):
         if layers is None:
             layers = self.image_layers
 
+        def has_component_with_physical_type(layer, phys_type):
+            for comp in layer.layer.components:
+                if u.Unit(layer.layer.get_component(comp).units).physical_type == phys_type:
+                    return True
+            return False
+
         for layer in layers:
             # DQ layer doesn't play nicely with this attribute
+            # TODO: detect a DQ layer in a way that doesn't depend on the label
+            if not hasattr(layer.state, 'attribute_display_unit'):
+                continue
             if "DQ" in layer.layer.label or isinstance(layer.layer, GroupedSubset):
                 continue
-            elif ("flux" not in [str(c) for c in layer.layer.components]
-                    or u.Unit(layer.layer.get_component("flux").units).physical_type != 'surface brightness'):  # noqa
+            if not has_component_with_physical_type(layer, 'surface brightness'):
+                # if no component with surface brightness units, skip this layer
                 continue
-            if hasattr(layer.state, 'attribute_display_unit'):
-                if self.config == "cubeviz":
-                    ctx = u.set_enabled_equivalencies(
-                        u.spectral() + u.spectral_density(self._cube_wave) +
-                        _eqv_flux_to_sb_pixel() +
-                        _eqv_pixar_sr(layer.layer.meta.get('_pixel_scale_factor', 1)))
-                else:
-                    ctx = nullcontext()
-                with ctx:
-                    layer.state.attribute_display_unit = _valid_glue_display_unit(
-                        attr_unit, layer, 'attribute')
+
+            if self.config == "cubeviz":
+                ctx = u.set_enabled_equivalencies(
+                    u.spectral() + u.spectral_density(self._cube_wave) +
+                    _eqv_flux_to_sb_pixel() +
+                    _eqv_pixar_sr(layer.layer.meta.get('_pixel_scale_factor', 1)))
+            else:
+                ctx = nullcontext()
+            with ctx:
+                layer.state.attribute_display_unit = _valid_glue_display_unit(
+                    attr_unit, layer, 'attribute')
