@@ -5,7 +5,7 @@ import pytest
 from astropy import units as u
 from astropy.io import fits
 from astropy.tests.helper import assert_quantity_allclose
-from specutils import Spectrum1D, SpectrumList, SpectrumCollection, SpectralRegion
+from specutils import Spectrum, SpectrumList, SpectrumCollection, SpectralRegion
 from astropy.utils.data import download_file
 
 from jdaviz.app import Application
@@ -33,7 +33,7 @@ class TestSpecvizHelper:
 
         data = self.spec_app.get_data()
 
-        assert isinstance(data, Spectrum1D)
+        assert isinstance(data, Spectrum)
 
     def test_load_hdulist(self):
         # Create a fake fits file with a 1D spectrum for testing.
@@ -52,17 +52,18 @@ class TestSpecvizHelper:
         self.label = "Test 1D Spectrum"
         self.spec_app.load_data(fake_hdulist)
         data = self.spec_app.get_data(data_label=self.label)
-        # HDUList should load as Spectrum1D
-        assert isinstance(data, Spectrum1D)
+        # HDUList should load as Spectrum
+        assert isinstance(data, Spectrum)
 
     def test_load_spectrum_list_no_labels(self):
         # now load three more spectra from a SpectrumList, without labels
         self.spec_app.load_data(self.spec_list)
         assert len(self.spec_app.app.data_collection) == 4
         for i in (1, 2, 3):
-            assert "specviz_data" in self.spec_app.app.data_collection[i].label
+            assert "1D Spectrum" in self.spec_app.app.data_collection[i].label
 
     def test_load_spectrum_list_with_labels(self):
+        # NOTE: will be removed after load_data deprecation is removed
         # now load three more spectra from a SpectrumList, with labels:
         labels = ["List test 1", "List test 2", "List test 3"]
         self.spec_app.load_data(self.spec_list, data_label=labels)
@@ -75,12 +76,13 @@ class TestSpecvizHelper:
         assert len(self.spec_app.app.data_collection) == 11
 
     def test_mismatched_label_length(self):
+        # NOTE: will be removed after load_data deprecation is removed
         with pytest.raises(ValueError, match='Length'):
             labels = ["List test 1", "List test 2"]
             self.spec_app.load_data(self.spec_list, data_label=labels)
 
     def test_load_spectrum_collection(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             collection = SpectrumCollection([1]*u.AA)
             self.spec_app.load_data(collection)
 
@@ -287,7 +289,7 @@ def test_get_spectral_regions_unit_conversion(specviz_helper, spectrum1d):
     spec_viewer = specviz_helper.app.get_viewer('spectrum-viewer')
 
     # Mouseover without data should not crash.
-    label_mouseover = specviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = specviz_helper._coords_info
     label_mouseover._viewer_mouse_event(spec_viewer,
                                         {'event': 'mousemove', 'domain': {'x': 6100, 'y': 12.5}})
     assert label_mouseover.as_text() == ('', '', '')
@@ -358,12 +360,14 @@ def test_subset_default_thickness(specviz_helper, spectrum1d):
 
 
 def test_app_links(specviz_helper, spectrum1d):
+    specviz_helper.load_data(spectrum1d)
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
     assert isinstance(sv.jdaviz_app, Application)
     assert isinstance(sv.jdaviz_helper, Specviz)
 
 
 @pytest.mark.remote_data
+@pytest.mark.xfail(reason='Multiple file support not yet implemented in loaders')
 def test_load_spectrum_list_directory(tmpdir, specviz_helper):
     # niriss_parser_test_data.zip
     test_data = 'https://stsci.box.com/shared/static/7ndets0vjjsa97la2hvjvm6xvnu5b594.zip'
@@ -390,6 +394,7 @@ def test_load_spectrum_list_directory(tmpdir, specviz_helper):
 
 
 @pytest.mark.remote_data
+@pytest.mark.xfail(reason='Multiple file support not yet implemented in loaders')
 def test_load_spectrum_list_directory_concat(tmpdir, specviz_helper):
     # niriss_parser_test_data.zip
     test_data = 'https://stsci.box.com/shared/static/7ndets0vjjsa97la2hvjvm6xvnu5b594.zip'
@@ -409,29 +414,29 @@ def test_load_spectrum_list_directory_concat(tmpdir, specviz_helper):
 
 def test_load_2d_flux(specviz_helper):
     # Test loading a spectrum with a 2D flux, which should be split into separate
-    # 1D Spectrum1D objects to load in Specviz.
-    spec = Spectrum1D(spectral_axis=np.linspace(4000, 6000, 10)*u.Angstrom,
-                      flux=np.ones((4, 10))*u.Unit("1e-17 erg / (Angstrom cm2 s)"))
+    # 1D Spectrum objects to load in Specviz.
+    spec = Spectrum(spectral_axis=np.linspace(4000, 6000, 10)*u.Angstrom,
+                    flux=np.ones((4, 10))*u.Unit("1e-17 erg / (Angstrom cm2 s)"))
     specviz_helper.load_data(spec, data_label="test")
 
     assert len(specviz_helper.app.data_collection) == 4
-    assert specviz_helper.app.data_collection[0].label == "test [0]"
+    assert specviz_helper.app.data_collection[0].label == "test_0"
 
-    spec2 = Spectrum1D(spectral_axis=np.linspace(4000, 6000, 10)*u.Angstrom,
-                       flux=np.ones((2, 10))*u.Unit("1e-17 erg / (Angstrom cm2 s)"))
+    spec2 = Spectrum(spectral_axis=np.linspace(4000, 6000, 10)*u.Angstrom,
+                     flux=np.ones((2, 10))*u.Unit("1e-17 erg / (Angstrom cm2 s)"))
 
     # Make sure 2D spectra in a SpectrumList also get split properly.
     spec_list = SpectrumList([spec, spec2])
     specviz_helper.load_data(spec_list, data_label="second test")
 
     assert len(specviz_helper.app.data_collection) == 10
-    assert specviz_helper.app.data_collection[-1].label == "second test [5]"
+    assert specviz_helper.app.data_collection[-1].label == "second test_5"
 
 
 def test_plot_uncertainties(specviz_helper, spectrum1d):
     specviz_helper.load_data(spectrum1d)
 
-    specviz_viewer = specviz_helper.app.get_viewer("spectrum-viewer")
+    specviz_viewer = specviz_helper.app.get_viewer('spectrum-viewer')
 
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
 
@@ -454,6 +459,8 @@ def test_plot_uncertainties(specviz_helper, spectrum1d):
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
 
 
+# Some API might be going through deprecation, so ignore the warning.
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_plugin_user_apis(specviz_helper):
     for plugin_name, plugin_api in specviz_helper.plugins.items():
         plugin = plugin_api._obj
@@ -472,18 +479,18 @@ def test_spectra_partial_overlap(specviz_helper):
 
     wave_1 = np.linspace(6000, 7000, 10) * u.AA
     flux_1 = ([1200] * wave_1.size) * u.nJy
-    sp_1 = Spectrum1D(flux=flux_1, spectral_axis=wave_1)
+    sp_1 = Spectrum(flux=flux_1, spectral_axis=wave_1)
 
     wave_2 = wave_1 + (800 * u.AA)
     flux_2 = ([60] * wave_2.size) * u.nJy
-    sp_2 = Spectrum1D(flux=flux_2, spectral_axis=wave_2)
+    sp_2 = Spectrum(flux=flux_2, spectral_axis=wave_2)
 
     specviz_helper.load_data(sp_1, data_label='left')
     specviz_helper.load_data(sp_2, data_label='right')
 
     # Test mouseover outside of left but in range for right.
     # Should show right spectrum even when mouse is near left flux.
-    label_mouseover = specviz_helper.app.session.application._tools['g-coords-info']
+    label_mouseover = specviz_helper._coords_info
     label_mouseover._viewer_mouse_event(spec_viewer,
                                         {'event': 'mousemove', 'domain': {'x': 7022, 'y': 1000}})
     assert label_mouseover.as_text() == ('Cursor 7.02200e+03, 1.00000e+03',
@@ -495,16 +502,18 @@ def test_spectra_partial_overlap(specviz_helper):
 def test_spectra_incompatible_flux(specviz_helper):
     """https://github.com/spacetelescope/jdaviz/issues/2459"""
     wav = [1.1, 1.2, 1.3] * u.um
-    sp1 = Spectrum1D(flux=[1, 1.1, 1] * (u.MJy / u.sr), spectral_axis=wav)
-    sp2 = Spectrum1D(flux=[1, 1, 1.1] * (u.MJy), spectral_axis=wav)
+    sp1 = Spectrum(flux=[1, 1.1, 1] * (u.MJy / u.sr), spectral_axis=wav)
+    sp2 = Spectrum(flux=[1, 1, 1.1] * (u.MJy), spectral_axis=wav)
     flux3 = ([1, 1.1, 1] * u.MJy).to(u.erg / u.s / u.cm / u.cm / u.AA, u.spectral_density(wav))
-    sp3 = Spectrum1D(flux=flux3, spectral_axis=wav)
+    sp3 = Spectrum(flux=flux3, spectral_axis=wav)
 
     specviz_helper.load_data(sp2, data_label="2")  # OK
     specviz_helper.load_data(sp1, data_label="1")  # Not OK
     specviz_helper.load_data(sp3, data_label="3")  # OK
 
-    assert specviz_helper.app.data_collection.labels == ["2", "3"]
+    # all 3 load into data-collection, but only two in the viewer (with snackbar error)
+    assert len(specviz_helper.app.data_collection.labels) == 3
+    assert len(specviz_helper.viewers['spectrum-viewer']._obj.layers) == 2
 
 
 def test_delete_data_with_subsets(specviz_helper, spectrum1d, spectrum1d_nm):
@@ -522,8 +531,8 @@ def test_delete_data_with_subsets(specviz_helper, spectrum1d, spectrum1d_nm):
     assert subset1.subset_state.att.parent.label == "my_spec_AA"
     np.testing.assert_allclose((subset1.subset_state.lo, subset1.subset_state.hi), (6200, 7000))
 
-    specviz_helper.app.remove_data_from_viewer("spectrum-viewer", "my_spec_AA")
-    specviz_helper.app.vue_data_item_remove({"item_name": "my_spec_AA"})
+    specviz_helper.app.remove_data_from_viewer('spectrum-viewer', "my_spec_AA")
+    specviz_helper.app.data_item_remove("my_spec_AA")
 
     # Check that the reparenting and coordinate recalculations happened
     assert subset1.subset_state.att.parent.label == "my_spec_nm"
