@@ -1,6 +1,5 @@
 import os
 
-import gwcs
 import numpy as np
 from astropy import units as u
 from astropy.coordinates import SkyCoord
@@ -166,10 +165,10 @@ class AstrowidgetsImageViewerMixin:
     @property
     def zoom_level(self):
         """
-        The zoom level for an image viewer (not linked by WCS).
+        The zoom level for an image viewer (not aligned on the sky).
 
         .. warning::
-            When a viewer is linked by WCS, the result corresponds
+            When a viewer is aligned by WCS, the result corresponds
             to the ``zoom_level`` of the reference data.
 
         * 1 means real-pixel-size.
@@ -496,15 +495,17 @@ class AstrowidgetsImageViewerMixin:
                 raise AttributeError(f'{getattr(image, "label", None)} does not have a valid WCS')
             sky = table[skycoord_colname]
             t_glue = Data(marker_name, ra=sky.ra.deg, dec=sky.dec.deg)
-            dcomps = image.components
-            if (isinstance(image.coords, gwcs.WCS) and
-                    image.coords.output_frame.reference_frame.name != 'galactic' and
-                    'Lon' in dcomps and 'Lat' in dcomps):
+
+            if (image.find_component_id('Lat') and image.find_component_id('Lon')):
                 ra_str = 'Lon'
                 dec_str = 'Lat'
-            else:
+            elif (image.find_component_id('Right Ascension')
+                  and image.find_component_id('Declination')):
                 ra_str = 'Right Ascension'
                 dec_str = 'Declination'
+            else:
+                raise ValueError("Image must contain either ('Lat', 'Lon') or "
+                                 "('Right Ascension', 'Declination') components.")
             with jglue.data_collection.delay_link_manager_update():
                 jglue.data_collection[marker_name] = t_glue
                 jglue.add_link(t_glue, 'ra', image, ra_str)
