@@ -2,10 +2,13 @@ import astropy.units as u
 from glue.core.message import Message
 
 __all__ = ['NewViewerMessage', 'ViewerAddedMessage', 'ViewerRemovedMessage', 'LoadDataMessage',
-           'AddDataMessage', 'SnackbarMessage', 'RemoveDataMessage',
+           'AddDataMessage', 'SnackbarMessage', 'RemoveDataMessage', 'SubsetRenameMessage',
+           'ViewerVisibleLayersChangedMessage',
            'AddLineListMessage', 'RowLockMessage',
            'SliceSelectSliceMessage', 'SliceValueUpdatedMessage',
            'SliceToolStateMessage',
+           'CatalogResultsChangedMessage', 'CatalogSelectClickEventMessage',
+           'FootprintSelectClickEventMessage',
            'TableClickMessage', 'LinkUpdatedMessage', 'ExitBatchLoadMessage',
            'AstrowidgetMarkersChangedMessage', 'MarkersPluginUpdate',
            'GlobalDisplayUnitChanged', 'ChangeRefDataMessage',
@@ -161,6 +164,27 @@ class ChangeRefDataMessage(Message):
         return self._old
 
 
+class SubsetRenameMessage(Message):
+    def __init__(self, subset_group, old_label, new_label, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self._subset_group = subset_group
+        self._old_label = old_label
+        self._new_label = new_label
+
+    @property
+    def subset_group(self):
+        return self._subset_group
+
+    @property
+    def old_label(self):
+        return self._old_label
+
+    @property
+    def new_label(self):
+        return self._new_label
+
+
 class SnackbarMessage(Message):
     def __init__(self, text, color=None, timeout=5000, loading=False,
                  *args, **kwargs):
@@ -234,6 +258,22 @@ class RemoveDataFromViewerMessage(Message):
         return self._data_label
 
 
+class ViewerVisibleLayersChangedMessage(Message):
+    def __init__(self, viewer_reference, visible_layers, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self._viewer_reference = viewer_reference
+        self._visible_layers = visible_layers
+
+    @property
+    def viewer_reference(self):
+        return self._viewer_reference
+
+    @property
+    def visible_layers(self):
+        return self._visible_layers
+
+
 class AddLineListMessage(Message):
     def __init__(self, table, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -269,6 +309,41 @@ class SpectralMarksChangedMessage(Message):
         return self._marks
 
 
+class CatalogResultsChangedMessage(Message):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+
+class FootprintMarkVisibilityChangedMessage(Message):
+    """Message to notify that footprint overlays have changed."""
+
+    def __init__(self, viewer_id, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._viewer_id = viewer_id
+
+    @property
+    def viewer_id(self):
+        return self._viewer_id
+
+
+class CatalogSelectClickEventMessage(Message):
+    def __init__(self, x, y, *args, **kwargs):
+        self.x = x
+        self.y = y
+        super().__init__(*args, **kwargs)
+
+
+class FootprintSelectClickEventMessage(Message):
+    """
+    Message emitted when a user clicks on a viewer to select an overlay.
+    """
+
+    def __init__(self, data, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.x = data["domain"]["x"]
+        self.y = data["domain"]["y"]
+
+
 class RedshiftMessage(Message):
     '''Messages related to Specviz redshift slider'''
     def __init__(self, param, value, *args, **kwargs):
