@@ -6,13 +6,12 @@ application objects.
 See also https://github.com/spacetelescope/jdaviz/issues/104 for more details
 on the motivation behind this concept.
 """
-import re
 import warnings
 from contextlib import contextmanager
 from inspect import isclass
 
 import numpy as np
-from glue.core import HubListener
+from glue.core import ComponentID, HubListener
 from glue.core.edit_subset_mode import NewMode
 from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage
 from glue.core.subset import Subset, MaskSubsetState
@@ -23,12 +22,17 @@ from astropy.nddata import NDDataArray, CCDData, StdDevUncertainty
 import astropy.units as u
 from astropy.utils.decorators import deprecated
 from regions.core.core import Region
-from specutils import Spectrum1D, SpectralRegion
+from specutils import Spectrum, SpectralRegion
 
 from jdaviz.app import Application
 from jdaviz.core.events import SnackbarMessage, ExitBatchLoadMessage, SliceSelectSliceMessage
+from jdaviz.core.loaders.resolvers import find_matching_resolver
 from jdaviz.core.template_mixin import show_widget
-from jdaviz.utils import data_has_valid_wcs, flux_conversion, spectral_axis_conversion
+from jdaviz.utils import data_has_valid_wcs
+from jdaviz.core.unit_conversion_utils import (all_flux_unit_conversion_equivs,
+                                               check_if_unit_is_per_solid_angle,
+                                               flux_conversion_general,
+                                               spectral_axis_conversion)
 
 
 __all__ = ['ConfigHelper', 'ImageConfigHelper', 'CubeConfigHelper']
@@ -52,6 +56,7 @@ class ConfigHelper(HubListener):
         Verbosity of the history logger in the application.
     """
     _default_configuration = 'default'
+    _component_ids = {}
 
     def __init__(self, app=None, verbosity='warning', history_verbosity='info'):
         if app is None:
@@ -111,6 +116,78 @@ class ConfigHelper(HubListener):
             kwargs['data_label'] = data_label
         self.app.load_data(data, parser_reference=parser_reference, **kwargs)
 
+    @property
+    def _coords_info(self):
+        return self.app.session.application._tools.get('g-coords-info')
+
+    @property
+    def loaders(self):
+        """
+        Access API objects for data loaders in the import dialog.
+
+        Returns
+        -------
+        loaders : dict
+            dict of loader objects
+        """
+        if not (self.app.state.dev_loaders or self.app.config in ('deconfigged', 'specviz', 'specviz2d', 'imviz')):  # noqa
+            raise NotImplementedError("loaders is under active development and requires a dev-flag to test")  # noqa
+        loaders = {item['label']: widget_serialization['from_json'](item['widget'], None).user_api
+                   for item in self.app.state.loader_items}
+        return loaders
+
+    @property
+    def new_viewers(self):
+        """
+        Access API objects for creating new viewers.
+
+        Returns
+        -------
+        new_viewers : dict
+            dict of viewer-creator objects
+        """
+        if not self.app.config == 'deconfigged':
+            raise NotImplementedError("new_viewers is only enabled in the deconfigged app")  # noqa
+        new_viewers = {item['label']: widget_serialization['from_json'](item['widget'], None).user_api  # noqa
+                       for item in self.app.state.new_viewer_items if item['is_relevant']}
+        return new_viewers
+
+    def _load(self, inp=None, loader=None, format=None, target=None, **kwargs):
+        """
+        Load data into the app.  A single valid loader/importer must be able to be
+        matched based on the input, otherwise an error will be raised suggesting
+        what further information to provide.  For an interactive approach,
+        see ``loaders``.
+
+        Parameters
+        ----------
+        inp : string or object or None
+            Input filename, url, data object, etc.
+        loader : string, optional
+            Only consider a specific loader/resolver
+        format : string, optional
+            Only consider a specific format
+        target : string, optional
+            Only consider a specific target
+        kwargs :
+            Additional kwargs are passed on to both the loader and importer, as applicable.
+            Any kwargs that do not match valid inputs are silently ignored.
+        """
+        resolver = find_matching_resolver(self.app, inp,
+                                          resolver=loader,
+                                          format=format,
+                                          target=target,
+                                          **kwargs)
+
+        # TODO: deprecate show_in_viewer?
+        show_in_viewer = kwargs.pop('show_in_viewer', True)
+        importer = resolver.importer
+
+        for k, v in kwargs.items():
+            if hasattr(importer, k) and v is not None:
+                setattr(importer, k, v)
+        return importer(show_in_viewer=show_in_viewer)
+
     @property
     def data_labels(self):
         """
@@ -136,11 +213,17 @@ class ConfigHelper(HubListener):
         plugins = {item['label']: widget_serialization['from_json'](item['widget'], None).user_api
                    for item in self.app.state.tray_items if item['is_relevant']}
 
+        msg_temp = "in the future, the formerly named \"{}\" plugin will only be available by its new name: \"{}\""  # noqa
+
+        old_new = (('Imviz Line Profiles (XY)', 'Image Profiles (XY)'),  # renamed in 4.0
+                   ('Spectral Extraction', '2D Spectral Extraction'),  # renamed in 4.3
+                   ('Spectral Extraction', '3D Spectral Extraction'))  # renamed in 4.3
+
         # handle renamed plugins during deprecation
-        if 'Image Profiles (XY)' in plugins:
-            # renamed in 4.0
-            plugins['Imviz Line Profiles (XY)'] = plugins['Image Profiles (XY)']._obj.user_api
-            plugins['Imviz Line Profiles (XY)']._deprecation_msg = 'in the future, the formerly named \"Imviz Line Profiles (XY)\" plugin will only be available by its new name: \"Image Profiles (XY)\".'  # noqa
+        for old, new in old_new:
+            if new in plugins:
+                plugins[old] = plugins[new]._obj.user_api
+                plugins[old]._deprecation_msg = msg_temp.format(old, new)
 
         return plugins
 
@@ -165,7 +248,33 @@ class ConfigHelper(HubListener):
         return {viewer._ref_or_id: viewer.user_api
                 for viewer in self.app._viewer_store.values()}
 
+    def _get_clone_viewer_reference(self, reference):
+        base_name = reference.split("[")[0]
+        name = base_name
+        ind = 0
+        while name in self.viewers.keys():
+            ind += 1
+            name = f"{base_name}[{ind}]"
+        return name
+
+    def _set_data_component(self, data, component_label, values):
+        if component_label in self._component_ids:
+            component_id = self._component_ids[component_label]
+        else:
+            existing_components = [component.label for component in data.components]
+            if component_label in existing_components:
+                component_id = data.components[existing_components.index(component_label)]
+            else:
+                component_id = ComponentID(component_label)
+                self._component_ids[component_label] = component_id
+
+        if component_id in data.components:
+            data.update_components({component_id: values})
+        else:
+            data.add_component(values, component_id)
+
     @property
+    @deprecated(since="4.2", alternative="plugins['Model Fitting'].fitted_models")
     def fitted_models(self):
         """
         Returns the fitted models.
@@ -175,9 +284,12 @@ class ConfigHelper(HubListener):
         parameters : dict
             dict of `astropy.modeling.Model` objects, or None.
         """
+        plg = self.plugins.get('Model Fitting', None)
+        if plg is None:
+            raise ValueError("Model Fitting plugins is not loaded")
+        return plg.fitted_models
 
-        return self.app.fitted_models
-
+    @deprecated(since="4.2", alternative="plugins['Model Fitting'].get_models")
     def get_models(self, models=None, model_label=None, x=None, y=None):
         """
         Loop through all models and output models of the label model_label.
@@ -203,42 +315,14 @@ class ConfigHelper(HubListener):
         selected_models : dict
             Dictionary of the selected models.
         """
-        selected_models = {}
-        # If models is not provided, use the app's fitted models
-        if not models:
-            models = self.fitted_models
-
-        # Loop through all keys in the dict models
-        for label in models:
-            # Prevent "Model 2" from being returned when model_label is "Model"
-            if model_label is not None:
-                if label.split(" (")[0] != model_label:
-                    continue
-
-            # If no label was provided, use label name without coordinates.
-            if model_label is None and " (" in label:
-                find_label = label.split(" (")[0]
-            # If coordinates are not present, just use the label.
-            elif model_label is None:
-                find_label = label
-            else:
-                find_label = model_label
-
-            # If x and y are set, return keys that match the model plus that
-            # coordinate pair. If only x or y is set, return keys that fit
-            # that value for the appropriate coordinate.
-            if x is not None and y is not None:
-                find_label = r"{} \({}, {}\)".format(find_label, x, y)
-            elif x:
-                find_label = r"{} \({}, .+\)".format(find_label, x)
-            elif y:
-                find_label = r"{} \(.+, {}\)".format(find_label, y)
-
-            if re.search(find_label, label):
-                selected_models[label] = models[label]
-
-        return selected_models
+        plg = self.plugins.get('Model Fitting', None)
+        if plg is None:
+            raise ValueError("Model Fitting plugins is not loaded")
+        return plg._obj.get_models(models=models,
+                                   model_label=model_label,
+                                   x=x, y=y)
 
+    @deprecated(since="4.2")
     def get_model_parameters(self, models=None, model_label=None, x=None, y=None):
         """
         Convert each parameter of model inside models into a coordinate that
@@ -268,111 +352,12 @@ class ConfigHelper(HubListener):
             Quantity object represents the parameter value and unit of one of
             spaxel models or the 1d models, respectively.
         """
-        if models and model_label:
-            models = self.get_models(models=models, model_label=model_label, x=x, y=y)
-        elif models is None and model_label:
-            models = self.get_models(model_label=model_label, x=x, y=y)
-        elif models is None:
-            models = self.fitted_models
-
-        data_shapes = {}
-        for label in models:
-            data_label = label.split(" (")[0]
-            if data_label not in data_shapes:
-                data_shapes[data_label] = self.app.data_collection[data_label].data.shape
-
-        param_dict = {}
-        parameters_cube = {}
-        param_x_y = {}
-        param_units = {}
-
-        for label in models:
-            # 3d models take the form of "Model (1,2)" so this if statement
-            # looks for that style and separates out the pertinent information.
-            if " (" in label:
-                label_split = label.split(" (")
-                model_name = label_split[0]
-                x = int(label_split[1].split(", ")[0])
-                y = int(label_split[1].split(", ")[1][:-1])
-
-                # x and y values are added to this dict where they will be used
-                # to convert the models of each spaxel into a single
-                # coordinate in the parameters_cube dictionary.
-                if model_name not in param_x_y:
-                    param_x_y[model_name] = {'x': [], 'y': []}
-                if x not in param_x_y[model_name]['x']:
-                    param_x_y[model_name]['x'].append(x)
-                if y not in param_x_y[model_name]['y']:
-                    param_x_y[model_name]['y'].append(y)
-
-            # 1d models will be handled by this else statement.
-            else:
-                model_name = label
-
-            if model_name not in param_dict:
-                param_dict[model_name] = list(models[label].param_names)
-
-        # This adds another dictionary as the value of
-        # parameters_cube[model_name] where the key is the parameter name
-        # and the value is either a 2d array of zeros or a single value, depending
-        # on whether the model in question is 3d or 1d, respectively.
-        for model_name in param_dict:
-            if model_name in param_x_y:
-                parameters_cube[model_name] = {x: np.zeros(shape=data_shapes[model_name][:2])
-                                               for x in param_dict[model_name]}
-            else:
-                parameters_cube[model_name] = {x: 0
-                                               for x in param_dict[model_name]}
-
-        # This loop handles the actual placement of param.values and
-        # param.units into the parameter_cubes dictionary.
-        for label in models:
-            if " (" in label:
-                label_split = label.split(" (")
-                model_name = label_split[0]
-
-                # If the get_models method is used to build a dictionary of
-                # models and a value is set for the x or y parameters, that
-                # will mean that only one x or y value is present in the
-                # models.
-                if len(param_x_y[model_name]['x']) == 1:
-                    x = 0
-                else:
-                    x = int(label_split[1].split(", ")[0])
-
-                if len(param_x_y[model_name]['y']) == 1:
-                    y = 0
-                else:
-                    y = int(label_split[1].split(", ")[1][:-1])
-
-                param_units[model_name] = {}
-
-                for name in param_dict[model_name]:
-                    param = getattr(models[label], name)
-                    parameters_cube[model_name][name][x][y] = param.value
-                    param_units[model_name][name] = param.unit
-            else:
-                model_name = label
-                param_units[model_name] = {}
-
-                # 1d models do not have anything set of param.unit, so the
-                # return_units and input_units properties need to be used
-                # instead, depending on the type of parameter `name` is.
-                for name in param_dict[model_name]:
-                    param = getattr(models[label], name)
-                    parameters_cube[model_name][name] = param.value
-                    param_units[model_name][name] = param.unit
-
-        # Convert values of parameters_cube[key][param_name] into u.Quantity
-        # objects that contain the appropriate unit set in
-        # param_units[key][param_name]
-        for key in parameters_cube:
-            for param_name in parameters_cube[key]:
-                parameters_cube[key][param_name] = u.Quantity(
-                    parameters_cube[key][param_name],
-                    param_units[key].get(param_name, None))
-
-        return parameters_cube
+        plg = self.plugins.get('Model Fitting', None)
+        if plg is None:
+            raise ValueError("Model Fitting plugins is not loaded")
+        return plg._obj.get_model_parameters(models=models,
+                                             model_label=model_label,
+                                             x=x, y=y)
 
     def show(self, loc="inline", title=None, height=None):  # pragma: no cover
         """Display the Jdaviz application.
@@ -433,6 +418,14 @@ class ConfigHelper(HubListener):
             self.app.layout.height = height
             self.app.state.settings['context']['notebook']['max_height'] = height
 
+        if self.app.config in ('specviz', 'specviz2d', 'lcviz') or self.app.state.dev_loaders:
+            if not len(self.viewers) and not len(self.app.state.drawer_content):
+                self.app.state.drawer_content = 'loaders'
+            else:
+                self.app.state.drawer_content = 'plugins'
+        else:
+            self.app.state.drawer_content = 'plugins'
+
         show_widget(self.app, loc=loc, title=title)
 
     def show_in_sidecar(self, anchor=None, title=None):  # pragma: no cover
@@ -470,11 +463,29 @@ class ConfigHelper(HubListener):
 
     def _handle_display_units(self, data, use_display_units=True):
         if use_display_units:
-            if isinstance(data, Spectrum1D):
+            if isinstance(data, Spectrum):
                 spectral_unit = self.app._get_display_unit('spectral')
                 if not spectral_unit:
                     return data
-                y_unit = self.app._get_display_unit('spectral_y')
+                if self.app.config == 'specviz' and self.app._get_display_unit('sb'):
+                    y_unit = self.app._get_display_unit('sb')
+                else:
+                    y_unit = self.app._get_display_unit('spectral_y')
+
+                # if there is no pixel scale factor, and the requested conversion
+                # is between flux/sb, then skip. this case is encountered when
+                # starting the app? ideally this should raise an error, and this
+                # should allow pix2/spaxel but it doesn't - keeping this
+                # condition as-is until further investigation
+                orig_sa = check_if_unit_is_per_solid_angle(u.Unit(data.flux.unit))
+                targ_sa = check_if_unit_is_per_solid_angle(u.Unit(y_unit))
+                skip_flux_conv = ('_pixel_scale_factor' not in data.meta) & (orig_sa != targ_sa)
+
+                # equivalencies for flux/sb unit conversions
+                pixar_sr = data.meta.get('_pixel_scale_factor', None)
+                eqv = all_flux_unit_conversion_equivs(pixar_sr=pixar_sr,
+                                                      cube_wave=data.spectral_axis)
+
                 # TODO: any other attributes (meta, wcs, etc)?
                 # TODO: implement uncertainty.to upstream
                 uncertainty = data.uncertainty
@@ -484,36 +495,47 @@ class ConfigHelper(HubListener):
                     if uncertainty.unit is None:
                         uncertainty.unit = data.flux.unit
                     if hasattr(uncertainty, 'represent_as'):
-                        new_uncert = uncertainty.represent_as(
-                            StdDevUncertainty
-                        )
+                        new_uncert = uncertainty.represent_as(StdDevUncertainty)
                     else:
                         # if not specified as NDUncertainty, assume stddev:
                         new_uncert = uncertainty
-                    if ('_pixel_scale_factor' in data.meta):
-                        new_uncert_converted = flux_conversion(new_uncert.quantity.value,
-                                                               new_uncert.unit, y_unit, spec=data)
-                        new_uncert = StdDevUncertainty(new_uncert_converted, unit=y_unit)
-                    else:
-                        new_uncert = StdDevUncertainty(new_uncert, unit=data.flux.unit)
 
+                    # convert uncertainty units to display units
+                    if skip_flux_conv:
+                        new_uncert = StdDevUncertainty(new_uncert, unit=data.flux.unit)
+                    else:
+                        new_uncert_conv = flux_conversion_general(new_uncert.quantity.value,
+                                                                  new_uncert.unit,
+                                                                  y_unit,
+                                                                  eqv,
+                                                                  with_unit=False)
+                        new_uncert = StdDevUncertainty(new_uncert_conv,
+                                                       unit=y_unit)
                 else:
                     new_uncert = None
-                if ('_pixel_scale_factor' in data.meta):
-                    new_y = flux_conversion(data.flux.value, data.flux.unit,
-                                            y_unit, data) * u.Unit(y_unit)
+
+                # convert flux/sb units to display units
+                if skip_flux_conv:
+                    new_y = data.flux.value * u.Unit(data.flux.unit)
                 else:
-                    new_y = flux_conversion(data.flux.value, data.flux.unit,
-                                            data.flux.unit, spec=data) * u.Unit(data.flux.unit)
+                    # multiply by unit rather than using with_unit because of
+                    # edge case for dimensionless we want here
+                    new_y = flux_conversion_general(data.flux.value,
+                                                    data.flux.unit,
+                                                    y_unit,
+                                                    eqv, with_unit=False) * u.Unit(y_unit)
+
+                # convert spectral axis to display units
                 new_spec = (spectral_axis_conversion(data.spectral_axis.value,
                                                      data.spectral_axis.unit,
                                                      spectral_unit)
                             * u.Unit(spectral_unit))
 
-                data = Spectrum1D(spectral_axis=new_spec,
-                                  flux=new_y,
-                                  uncertainty=new_uncert,
-                                  mask=data.mask)
+                data = Spectrum(spectral_axis=new_spec,
+                                flux=new_y,
+                                uncertainty=new_uncert,
+                                mask=data.mask,
+                                spectral_axis_index=data.spectral_axis_index)
             else:  # pragma: nocover
                 raise NotImplementedError(f"converting {data.__class__.__name__} to display units is not supported")  # noqa
         return data
@@ -548,17 +570,20 @@ class ConfigHelper(HubListener):
 
         if temporal_subset:
             if mask_subset is not None:
-                raise ValueError("cannot use both mask_subset and spectral_subset")
+                raise ValueError("cannot use both mask_subset and temporal_subset")
             mask_subset = temporal_subset
 
         # End validity checks and start data retrieval
         data = self.app.data_collection[data_label]
 
         if not cls:
-            if 'Trace' in data.meta:
+            if hasattr(data, '_native_data_cls'):
+                cls = data._native_data_cls
+            # TODO: once everything goes through loaders, can we remove everything below?
+            elif 'Trace' in data.meta:
                 cls = None
             elif data.ndim == 2 and self.app.config == "specviz2d":
-                cls = Spectrum1D
+                cls = Spectrum
             elif data.ndim == 2:
                 cls = CCDData
             elif data.ndim in [1, 3]:
@@ -566,16 +591,17 @@ class ConfigHelper(HubListener):
                     cls = NDDataArray
                 else:
                     # for cubeviz, specviz, mosviz, this must be a spectrum:
-                    cls = Spectrum1D
+                    cls = Spectrum
 
         object_kwargs = {}
-        if cls == Spectrum1D:
+        if cls == Spectrum:
             object_kwargs['statistic'] = None
+            if 'spectral_axis_index' in data.meta:
+                object_kwargs['spectral_axis_index'] = data.meta['spectral_axis_index']
 
         if not spatial_subset and not mask_subset:
             if 'Trace' in data.meta:
-                if cls is not None:  # pragma: no cover
-                    raise ValueError("cls not supported for Trace object")
+                # ignore cls
                 data = data.get_object()
             else:
                 data = data.get_object(cls=cls, **object_kwargs)
@@ -628,7 +654,7 @@ class ConfigHelper(HubListener):
                               f" subset {mask_subset} applied of type {cls}."
                               f" Exception: {e}")
             if spatial_subset:
-                # Return collapsed Spectrum1D object with spectral subset mask applied
+                # Return collapsed Spectrum object with spectral subset mask applied
                 data.mask = spec_subset.mask
             else:
                 data = spec_subset
@@ -643,7 +669,7 @@ class ConfigHelper(HubListener):
         ----------
         data_label : str, optional
             Provide a label to retrieve a specific data set from data_collection.
-        cls : `~specutils.Spectrum1D`, `~astropy.nddata.CCDData`, optional
+        cls : `~specutils.Spectrum`, `~astropy.nddata.CCDData`, optional
             The type that data will be returned as.
         use_display_units : bool, optional
             Whether to convert to the display units defined in the <unit-conversion> plugin.
@@ -675,6 +701,7 @@ class ImageConfigHelper(ConfigHelper):
         (e.g., "imviz-0" or "cubeviz-0")."""
         return self._default_viewer.user_api
 
+    @deprecated(since="4.2", alternative="subset_tools.import_region")
     def load_regions_from_file(self, region_file, region_format='ds9', max_num_regions=20,
                                **kwargs):
         """Load regions defined in the given file.
@@ -707,6 +734,7 @@ class ImageConfigHelper(ConfigHelper):
         raw_regs = Regions.read(region_file, format=region_format)
         return self.load_regions(raw_regs, max_num_regions=max_num_regions, **kwargs)
 
+    @deprecated(since="4.2", alternative="subset_tools.import_region")
     def load_regions(self, regions, max_num_regions=None, refdata_label=None,
                      return_bad_regions=False, **kwargs):
         """Load given region(s) into the viewer.
@@ -811,7 +839,7 @@ class ImageConfigHelper(ConfigHelper):
                                     CirclePixelRegion, EllipsePixelRegion,
                                     RectanglePixelRegion, CircleAnnulusPixelRegion))
                     and self.app._align_by == "wcs"):
-                bad_regions.append((region, 'Pixel region provided by data is linked by WCS'))
+                bad_regions.append((region, 'Pixel region provided by data is aligned by WCS'))
                 continue
 
             # photutils: Convert to regions shape first
@@ -924,11 +952,6 @@ class ImageConfigHelper(ConfigHelper):
             subset_data = lyr.layer
             subset_label = subset_data.label
 
-            # TODO: Remove this when Jdaviz support round-tripping, see
-            # https://github.com/spacetelescope/jdaviz/pull/721
-            if not subset_label.startswith('Subset'):
-                continue
-
             try:
                 if self.app.config == "imviz" and to_sky:
                     region = roi_subset_state_to_region(subset_data.subset_state, to_sky=to_sky)
@@ -947,19 +970,6 @@ class ImageConfigHelper(ConfigHelper):
 
         return regions
 
-    # See https://github.com/glue-viz/glue-jupyter/issues/253
-    def _apply_interactive_region(self, toolname, from_pix, to_pix):
-        """Mimic interactive region drawing.
-        This is for internal testing only.
-        """
-        self.app.session.edit_subset_mode._mode = NewMode
-        tool = self.default_viewer._obj.toolbar.tools[toolname]
-        tool.activate()
-        tool.interact.brushing = True
-        tool.interact.selected = [from_pix, to_pix]
-        tool.interact.brushing = False
-        self.app.session.edit_subset_mode.edit_subset = None  # No overwrite next iteration
-
     # TODO: Make this public API?
     def _delete_region(self, subset_label):
         """Delete region given the Subset label."""
@@ -970,12 +980,6 @@ class ImageConfigHelper(ConfigHelper):
         subset_grp = self.app.data_collection.subset_groups[i]
         self.app.data_collection.remove_subset_group(subset_grp)
 
-    # TODO: Make this public API?
-    def _delete_all_regions(self):
-        """Delete all regions."""
-        for subset_grp in self.app.data_collection.subset_groups:  # should be a copy
-            self.app.data_collection.remove_subset_group(subset_grp)
-
 
 def _next_subset_num(label_prefix, subset_groups):
     """Assumes ``prefix i`` format.
@@ -1008,6 +1012,7 @@ class CubeConfigHelper(ImageConfigHelper):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
+    @deprecated(since="4.1", alternative="plugins['Slice'].slice")
     def select_slice(self, value):
         """
         Select the slice closest to the provided value.
