@@ -42,12 +42,12 @@ class TestConfigHelper:
         self.spec_app.load_data(self.spec2, data_label=self.label2)
 
         # Add 3 subsets to cover different parts of spec and spec2
-        self.spec_app.plugins['Subset Tools']._obj.import_region(
+        self.spec_app.plugins['Subset Tools'].import_region(
             SpectralRegion(6000*spectral_axis_unit, 6500*spectral_axis_unit))
-        self.spec_app.plugins['Subset Tools']._obj.import_region(
+        self.spec_app.plugins['Subset Tools'].import_region(
             SpectralRegion(6700*spectral_axis_unit, 7200*spectral_axis_unit),
             combination_mode='new')
-        self.spec_app.plugins['Subset Tools']._obj.import_region(
+        self.spec_app.plugins['Subset Tools'].import_region(
             SpectralRegion(8200*spectral_axis_unit, 8800*spectral_axis_unit),
             combination_mode='new')
 
