@@ -1,3 +1,4 @@
+from collections.abc import Iterable
 import itertools
 
 from astropy import units as u
@@ -10,12 +11,14 @@ from jdaviz.core.custom_units_and_equivs import (PIX2,
                                                  _spectral_and_photon_flux_density_units)
 
 __all__ = ["all_flux_unit_conversion_equivs", "check_if_unit_is_per_solid_angle",
-           "combine_flux_and_angle_units", "convert_integrated_sb_unit",
+           "coerce_unit", "combine_flux_and_angle_units",
+           "convert_integrated_sb_unit",
            "create_equivalent_angle_units_list",
            "create_equivalent_flux_units_list",
            "create_equivalent_spectral_axis_units_list",
            "flux_conversion_general", "handle_squared_flux_unit_conversions",
-           "supported_sq_angle_units", "units_to_strings"]
+           "supported_sq_angle_units", "spectral_axis_conversion",
+           "units_to_strings"]
 
 
 def all_flux_unit_conversion_equivs(pixar_sr=None, cube_wave=None):
@@ -53,6 +56,62 @@ def all_flux_unit_conversion_equivs(pixar_sr=None, cube_wave=None):
     return equivs
 
 
+def viewer_flux_conversion_equivalencies(values, spec):
+    """
+    Generate a list of flux and surface brightness unit conversion equivalencies
+    specifically for converting units in the viewer, accounting for the special
+    case of viewer limits that may need to index the spectral axis or list of
+    pixel scale factors.
+
+    This function assumes that if exactly two values are being converted, they
+    represent the y-axis limits of the viewer. In this case, the first spectral
+    axis value is used for spectral density conversions, and the min/max value
+    of pixel scale factor are used if there are more than 2 values in
+    meta['_pixel_scale_factor'].
+
+    Parameters:
+    ----------
+    values : array-like
+        The values to be converted, which may represent flux or surface brightness.
+    spec : Spectrum
+
+    Returns:
+    -------
+    equivs : list
+        A list of unit equivalencies for flux and surface brightness conversions.
+    """
+
+    # if we are converting only 2 values, assume it is a viewer y limits case.
+    is_viewer_limits = len(values) == 2
+
+    # for viewer limits case, use only the 0th spectral axis value for spectral_density
+    spectral_values = spec.spectral_axis
+    if not np.isscalar(values) and is_viewer_limits:
+        spectral_values = spectral_values[0]
+
+    # Need this for setting the y-limits but values from viewer might be downscaled
+    if len(values) != spectral_values.size:
+        spectral_values = spec.spectral_axis[0]
+
+    # Next, pixel scale factor
+    pix_fac = None
+    if '_pixel_scale_factor' in spec.meta:
+        pix_fac = spec.meta['_pixel_scale_factor']
+
+        if isinstance(pix_fac, u.Quantity):
+            pix_fac = pix_fac.value
+
+        # If 2 values are being converted (considered to be viewer y limits),
+        # use min and max scale factors.
+        if is_viewer_limits and isinstance(pix_fac, Iterable):
+            pix_fac = [min(pix_fac), max(pix_fac)]
+
+    # combine scale factor and spectral axis for u.spectral with other flux<>sb equivalencies
+    equivs = all_flux_unit_conversion_equivs(pix_fac, spectral_values)
+
+    return equivs
+
+
 def check_if_unit_is_per_solid_angle(unit, return_unit=False):
     """
     Check if a given Unit or unit string (that can be converted to
@@ -229,7 +288,7 @@ def create_equivalent_spectral_axis_units_list(spectral_axis_unit,
                                                         u.lsec]):
     """Get all possible conversions from current spectral_axis_unit."""
     if spectral_axis_unit in (u.pix, u.dimensionless_unscaled):
-        return [spectral_axis_unit]
+        return [spectral_axis_unit.to_string()]
 
     # Get unit equivalencies.
     try:
@@ -297,10 +356,15 @@ def flux_conversion_general(values, original_unit, target_unit,
 
     """
 
+    # we set surface brightness choices and selection before flux, which can
+    # cause a dimensionless translation attempt at instantiation
+    if not target_unit:
+        return values
+
     if original_unit == target_unit:
         if not with_unit:
             return values
-        return values * original_unit
+        return values * u.Unit(original_unit)
 
     if isinstance(original_unit, str):
         original_unit = u.Unit(original_unit)
@@ -339,6 +403,7 @@ def flux_conversion_general(values, original_unit, target_unit,
 
         if not with_unit:
             return converted_values.value
+
         return converted_values
 
 
@@ -384,6 +449,11 @@ def handle_squared_flux_unit_conversions(value, original_unit=None,
     return converted
 
 
+def spectral_axis_conversion(values, original_units, target_units):
+    eqv = u.spectral() + u.pixel_scale(1*u.pix)
+    return (values * u.Unit(original_units)).to_value(u.Unit(target_units), equivalencies=eqv)
+
+
 def supported_sq_angle_units(as_strings=False):
     """
     Returns a list of squared angle units supported by the app. If a new
@@ -425,8 +495,8 @@ def convert_integrated_sb_unit(u1, spectral_axis_unit, desired_freq_unit, desire
     based on the physical type of the flux unit (per-frequency or per-wavelength) and
     the provided spectral axis unit.
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
     u1 : astropy.units.Unit
         The unit of the integrated flux that needs conversion.
 
@@ -434,11 +504,11 @@ def convert_integrated_sb_unit(u1, spectral_axis_unit, desired_freq_unit, desire
         The unit of the spectral axis over which the flux was integrated (e.g., Angstrom
         for wavelength or Hz for frequency).
 
-    Returns:
-    --------
+    Returns
+    -------
     astropy.units.Unit
         The converted flux unit compatible with the given spectral axis unit. If the
-        units are already compatible, the input unit `u1` is returned unchanged.
+        units are already compatible, the input unit ``u1`` is returned unchanged.
     """
 
     uu = u1 / spectral_axis_unit
@@ -465,3 +535,27 @@ def convert_integrated_sb_unit(u1, spectral_axis_unit, desired_freq_unit, desire
         return u1  # units are compatible, return input
 
     return uu * spec_axis_conversion_scale_factor
+
+
+def coerce_unit(quantity):
+    """
+    coerce the unit on a quantity to have a single length unit (will take the first length
+    unit with a power of 1) and to strip any constants from the units.
+    """
+    # for some reason, quantity.unit.powers gives floats which then raise an error in
+    # quantity.to and we want to avoid casting to integer in case of fractional powers
+    unit = u.Unit(str(quantity.unit))
+    unit_types = [str(subunit.physical_type) for subunit in unit.bases]
+    length_inds = [ind for ind, (base, power, unit_type)
+                   in enumerate(zip(unit.bases, unit.powers, unit_types))
+                   if unit_type == 'length' and abs(power) == 1]
+    # we want to force all length units (not area) to use the same base unit so they can
+    # combine/cancel appropriately
+    coerced_bases = [unit.bases[i if i not in length_inds else length_inds[0]]
+                     for i in range(len(unit.bases))]
+    coerced_unit_string = ' * '.join([f'{base}**{power}'
+                                      for base, power in zip(coerced_bases, unit.powers)])
+    coerced_quantity = quantity.to(coerced_unit_string)
+    if getattr(quantity, 'uncertainty', None) is not None:
+        coerced_quantity.uncertainty = quantity.uncertainty.to(coerced_unit_string)
+    return coerced_quantity
