@@ -3,10 +3,12 @@ import pytest
 import numpy as np
 from astropy import units as u
 from astropy.wcs import WCS
-from specutils import Spectrum1D
-from jdaviz import Application, Specviz
+from specutils import Spectrum
+from jdaviz import Specviz
+from jdaviz.app import Application
 from jdaviz.configs.default.plugins.gaussian_smooth.gaussian_smooth import GaussianSmooth
-from jdaviz.utils import flux_conversion
+from jdaviz.core.unit_conversion_utils import (flux_conversion_general,
+                                               viewer_flux_conversion_equivalencies)
 
 
 # This applies to all viz but testing with Imviz should be enough.
@@ -20,10 +22,11 @@ def test_get_tray_item_from_name():
     plg = app.get_tray_item_from_name('g-gaussian-smooth')
     assert isinstance(plg, GaussianSmooth)
 
-    with pytest.raises(KeyError, match='not found in app'):
+    with pytest.raises(KeyError, match='imviz-compass not found'):
         app.get_tray_item_from_name('imviz-compass')
 
 
+@pytest.mark.xfail(reason="hardcoded config logic during deconfigging process")
 def test_nonstandard_specviz_viewer_name(spectrum1d):
     config = {'settings': {'configuration': 'nonstandard',
                            'data': {'parser': 'specviz-spectrum1d-parser'},
@@ -45,10 +48,10 @@ def test_nonstandard_specviz_viewer_name(spectrum1d):
               'viewer_area': [{'container': 'col',
                                'children': [{'container': 'row',
                                              'viewers': [{'name': 'H',
-                                                          'plot': 'specviz-profile-viewer',
+                                                          'plot': 'spectrum-1d-viewer',
                                                           'reference': 'h'},
                                                          {'name': 'K',
-                                                          'plot': 'specviz-profile-viewer',
+                                                          'plot': 'spectrum-1d-viewer',
                                                           'reference': 'k'}]}]}]}
 
     class Customviz(Specviz):
@@ -189,7 +192,7 @@ def test_data_associations(imviz_helper):
     assert imviz_helper.app._get_assoc_data_children('parent_data') == ['child_data']
     assert imviz_helper.app._get_assoc_data_parent('child_data') == 'parent_data'
 
-    with pytest.raises(NotImplementedError):
+    with pytest.raises(ValueError):
         # we don't (yet) allow children of children:
         imviz_helper.load_data(data_child, data_label='grandchild_data', parent='child_data')
 
@@ -207,11 +210,11 @@ def test_to_unit(cubeviz_helper):
     w = WCS(wcs_dict)
     flux = np.zeros((30, 20, 3001), dtype=np.float32)
     flux[5:15, 1:11, :] = 1
-    cube = Spectrum1D(flux=flux * (u.MJy / u.sr), wcs=w, meta=wcs_dict)
+    cube = Spectrum(flux=flux * (u.MJy / u.sr), wcs=w, meta=wcs_dict)
     cubeviz_helper.load_data(cube, data_label="test")
 
-    # this can be removed once spectra pass through spectral extraction
-    extract_plg = cubeviz_helper.plugins['Spectral Extraction']
+    # this can be removed once spectra pass through cube spectral extraction
+    extract_plg = cubeviz_helper.plugins['3D Spectral Extraction']
 
     extract_plg.aperture = extract_plg.aperture.choices[-1]
     extract_plg.aperture_method.selected = 'Exact'
@@ -227,8 +230,11 @@ def test_to_unit(cubeviz_helper):
     original_units = u.MJy / u.sr
     target_units = u.MJy
 
-    value = flux_conversion(values, original_units,
-                            target_units, data.get_object(cls=Spectrum1D))
+    spec = data.get_object(cls=Spectrum)
+    viewer_equivs = viewer_flux_conversion_equivalencies(values, spec)
+    value = flux_conversion_general(values, original_units,
+                                    target_units, viewer_equivs,
+                                    with_unit=False)
 
     # will be a uniform array since not wavelength dependent
     # so test first value in array
@@ -240,8 +246,10 @@ def test_to_unit(cubeviz_helper):
     original_units = u.MJy
     target_units = u.erg / u.cm**2 / u.s / u.AA
 
-    new_values = flux_conversion(values, original_units,
-                                 target_units, data.get_object(cls=Spectrum1D))
+    viewer_equivs = viewer_flux_conversion_equivalencies(values, spec)
+    new_values = flux_conversion_general(values, original_units,
+                                         target_units, viewer_equivs,
+                                         with_unit=False)
 
     assert np.allclose(new_values,
                        (values * original_units)
@@ -255,8 +263,10 @@ def test_to_unit(cubeviz_helper):
     original_units = u.MJy
     target_units = u.erg / u.cm**2 / u.s / u.AA
 
-    new_values = flux_conversion(values, original_units,
-                                 target_units, data.get_object(cls=Spectrum1D))
+    viewer_equivs = viewer_flux_conversion_equivalencies(values, spec)
+    new_values = flux_conversion_general(values, original_units,
+                                         target_units, viewer_equivs,
+                                         with_unit=False)
 
     # In this case we do a regular spectral density conversion, but using the
     # first value in the spectral axis for the equivalency
