@@ -1,22 +1,26 @@
+import operator
 import os
 import time
 import threading
 import warnings
 from collections import deque
-from collections.abc import Iterable
 from urllib.parse import urlparse
 
+import asdf
 import numpy as np
 from astropy.io import fits
 from astropy.utils import minversion
 from astropy.utils.data import download_file
+from astropy.wcs import WCS
 from astropy.wcs.wcsapi import BaseHighLevelWCS
-from astropy.units import Quantity
-from astropy import units as u
 from astroquery.mast import Observations, conf
+from gwcs import WCS as gwcs
+from gwcs.coordinate_frames import CompositeFrame, SpectralFrame
 from matplotlib import colors as mpl_colors
 import matplotlib.cm as cm
 from photutils.utils import make_random_cmap
+from regions import CirclePixelRegion, CircleAnnulusPixelRegion
+from specutils.utils.wcs_utils import SpectralGWCS
 
 from glue.config import settings
 from glue.config import colormaps as glue_colormaps
@@ -26,15 +30,14 @@ from glue.core.subset import SubsetState, RangeSubsetState, RoiSubsetState
 from glue_astronomy.spectral_coordinates import SpectralCoordinates
 from ipyvue import watch
 
-from jdaviz.core.custom_units_and_equivs import PIX2, _eqv_pixar_sr, _eqv_flux_to_sb_pixel
-from jdaviz.core.unit_conversion_utils import check_if_unit_is_per_solid_angle
 
 __all__ = ['SnackbarQueue', 'enable_hot_reloading', 'bqplot_clear_figure',
-           'standardize_metadata', 'ColorCycler', 'alpha_index', 'get_subset_type',
-           'download_uri_to_path', 'flux_conversion', 'spectral_axis_conversion',
-           'layer_is_2d', 'layer_is_2d_or_3d', 'layer_is_image_data', 'layer_is_wcs_only',
-           'get_wcs_only_layer_labels', 'get_top_layer_index', 'get_reference_image_data',
-           'standardize_roman_metadata', 'cmap_samples', 'glue_colormaps']
+           'standardize_metadata', 'ColorCycler', 'alpha_index',
+           'get_subset_type', 'cached_uri', 'download_uri_to_path', 'layer_is_2d',
+           'layer_is_2d_or_3d', 'layer_is_image_data', 'layer_is_wcs_only',
+           'get_wcs_only_layer_labels', 'get_top_layer_index',
+           'get_reference_image_data', 'standardize_roman_metadata',
+           'cmap_samples', 'glue_colormaps']
 
 NUMPY_LT_2_0 = not minversion("numpy", "2.0.dev")
 
@@ -48,6 +51,7 @@ class SnackbarQueue:
     Class that performs the role of VSnackbarQueue, which is not
     implemented in ipyvuetify.
     '''
+
     def __init__(self):
         self.queue = deque()
         # track whether we're showing a loading message which won't clear by timeout,
@@ -57,19 +61,19 @@ class SnackbarQueue:
         # to give time for the app to load.
         self.first = True
 
-    def put(self, state, msg, history=True, popup=True):
+    def put(self, state, logger_plg, msg, history=True, popup=True):
         if msg.color not in ['info', 'warning', 'error', 'success', None]:
             raise ValueError(f"color ({msg.color}) must be on of: info, warning, error, success")
 
-        if not msg.loading and history:
+        if not msg.loading and history and logger_plg is not None:
             now = time.localtime()
             timestamp = f'{now.tm_hour}:{now.tm_min:02d}:{now.tm_sec:02d}'
             new_history = {'time': timestamp, 'text': msg.text, 'color': msg.color}
             # for now, we'll hardcode the max length of the stored history
-            if len(state.snackbar_history) >= 50:
-                state.snackbar_history = state.snackbar_history[1:] + [new_history]
+            if len(logger_plg.history) >= 50:
+                logger_plg.history = logger_plg.history[1:] + [new_history]
             else:
-                state.snackbar_history.append(new_history)
+                logger_plg.history = logger_plg.history + [new_history]
 
         if not (popup or msg.loading):
             if self.loading:
@@ -128,7 +132,11 @@ class SnackbarQueue:
             # so they are not missed
             msg = self.queue[0]
             if msg.text == state.snackbar['text']:
-                _ = self.queue.popleft()
+                try:
+                    _ = self.queue.popleft()
+                except IndexError:
+                    # in case the queue has been cleared in the meantime
+                    pass
 
         # in case there are messages in the queue still,
         # display the next.
@@ -237,6 +245,30 @@ def alpha_index(index):
         return chr(97 + index//26 - 1) + chr(97 + index % 26)
 
 
+def _try_gwcs_to_fits_sip(gw):
+    """
+    Try to convert this GWCS to FITS SIP. Some GWCS models
+    cannot be converted to FITS SIP. In that case, a warning
+    is raised and the GWCS is used, as is.
+    """
+    if isinstance(gw, gwcs):
+        try:
+            result = WCS(gw.to_fits_sip(), relax=True)
+
+        except ValueError as err:
+            warnings.warn(
+                "The GWCS coordinates could not be simplified to "
+                "a SIP-based FITS WCS, the following error was "
+                f"raised: {err}",
+                UserWarning
+            )
+            result = gw
+    else:
+        result = gw
+
+    return result
+
+
 def data_has_valid_wcs(data, ndim=None):
     """Check if given glue Data has WCS that is compatible with APE 14."""
     status = hasattr(data, 'coords') and isinstance(data.coords, BaseHighLevelWCS)
@@ -312,8 +344,8 @@ def standardize_roman_metadata(data_model):
     d : dict
         Flattened dictionary of metadata
     """
-    import roman_datamodels.datamodels as rdm
-    if isinstance(data_model, rdm.DataModel):
+    # if the file is a Roman DataModel:
+    if hasattr(data_model, 'to_flat_dict'):
         # Roman metadata are in nested dicts that we flatten:
         flat_dict_meta = data_model.to_flat_dict()
 
@@ -323,226 +355,9 @@ def standardize_roman_metadata(data_model):
             for k, v in flat_dict_meta.items()
             if 'roman.meta' in k
         }
-
-
-def indirect_units():
-    from jdaviz.core.unit_conversion_utils import supported_sq_angle_units
-
-    units = []
-
-    for angle_unit in supported_sq_angle_units():
-        units += [
-                  u.erg / (u.s * u.cm**2 * u.Angstrom * angle_unit),
-                  u.erg / (u.s * u.cm**2 * u.Hz * angle_unit),
-                  u.ph / (u.Angstrom * u.s * u.cm**2 * angle_unit),
-                  u.ph / (u.Angstrom * u.s * angle_unit * u.cm**2),
-                  u.ph / (u.s * u.cm**2 * u.Hz * angle_unit)
-                 ]
-
-    return units
-
-
-def flux_conversion(values, original_units, target_units, spec=None, eqv=None, spectral_axis=None):
-    """
-    Convert flux or surface brightness values from original units to target units.
-
-    This function handles the conversion of flux or surface brightness values between different
-    units, taking into account changes between flux and surface brightness. It supports complex
-    conversions for Spectrum1D objects or cube image data.
-
-    Parameters
-    ----------
-    values : float array
-        Flux or surface brightness values in the original units.
-
-    original_units : str
-        The flux or surface brightness units of the spec object or cube image.
-
-    target_units : str
-        The units the flux or surface brightness will be converted to.
-
-    spec : `~specutils.Spectrum1D`, optional
-        The Spectrum1D object that will have converted flux or surface brightness units.
-
-    eqv : list of :ref:`astropy:unit_equivalencies`, optional
-        A list of Astropy equivalencies necessary for complex unit conversions/translations.
-
-    spectral_axis : `astropy.units.Quantity`, optional
-        Provide spectral axis associated with values array. Necessary for simple and complex unit
-        conversions/translations that require spectral density equivalencies.
-
-    Returns
-    -------
-    result : float array
-        Flux or surface brightness values in the target units.
-    """
-    # we set surface brightness choices and selection before flux, which can
-    # cause a dimensionless translation attempt at instantiation
-    if not target_units:
-        return values
-    # If there are only two values, this is likely the limits being converted, so then
-    # in case we need to use the spectral density equivalency, we need to provide only
-    # to spectral axis values. If there is only one value
-    indirect_needs_spec_axis = False
-    if spec:
-        if not np.isscalar(values) and len(values) == 2:
-            spectral_values = spec.spectral_axis[0]
-        else:
-            spectral_values = spec.spectral_axis
-
-        # the unit of the data collection item object, could be flux or surface brightness
-        spec_unit = str(spec.flux.unit)
-
-        # Need this for setting the y-limits but values from viewer might be downscaled
-        if len(values) == spectral_values.size:
-            eqv = u.spectral_density(spectral_values)
-        else:
-            eqv = u.spectral_density(spec.spectral_axis[0])
-    elif spectral_axis is not None:
-        if not isinstance(spectral_axis, u.Quantity):
-            raise TypeError("spectral_axis must be an instance of astropy.units.Quantity.")
-
-        if eqv:
-            indirect_needs_spec_axis = True
-            # Needed to convert Flux to Flux for complex conversion/translation of non-spectrum data
-            eqv += u.spectral_density(spectral_axis)
-        elif not np.isscalar(values) and len(values) != spectral_axis.size:
-            raise ValueError(f"Size mismatch: values size : ({values.size}) must match spectral_axis size : ({spectral_axis.size}).")  # noqa
-        else:
-            eqv = u.spectral_density(spectral_axis)
-
-    orig_units = u.Unit(original_units)
-    targ_units = u.Unit(target_units)
-
-    solid_angle_in_orig = check_if_unit_is_per_solid_angle(orig_units, return_unit=True)
-    solid_angle_in_targ = check_if_unit_is_per_solid_angle(targ_units, return_unit=True)
-
-    # Ensure a spectrum passed through Spectral Extraction plugin
-    if ((((spec and ('_pixel_scale_factor' in spec.meta))) or
-        (spectral_axis is not None and ('_pixel_scale_factor' in spectral_axis.info.meta)))
-        and
-            (((solid_angle_in_orig) and (not solid_angle_in_targ)) or
-             ((not solid_angle_in_orig) and (solid_angle_in_targ)))):
-        # Data item in data collection does not update from conversion/translation.
-        # App-wide original data units are used for conversion, original and
-        # target_units dictate the conversion to take place.
-        n_values = len(values)
-
-        # Make sure they are float (can be Quantity).
-        if spec:
-            fac = spec.meta['_pixel_scale_factor']
-        elif spectral_axis is not None:
-            # Quantity.info.meta gets casted to set
-            fac = next((item for item in spectral_axis.info.meta if isinstance(item, float)), None)
-            spec_unit = orig_units
-
-        if isinstance(fac, Quantity):
-            fac = fac.value
-
-        # Get min and max scale factors, to use with min and max of spec for y-limits.
-        if n_values == 2 and isinstance(fac, Iterable):
-            eqv_in = [min(fac), max(fac)]
-        else:
-            eqv_in = fac
-        eqv += _eqv_pixar_sr(np.array(eqv_in))
-
-        # may need equivalencies between flux and flux per square pixel
-        eqv += _eqv_flux_to_sb_pixel()
-
-        # when angle<>pixel translations are enabled
-        # eqv += _eqv_sb_per_pixel_to_per_angle(u.Jy)
-
-        # indirect units cannot be directly converted, and require
-        # additional conversions to reach the desired end unit.
-        # if spec_unit in [original_units, target_units]:
-        result = _indirect_conversion(
-                    values=values, orig_units=orig_units, targ_units=targ_units,
-                    eqv=eqv, spec_unit=spec_unit
-                )
-
-        if result and len(result) == 2:
-            values, updated_units = result
-            orig_units = updated_units
-        else:
-            values, updated_units, selected_unit_updated = result
-            if selected_unit_updated == 'targ':
-                targ_units = updated_units
-            elif selected_unit_updated == 'orig':
-                orig_units = updated_units
-
-    elif indirect_needs_spec_axis:
-        values, orig_units = _indirect_conversion(
-            values=values, orig_units=orig_units, targ_units=targ_units,
-            eqv=eqv, indirect_needs_spec_axis=indirect_needs_spec_axis
-            )
-    elif solid_angle_in_orig == solid_angle_in_targ == PIX2:
-        # in the case where we have 2 SBs per solid pixel that need
-        # u.spectral_density equivalency, they can't be directly converted
-        # for whatever reason (i.e 'Jy / pix2' and 'erg / (Angstrom s cm2 pix2)'
-        # are not convertible). In this case, multiply out the factor of pix2 for
-        # conversion (same kind of thing _indirect_conversion is
-        # doing but we already know the exact angle units.
-        orig_units *= PIX2
-        targ_units *= PIX2
-
-    return (values * orig_units).to_value(targ_units, equivalencies=eqv)
-
-
-def _indirect_conversion(values, orig_units, targ_units, eqv,
-                         spec_unit=None, indirect_needs_spec_axis=None):
-
-    # Note: is there a way we could write this to not require 'spec_unit'? It
-    # seems like it falls back on this to get a solid angle unit, but can we
-    # assume pix2 now if there are none? or use the display units?
-
-    solid_angle_in_orig = check_if_unit_is_per_solid_angle(orig_units, return_unit=True)
-    solid_angle_in_targ = check_if_unit_is_per_solid_angle(targ_units, return_unit=True)
-    if spec_unit is not None:
-        solid_angle_in_spec = check_if_unit_is_per_solid_angle(spec_unit, return_unit=True)
-    else:
-        solid_angle_in_spec = None
-
-    # indirect units cannot be directly converted, and require
-    # additional conversions to reach the desired end unit.
-    if (spec_unit and spec_unit in [orig_units, targ_units] and not solid_angle_in_spec):
-        if u.Unit(targ_units) in indirect_units():
-            temp_targ = targ_units * solid_angle_in_targ
-            values = (values * orig_units).to_value(temp_targ, equivalencies=eqv)
-            orig_units = u.Unit(temp_targ)
-            return values, orig_units, 'orig'
-        elif u.Unit(orig_units) in indirect_units():
-            temp_orig = orig_units * solid_angle_in_orig
-            values = (values * orig_units).to_value(temp_orig, equivalencies=eqv)
-            targ_units = u.Unit(temp_orig)
-            return values, targ_units, 'targ'
-
-        return values, targ_units, 'targ'
-
-    elif indirect_needs_spec_axis or (spec_unit and solid_angle_in_spec):
-        if not solid_angle_in_targ:
-            targ_units /= solid_angle_in_spec
-            solid_angle_in_targ = solid_angle_in_spec
-        if ((u.Unit(targ_units) in indirect_units()) or
-           (u.Unit(orig_units) in indirect_units())):
-            # SB -> Flux -> Flux -> SB
-            temp_orig = orig_units * solid_angle_in_orig
-            temp_targ = targ_units * solid_angle_in_targ
-
-            # Convert Surface Brightness to Flux, then Flux to Flux
-            values = (values * orig_units).to_value(temp_orig, equivalencies=eqv)
-            values = (values * temp_orig).to_value(temp_targ, equivalencies=eqv)
-
-            # Lastly a Flux to Surface Brightness translation in the return statement
-            orig_units = temp_targ
-
-            return values, orig_units
-
-        return values, orig_units
-
-
-def spectral_axis_conversion(values, original_units, target_units):
-    eqv = u.spectral() + u.pixel_scale(1*u.pix)
-    return (values * u.Unit(original_units)).to_value(u.Unit(target_units), equivalencies=eqv)
+    elif isinstance(data_model, asdf.AsdfFile):
+        # otherwise use default standardization
+        return standardize_metadata(data_model['roman']['meta'])
 
 
 class ColorCycler:
@@ -583,6 +398,75 @@ class ColorCycler:
         self.counter = -1
 
 
+def _chain_regions(regions, ops):
+    """
+    Combine multiple regions into a compound pixel/sky region based on the
+    specified operators.
+
+    If the operators are valid binary operators recognized  by both glue and
+    Regions, the function returns a compound region. Otherwise, it returns a
+    list of individual regions paired with their respective operators, or just
+    returns the region if regions only contains one region.
+
+    Parameters
+    ----------
+    regions : list
+        A list of region objects.
+    ops : list of str
+        A list of glue states that map to operator names to describe how to
+        combine regions (e.g. 'AndState').
+
+    Returns
+    -------
+    Compound region or list
+        A single compound region if valid operators are provided; otherwise,
+        a list of tuples containing individual regions and their associated
+        operators.
+    """
+
+    if len(regions) == 1:
+        return regions[0]
+
+    valid_operators = {
+        'AndState': operator.and_,
+        'OrState': operator.or_,
+        'XorState': operator.xor
+    }
+
+    operators = ops[1:]  # first subset doesn't need an operator
+
+    # if regions cant be combined into a compound region as an annulus or with
+    # and/or/xor, return list of tuples of (region, operator)
+    annulus = _combine_if_annulus(regions[0], regions[1], operators[0])
+    if annulus is None:
+        if not np.all(np.isin(operators, list(valid_operators.keys()))):
+            return list(zip(regions, [''] + operators))
+
+    r1 = annulus or regions[0]
+    for i in range(2 if annulus else 0, len(operators)):
+        r1 = valid_operators[operators[i]](r1, regions[i + 1])
+
+    return r1
+
+
+def _combine_if_annulus(region1, region2, op):
+    """
+    Determine whether applying `region2` to `region1` using the specified
+    operator results in a circular annulus. If the conditions are met,
+    return a `CircleAnnulusPixelRegion`; otherwise, return `None`.
+    """
+    if (
+        isinstance(region1, (CirclePixelRegion))
+        and isinstance(region1, (CirclePixelRegion))
+        and op == 'AndNotState'
+        and region1.center == region2.center
+        and region1.radius > region2.radius
+    ):
+        return CircleAnnulusPixelRegion(center=region1.center,
+                                        inner_radius=region2.radius,
+                                        outer_radius=region1.radius)
+
+
 def get_subset_type(subset):
     """
     Determine the subset type of a subset or layer
@@ -624,6 +508,17 @@ def get_subset_type(subset):
                 # subset type:
                 continue
 
+            # check for spectral coordinate in GWCS by looking for SpectralFrame
+            if isinstance(ss_data.coords, gwcs):
+                if isinstance(ss_data.coords, (SpectralFrame, SpectralGWCS)):
+                    return 'spectral'
+                elif isinstance(ss_data.coords, CompositeFrame):
+                    if np.any([isinstance(frame, SpectralFrame) for frame in
+                               ss_data.coords.output_frame.frames]):
+                        return 'spectral'
+                else:
+                    continue
+
             # check for a spectral coordinate in FITS WCS:
             wcs_coords = (
                 ss_data.coords.wcs.ctype if hasattr(ss_data.coords, 'wcs')
@@ -634,7 +529,7 @@ def get_subset_type(subset):
                 any(str(coord).startswith('WAVE') for coord in wcs_coords) or
 
                 # also check for a spectral coordinate from the glue_astronomy translator:
-                isinstance(ss_data.coords, SpectralCoordinates)
+                isinstance(ss_data.coords, (SpectralCoordinates, SpectralGWCS))
             )
 
             if has_spectral_coords:
@@ -688,6 +583,67 @@ class MultiMaskSubsetState(SubsetState):
         return cls(masks=masks)
 
 
+def get_cloud_fits(possible_uri, ext=None):
+    """
+    Retrieve and open a FITS file from an S3 URI using fsspec. Return the input
+    unchanged if it is not an S3 URI.
+
+    If ``possible_uri`` is an S3 URI, the specified extensions from the FITS
+    file will be opened remotely using `astropy.io.fits` with `fsspec`.
+    Anonymous access is assumed for S3. If the URI is not S3-based, the input
+    is returned as-is.
+
+    Parameters
+    ----------
+    possible_uri : str
+        A path or URI to the FITS file. If the URI uses the ``s3://`` scheme,
+        the file is accessed via fsspec and returned as an `~astropy.io.fits.HDUList`.
+        Otherwise, the string is returned unchanged.
+    ext : int, str, or list, optional
+        Extension(s) to load from the FITS file. Can be an integer index (e.g., 0),
+        a string name (e.g., "SCI"), or a list of such values. If `None`, all extensions
+        are loaded.
+
+    Returns
+    -------
+    file_obj : `~astropy.io.fits.HDUList` or str
+        If the URI is an S3 FITS file, returns an `HDUList` containing the requested
+        extensions. Otherwise, returns the original input string.
+    """
+
+    parsed_uri = urlparse(possible_uri)
+
+    # TODO: Add caching logic
+    if not parsed_uri.scheme.lower() == 's3':
+        raise ValueError("Not an S3 URI: {}".format(possible_uri))
+
+    downloaded_hdus = []
+    # this loads the requested extensions into local memory:
+    with fits.open(possible_uri, fsspec_kwargs={"anon": True}) as hdul:
+        if ext is None:
+            ext_list = list(range(len(hdul)))
+        elif not isinstance(ext, list):
+            ext_list = [ext]
+        else:
+            ext_list = ext
+        for extension in ext_list:
+            hdu_obj = hdul[extension]
+            downloaded_hdus.append(hdu_obj.copy())
+
+        file_obj = fits.HDUList(downloaded_hdus)
+        return file_obj
+
+
+def cached_uri(uri):
+    # return a filename if it exists in the working directory, otherwise return the URI
+    # this is used in CI tests where the MAST files are downloaded by a separate workflow,
+    # cached, and restored in the tox working directory to avoid downloading them again
+    fname = uri.split(':')[-1].split('/')[-1]
+    if os.path.isfile(fname):
+        return fname
+    return uri
+
+
 def download_uri_to_path(possible_uri, cache=None, local_path=os.curdir, timeout=None,
                          dryrun=False):
     """
@@ -747,6 +703,7 @@ def download_uri_to_path(possible_uri, cache=None, local_path=os.curdir, timeout
         # avoiding creating local paths in a tmp dir when in standalone:
         local_path = os.path.join(os.environ["JDAVIZ_START_DIR"], local_path)
 
+    timeout = int(timeout) if timeout is not None else timeout
     parsed_uri = urlparse(possible_uri)
 
     cache_none_msg = (
@@ -830,6 +787,11 @@ def layer_is_2d(layer):
     return isinstance(layer, BaseData) and layer.ndim == 2
 
 
+def layer_is_3d(layer):
+    # returns True for subclasses of BaseData with ndim=3:
+    return isinstance(layer, BaseData) and layer.ndim == 3
+
+
 def layer_is_2d_or_3d(layer):
     return isinstance(layer, BaseData) and layer.ndim in (2, 3)
 
@@ -876,7 +838,10 @@ def get_reference_image_data(app, viewer_id=None):
     By default, the first viewer is used.
     """
     if viewer_id is None:
-        refdata = app._jdaviz_helper.default_viewer._obj.state.reference_data
+        if len(image_viewers := app.get_viewers_of_cls('ImvizImageView')) > 0:
+            refdata = image_viewers[0].state.reference_data
+        else:
+            refdata = None
     else:
         viewer = app.get_viewer_by_id(viewer_id)
         refdata = viewer.state.reference_data
